#line 1 "ad_aarch64.cpp"
//
// Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
// Copyright (c) 2014, 2020, Red Hat, Inc. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_aarch64.hpp"
#include "adfiles/ad_aarch64.hpp"
#include "memory/allocation.inline.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/compressedOops.hpp"
#include "oops/markWord.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/c2_MacroAssembler.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "utilities/powerOfTwo.hpp"

//SourceForm

#line 1057 "src/hotspot/cpu/aarch64/aarch64.ad"


  // Derived RegMask with conditionally allocatable registers

  void PhaseOutput::pd_perform_mach_node_analysis() {
  }

  int MachNode::pd_alignment_required() const {
    return 1;
  }

  int MachNode::compute_padding(int current_offset) const {
    return 0;
  }

  RegMask _ANY_REG32_mask;
  RegMask _ANY_REG_mask;
  RegMask _PTR_REG_mask;
  RegMask _NO_SPECIAL_REG32_mask;
  RegMask _NO_SPECIAL_REG_mask;
  RegMask _NO_SPECIAL_PTR_REG_mask;

  void reg_mask_init() {
    // We derive below RegMask(s) from the ones which are auto-generated from
    // adlc register classes to make AArch64 rheapbase (r27) and rfp (r29)
    // registers conditionally reserved.

    _ANY_REG32_mask = _ALL_REG32_mask;
    _ANY_REG32_mask.Remove(OptoReg::as_OptoReg(r31_sp->as_VMReg()));

    _ANY_REG_mask = _ALL_REG_mask;

    _PTR_REG_mask = _ALL_REG_mask;

    _NO_SPECIAL_REG32_mask = _ALL_REG32_mask;
    _NO_SPECIAL_REG32_mask.SUBTRACT(_NON_ALLOCATABLE_REG32_mask);

    _NO_SPECIAL_REG_mask = _ALL_REG_mask;
    _NO_SPECIAL_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

    _NO_SPECIAL_PTR_REG_mask = _ALL_REG_mask;
    _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

    // r27 is not allocatable when compressed oops is on and heapbase is not
    // zero, compressed klass pointers doesn't use r27 after JDK-8234794
    if (UseCompressedOops && (CompressedOops::ptrs_base() != NULL || UseAOT)) {
      _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(r27->as_VMReg()));
      _NO_SPECIAL_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
      _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
    }

    // r29 is not allocatable when PreserveFramePointer is on
    if (PreserveFramePointer) {
      _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(r29->as_VMReg()));
      _NO_SPECIAL_REG_mask.SUBTRACT(_FP_REG_mask);
      _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_FP_REG_mask);
    }
  }

  // Optimizaton of volatile gets and puts
  // -------------------------------------
  //
  // AArch64 has ldar<x> and stlr<x> instructions which we can safely
  // use to implement volatile reads and writes. For a volatile read
  // we simply need
  //
  //   ldar<x>
  //
  // and for a volatile write we need
  //
  //   stlr<x>
  //
  // Alternatively, we can implement them by pairing a normal
  // load/store with a memory barrier. For a volatile read we need
  //
  //   ldr<x>
  //   dmb ishld
  //
  // for a volatile write
  //
  //   dmb ish
  //   str<x>
  //   dmb ish
  //
  // We can also use ldaxr and stlxr to implement compare and swap CAS
  // sequences. These are normally translated to an instruction
  // sequence like the following
  //
  //   dmb      ish
  // retry:
  //   ldxr<x>   rval raddr
  //   cmp       rval rold
  //   b.ne done
  //   stlxr<x>  rval, rnew, rold
  //   cbnz      rval retry
  // done:
  //   cset      r0, eq
  //   dmb ishld
  //
  // Note that the exclusive store is already using an stlxr
  // instruction. That is required to ensure visibility to other
  // threads of the exclusive write (assuming it succeeds) before that
  // of any subsequent writes.
  //
  // The following instruction sequence is an improvement on the above
  //
  // retry:
  //   ldaxr<x>  rval raddr
  //   cmp       rval rold
  //   b.ne done
  //   stlxr<x>  rval, rnew, rold
  //   cbnz      rval retry
  // done:
  //   cset      r0, eq
  //
  // We don't need the leading dmb ish since the stlxr guarantees
  // visibility of prior writes in the case that the swap is
  // successful. Crucially we don't have to worry about the case where
  // the swap is not successful since no valid program should be
  // relying on visibility of prior changes by the attempting thread
  // in the case where the CAS fails.
  //
  // Similarly, we don't need the trailing dmb ishld if we substitute
  // an ldaxr instruction since that will provide all the guarantees we
  // require regarding observation of changes made by other threads
  // before any change to the CAS address observed by the load.
  //
  // In order to generate the desired instruction sequence we need to
  // be able to identify specific 'signature' ideal graph node
  // sequences which i) occur as a translation of a volatile reads or
  // writes or CAS operations and ii) do not occur through any other
  // translation or graph transformation. We can then provide
  // alternative aldc matching rules which translate these node
  // sequences to the desired machine code sequences. Selection of the
  // alternative rules can be implemented by predicates which identify
  // the relevant node sequences.
  //
  // The ideal graph generator translates a volatile read to the node
  // sequence
  //
  //   LoadX[mo_acquire]
  //   MemBarAcquire
  //
  // As a special case when using the compressed oops optimization we
  // may also see this variant
  //
  //   LoadN[mo_acquire]
  //   DecodeN
  //   MemBarAcquire
  //
  // A volatile write is translated to the node sequence
  //
  //   MemBarRelease
  //   StoreX[mo_release] {CardMark}-optional
  //   MemBarVolatile
  //
  // n.b. the above node patterns are generated with a strict
  // 'signature' configuration of input and output dependencies (see
  // the predicates below for exact details). The card mark may be as
  // simple as a few extra nodes or, in a few GC configurations, may
  // include more complex control flow between the leading and
  // trailing memory barriers. However, whatever the card mark
  // configuration these signatures are unique to translated volatile
  // reads/stores -- they will not appear as a result of any other
  // bytecode translation or inlining nor as a consequence of
  // optimizing transforms.
  //
  // We also want to catch inlined unsafe volatile gets and puts and
  // be able to implement them using either ldar<x>/stlr<x> or some
  // combination of ldr<x>/stlr<x> and dmb instructions.
  //
  // Inlined unsafe volatiles puts manifest as a minor variant of the
  // normal volatile put node sequence containing an extra cpuorder
  // membar
  //
  //   MemBarRelease
  //   MemBarCPUOrder
  //   StoreX[mo_release] {CardMark}-optional
  //   MemBarCPUOrder
  //   MemBarVolatile
  //
  // n.b. as an aside, a cpuorder membar is not itself subject to
  // matching and translation by adlc rules.  However, the rule
  // predicates need to detect its presence in order to correctly
  // select the desired adlc rules.
  //
  // Inlined unsafe volatile gets manifest as a slightly different
  // node sequence to a normal volatile get because of the
  // introduction of some CPUOrder memory barriers to bracket the
  // Load. However, but the same basic skeleton of a LoadX feeding a
  // MemBarAcquire, possibly thorugh an optional DecodeN, is still
  // present
  //
  //   MemBarCPUOrder
  //        ||       \\
  //   MemBarCPUOrder LoadX[mo_acquire]
  //        ||            |
  //        ||       {DecodeN} optional
  //        ||       /
  //     MemBarAcquire
  //
  // In this case the acquire membar does not directly depend on the
  // load. However, we can be sure that the load is generated from an
  // inlined unsafe volatile get if we see it dependent on this unique
  // sequence of membar nodes. Similarly, given an acquire membar we
  // can know that it was added because of an inlined unsafe volatile
  // get if it is fed and feeds a cpuorder membar and if its feed
  // membar also feeds an acquiring load.
  //
  // Finally an inlined (Unsafe) CAS operation is translated to the
  // following ideal graph
  //
  //   MemBarRelease
  //   MemBarCPUOrder
  //   CompareAndSwapX {CardMark}-optional
  //   MemBarCPUOrder
  //   MemBarAcquire
  //
  // So, where we can identify these volatile read and write
  // signatures we can choose to plant either of the above two code
  // sequences. For a volatile read we can simply plant a normal
  // ldr<x> and translate the MemBarAcquire to a dmb. However, we can
  // also choose to inhibit translation of the MemBarAcquire and
  // inhibit planting of the ldr<x>, instead planting an ldar<x>.
  //
  // When we recognise a volatile store signature we can choose to
  // plant at a dmb ish as a translation for the MemBarRelease, a
  // normal str<x> and then a dmb ish for the MemBarVolatile.
  // Alternatively, we can inhibit translation of the MemBarRelease
  // and MemBarVolatile and instead plant a simple stlr<x>
  // instruction.
  //
  // when we recognise a CAS signature we can choose to plant a dmb
  // ish as a translation for the MemBarRelease, the conventional
  // macro-instruction sequence for the CompareAndSwap node (which
  // uses ldxr<x>) and then a dmb ishld for the MemBarAcquire.
  // Alternatively, we can elide generation of the dmb instructions
  // and plant the alternative CompareAndSwap macro-instruction
  // sequence (which uses ldaxr<x>).
  //
  // Of course, the above only applies when we see these signature
  // configurations. We still want to plant dmb instructions in any
  // other cases where we may see a MemBarAcquire, MemBarRelease or
  // MemBarVolatile. For example, at the end of a constructor which
  // writes final/volatile fields we will see a MemBarRelease
  // instruction and this needs a 'dmb ish' lest we risk the
  // constructed object being visible without making the
  // final/volatile field writes visible.
  //
  // n.b. the translation rules below which rely on detection of the
  // volatile signatures and insert ldar<x> or stlr<x> are failsafe.
  // If we see anything other than the signature configurations we
  // always just translate the loads and stores to ldr<x> and str<x>
  // and translate acquire, release and volatile membars to the
  // relevant dmb instructions.
  //

  // is_CAS(int opcode, bool maybe_volatile)
  //
  // return true if opcode is one of the possible CompareAndSwapX
  // values otherwise false.

  bool is_CAS(int opcode, bool maybe_volatile)
  {
    switch(opcode) {
      // We handle these
    case Op_CompareAndSwapI:
    case Op_CompareAndSwapL:
    case Op_CompareAndSwapP:
    case Op_CompareAndSwapN:
    case Op_ShenandoahCompareAndSwapP:
    case Op_ShenandoahCompareAndSwapN:
    case Op_CompareAndSwapB:
    case Op_CompareAndSwapS:
    case Op_GetAndSetI:
    case Op_GetAndSetL:
    case Op_GetAndSetP:
    case Op_GetAndSetN:
    case Op_GetAndAddI:
    case Op_GetAndAddL:
      return true;
    case Op_CompareAndExchangeI:
    case Op_CompareAndExchangeN:
    case Op_CompareAndExchangeB:
    case Op_CompareAndExchangeS:
    case Op_CompareAndExchangeL:
    case Op_CompareAndExchangeP:
    case Op_WeakCompareAndSwapB:
    case Op_WeakCompareAndSwapS:
    case Op_WeakCompareAndSwapI:
    case Op_WeakCompareAndSwapL:
    case Op_WeakCompareAndSwapP:
    case Op_WeakCompareAndSwapN:
    case Op_ShenandoahWeakCompareAndSwapP:
    case Op_ShenandoahWeakCompareAndSwapN:
    case Op_ShenandoahCompareAndExchangeP:
    case Op_ShenandoahCompareAndExchangeN:
      return maybe_volatile;
    default:
      return false;
    }
  }

  // helper to determine the maximum number of Phi nodes we may need to
  // traverse when searching from a card mark membar for the merge mem
  // feeding a trailing membar or vice versa

// predicates controlling emit of ldr<x>/ldar<x>

bool unnecessary_acquire(const Node *barrier)
{
  assert(barrier->is_MemBar(), "expecting a membar");

  MemBarNode* mb = barrier->as_MemBar();

  if (mb->trailing_load()) {
    return true;
  }

  if (mb->trailing_load_store()) {
    Node* load_store = mb->in(MemBarNode::Precedent);
    assert(load_store->is_LoadStore(), "unexpected graph shape");
    return is_CAS(load_store->Opcode(), true);
  }

  return false;
}

bool needs_acquiring_load(const Node *n)
{
  assert(n->is_Load(), "expecting a load");
  LoadNode *ld = n->as_Load();
  return ld->is_acquire();
}

bool unnecessary_release(const Node *n)
{
  assert((n->is_MemBar() &&
          n->Opcode() == Op_MemBarRelease),
         "expecting a release membar");

  MemBarNode *barrier = n->as_MemBar();
  if (!barrier->leading()) {
    return false;
  } else {
    Node* trailing = barrier->trailing_membar();
    MemBarNode* trailing_mb = trailing->as_MemBar();
    assert(trailing_mb->trailing(), "Not a trailing membar?");
    assert(trailing_mb->leading_membar() == n, "inconsistent leading/trailing membars");

    Node* mem = trailing_mb->in(MemBarNode::Precedent);
    if (mem->is_Store()) {
      assert(mem->as_Store()->is_release(), "");
      assert(trailing_mb->Opcode() == Op_MemBarVolatile, "");
      return true;
    } else {
      assert(mem->is_LoadStore(), "");
      assert(trailing_mb->Opcode() == Op_MemBarAcquire, "");
      return is_CAS(mem->Opcode(), true);
    }
  }
  return false;
}

bool unnecessary_volatile(const Node *n)
{
  // assert n->is_MemBar();
  MemBarNode *mbvol = n->as_MemBar();

  bool release = mbvol->trailing_store();
  assert(!release || (mbvol->in(MemBarNode::Precedent)->is_Store() && mbvol->in(MemBarNode::Precedent)->as_Store()->is_release()), "");
#ifdef ASSERT
  if (release) {
    Node* leading = mbvol->leading_membar();
    assert(leading->Opcode() == Op_MemBarRelease, "");
    assert(leading->as_MemBar()->leading_store(), "");
    assert(leading->as_MemBar()->trailing_membar() == mbvol, "");
  }
#endif

  return release;
}

// predicates controlling emit of str<x>/stlr<x>

bool needs_releasing_store(const Node *n)
{
  // assert n->is_Store();
  StoreNode *st = n->as_Store();
  return st->trailing_membar() != NULL;
}

// predicate controlling translation of CAS
//
// returns true if CAS needs to use an acquiring load otherwise false

bool needs_acquiring_load_exclusive(const Node *n)
{
  assert(is_CAS(n->Opcode(), true), "expecting a compare and swap");
  LoadStoreNode* ldst = n->as_LoadStore();
  if (is_CAS(n->Opcode(), false)) {
    assert(ldst->trailing_membar() != NULL, "expected trailing membar");
  } else {
    return ldst->trailing_membar() != NULL;
  }

  // so we can just return true here
  return true;
}

#define __ _masm.

// advance declarations for helper functions to convert register
// indices to register objects

// the ad file has to provide implementations of certain methods
// expected by the generic code
//
// REQUIRED FUNCTIONALITY

//=============================================================================

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.

int MachCallStaticJavaNode::ret_addr_offset()
{
  // call should be a simple bl
  int off = 4;
  return off;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 16; // movz, movk, movk, bl
}

int MachCallRuntimeNode::ret_addr_offset() {
  // for generated stubs the call will be
  //   far_call(addr)
  // for real runtime callouts it will be six instructions
  // see aarch64_enc_java_to_runtime
  //   adr(rscratch2, retaddr)
  //   lea(rscratch1, RuntimeAddress(addr)
  //   stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)))
  //   blr(rscratch1)
  CodeBlob *cb = CodeCache::find_blob(_entry_point);
  if (cb) {
    return MacroAssembler::far_branch_size();
  } else {
    return 6 * NativeInstruction::instruction_size;
  }
}

// Indicate if the safepoint node needs the polling page as an input

// the shared code plants the oop data at the start of the generated
// code for the safepoint node and that needs ot be at the load
// instruction itself. so we cannot plant a mov of the safepoint poll
// address followed by a load. setting this to true means the mov is
// scheduled as a prior instruction. that's better for scheduling
// anyway.

bool SafePointNode::needs_polling_address_input()
{
  return true;
}

//=============================================================================

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  st->print("BREAKPOINT");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  C2_MacroAssembler _masm(&cbuf);
  __ brk(0);
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    C2_MacroAssembler _masm(&cbuf);
    for (int i = 0; i < _count; i++) {
      __ nop();
    }
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count * NativeInstruction::instruction_size;
  }

//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("-- \t// MachConstantBaseNode (empty encoding)");
}
#endif

#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;

  int framesize = C->output()->frame_slots() << LogBytesPerInt;

  if (C->output()->need_stack_bang(framesize))
    st->print("# stack bang size=%d\n\t", framesize);

  if (framesize < ((1 << 9) + 2 * wordSize)) {
    st->print("sub  sp, sp, #%d\n\t", framesize);
    st->print("stp  rfp, lr, [sp, #%d]", framesize - 2 * wordSize);
    if (PreserveFramePointer) st->print("\n\tadd  rfp, sp, #%d", framesize - 2 * wordSize);
  } else {
    st->print("stp  lr, rfp, [sp, #%d]!\n\t", -(2 * wordSize));
    if (PreserveFramePointer) st->print("mov  rfp, sp\n\t");
    st->print("mov  rscratch1, #%d\n\t", framesize - 2 * wordSize);
    st->print("sub  sp, sp, rscratch1");
  }
  if (C->stub_function() == NULL && BarrierSet::barrier_set()->barrier_set_nmethod() != NULL) {
    st->print("\n\t");
    st->print("ldr  rscratch1, [guard]\n\t");
    st->print("dmb ishld\n\t");
    st->print("ldr  rscratch2, [rthread, #thread_disarmed_offset]\n\t");
    st->print("cmp  rscratch1, rscratch2\n\t");
    st->print("b.eq skip");
    st->print("\n\t");
    st->print("blr #nmethod_entry_barrier_stub\n\t");
    st->print("b skip\n\t");
    st->print("guard: int\n\t");
    st->print("\n\t");
    st->print("skip:\n\t");
  }
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  C2_MacroAssembler _masm(&cbuf);

  // n.b. frame size includes space for return pc and rfp
  const long framesize = C->output()->frame_size_in_bytes();
  assert(framesize%(2*wordSize) == 0, "must preserve 2*wordSize alignment");

  // insert a nop at the start of the prolog so we can patch in a
  // branch if we need to invalidate the method later
  __ nop();

  if (C->clinit_barrier_on_entry()) {
    assert(!C->method()->holder()->is_not_initialized(), "initialization should have been started");

    Label L_skip_barrier;

    __ mov_metadata(rscratch2, C->method()->holder()->constant_encoding());
    __ clinit_barrier(rscratch2, rscratch1, &L_skip_barrier);
    __ far_jump(RuntimeAddress(SharedRuntime::get_handle_wrong_method_stub()));
    __ bind(L_skip_barrier);
  }

  int bangsize = C->output()->bang_size_in_bytes();
  if (C->output()->need_stack_bang(bangsize) && UseStackBanging)
    __ generate_stack_overflow_check(bangsize);

  __ build_frame(framesize);

  if (C->stub_function() == NULL) {
    BarrierSetAssembler* bs = BarrierSet::barrier_set()->barrier_set_assembler();
    bs->nmethod_entry_barrier(&_masm);
  }

  if (VerifyStackAtCalls) {
    Unimplemented();
  }

  C->output()->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    ConstantTable& constant_table = C->output()->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0;
}

//=============================================================================

#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;
  int framesize = C->output()->frame_slots() << LogBytesPerInt;

  st->print("# pop frame %d\n\t",framesize);

  if (framesize == 0) {
    st->print("ldp  lr, rfp, [sp],#%d\n\t", (2 * wordSize));
  } else if (framesize < ((1 << 9) + 2 * wordSize)) {
    st->print("ldp  lr, rfp, [sp,#%d]\n\t", framesize - 2 * wordSize);
    st->print("add  sp, sp, #%d\n\t", framesize);
  } else {
    st->print("mov  rscratch1, #%d\n\t", framesize - 2 * wordSize);
    st->print("add  sp, sp, rscratch1\n\t");
    st->print("ldp  lr, rfp, [sp],#%d\n\t", (2 * wordSize));
  }

  if (do_polling() && C->is_method_compilation()) {
    st->print("# touch polling page\n\t");
    st->print("ldr rscratch1, [rthread],#polling_page_offset\n\t");
    st->print("ldr zr, [rscratch1]");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  C2_MacroAssembler _masm(&cbuf);
  int framesize = C->output()->frame_slots() << LogBytesPerInt;

  __ remove_frame(framesize);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    __ fetch_and_read_polling_page(rscratch1, relocInfo::poll_return_type);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 for polling page.
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

//=============================================================================

// Figure out which register class each belongs in: rc_int, rc_float or
// rc_stack.
enum RC { rc_bad, rc_int, rc_float, rc_stack };

static enum RC rc_class(OptoReg::Name reg) {

  if (reg == OptoReg::Bad) {
    return rc_bad;
  }

  // we have 30 int registers * 2 halves
  // (rscratch1 and rscratch2 are omitted)
  int slots_of_int_registers = RegisterImpl::max_slots_per_register * (RegisterImpl::number_of_registers - 2);

  if (reg < slots_of_int_registers) {
    return rc_int;
  }

  // we have 32 float register * 4 halves
  if (reg < slots_of_int_registers + FloatRegisterImpl::max_slots_per_register * FloatRegisterImpl::number_of_registers) {
    return rc_float;
  }

  // Between float regs & stack is the flags regs.
  assert(OptoReg::is_stack(reg), "blow up if spilling flags");

  return rc_stack;
}

uint MachSpillCopyNode::implementation(CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream *st) const {
  Compile* C = ra_->C;

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(this);
  OptoReg::Name dst_lo = ra_->get_reg_first(this);

  enum RC src_hi_rc = rc_class(src_hi);
  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_hi_rc = rc_class(dst_hi);
  enum RC dst_lo_rc = rc_class(dst_lo);

  assert(src_lo != OptoReg::Bad && dst_lo != OptoReg::Bad, "must move at least 1 register");

  if (src_hi != OptoReg::Bad) {
    assert((src_lo&1)==0 && src_lo+1==src_hi &&
           (dst_lo&1)==0 && dst_lo+1==dst_hi,
           "expected aligned-adjacent pairs");
  }

  if (src_lo == dst_lo && src_hi == dst_hi) {
    return 0;            // Self copy, no move.
  }

  bool is64 = (src_lo & 1) == 0 && src_lo + 1 == src_hi &&
              (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi;
  int src_offset = ra_->reg2offset(src_lo);
  int dst_offset = ra_->reg2offset(dst_lo);

  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    assert(ireg == Op_VecD || ireg == Op_VecX, "must be 64 bit or 128 bit vector");
    if (cbuf) {
      C2_MacroAssembler _masm(cbuf);
      assert((src_lo_rc != rc_int && dst_lo_rc != rc_int), "sanity");
      if (src_lo_rc == rc_stack && dst_lo_rc == rc_stack) {
        // stack->stack
        assert((src_offset & 7) == 0 && (dst_offset & 7) == 0, "unaligned stack offset");
        if (ireg == Op_VecD) {
          __ unspill(rscratch1, true, src_offset);
          __ spill(rscratch1, true, dst_offset);
        } else {
          __ spill_copy128(src_offset, dst_offset);
        }
      } else if (src_lo_rc == rc_float && dst_lo_rc == rc_float) {
        __ mov(as_FloatRegister(Matcher::_regEncode[dst_lo]),
               ireg == Op_VecD ? __ T8B : __ T16B,
               as_FloatRegister(Matcher::_regEncode[src_lo]));
      } else if (src_lo_rc == rc_float && dst_lo_rc == rc_stack) {
        __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                       ireg == Op_VecD ? __ D : __ Q,
                       ra_->reg2offset(dst_lo));
      } else if (src_lo_rc == rc_stack && dst_lo_rc == rc_float) {
        __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       ireg == Op_VecD ? __ D : __ Q,
                       ra_->reg2offset(src_lo));
      } else {
        ShouldNotReachHere();
      }
    }
  } else if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    switch (src_lo_rc) {
    case rc_int:
      if (dst_lo_rc == rc_int) {  // gpr --> gpr copy
        if (is64) {
            __ mov(as_Register(Matcher::_regEncode[dst_lo]),
                   as_Register(Matcher::_regEncode[src_lo]));
        } else {
            C2_MacroAssembler _masm(cbuf);
            __ movw(as_Register(Matcher::_regEncode[dst_lo]),
                    as_Register(Matcher::_regEncode[src_lo]));
        }
      } else if (dst_lo_rc == rc_float) { // gpr --> fpr copy
        if (is64) {
            __ fmovd(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_Register(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_Register(Matcher::_regEncode[src_lo]));
        }
      } else {                    // gpr --> stack spill
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ spill(as_Register(Matcher::_regEncode[src_lo]), is64, dst_offset);
      }
      break;
    case rc_float:
      if (dst_lo_rc == rc_int) {  // fpr --> gpr copy
        if (is64) {
            __ fmovd(as_Register(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_Register(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        }
      } else if (dst_lo_rc == rc_float) { // fpr --> fpr copy
          if (cbuf) {
            __ fmovd(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        }
      } else {                    // fpr --> stack spill
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                 is64 ? __ D : __ S, dst_offset);
      }
      break;
    case rc_stack:
      if (dst_lo_rc == rc_int) {  // stack --> gpr load
        __ unspill(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
      } else if (dst_lo_rc == rc_float) { // stack --> fpr load
        __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                   is64 ? __ D : __ S, src_offset);
      } else {                    // stack --> stack copy
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ unspill(rscratch1, is64, src_offset);
        __ spill(rscratch1, is64, dst_offset);
      }
      break;
    default:
      assert(false, "bad rc_class for spill");
      ShouldNotReachHere();
    }
  }

  if (st) {
    st->print("spill ");
    if (src_lo_rc == rc_stack) {
      st->print("[sp, #%d] -> ", ra_->reg2offset(src_lo));
    } else {
      st->print("%s -> ", Matcher::regName[src_lo]);
    }
    if (dst_lo_rc == rc_stack) {
      st->print("[sp, #%d]", ra_->reg2offset(dst_lo));
    } else {
      st->print("%s", Matcher::regName[dst_lo]);
    }
    if (bottom_type()->isa_vect() != NULL) {
      st->print("\t# vector spill size = %d", ideal_reg()==Op_VecD ? 64:128);
    } else {
      st->print("\t# spill size = %d", is64 ? 64:32);
    }
  }

  return 0;

}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  if (!ra_)
    st->print("N%d = SpillCopy(N%d)", _idx, in(1)->_idx);
  else
    implementation(NULL, ra_, false, st);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("add %s, rsp, #%d]\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  C2_MacroAssembler _masm(&cbuf);

  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg    = ra_->get_encode(this);

  if (Assembler::operand_valid_for_add_sub_immediate(offset)) {
    __ add(as_Register(reg), sp, offset);
  } else {
    ShouldNotReachHere();
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_).
  return 4;
}

//=============================================================================

#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  st->print_cr("# MachUEPNode");
  if (UseCompressedClassPointers) {
    st->print_cr("\tldrw rscratch1, j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (CompressedKlassPointers::shift() != 0) {
      st->print_cr("\tdecode_klass_not_null rscratch1, rscratch1");
    }
  } else {
   st->print_cr("\tldr rscratch1, j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
  }
  st->print_cr("\tcmp r0, rscratch1\t # Inline cache check");
  st->print_cr("\tbne, SharedRuntime::_ic_miss_stub");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  // This is the unverified entry point.
  C2_MacroAssembler _masm(&cbuf);

  __ cmp_klass(j_rarg0, rscratch2, rscratch1);
  Label skip;
  // TODO
  // can we avoid this skip and still use a reloc?
  __ br(Assembler::EQ, skip);
  __ far_jump(RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  __ bind(skip);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_);
}

// REQUIRED EMIT CODE

//=============================================================================

// Emit exception handler code.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf)
{
  // mov rscratch1 #exception_blob_entry_point
  // br rscratch1
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ far_jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf)
{
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

  __ adr(lr, __ pc());
  __ far_jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));

  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// REQUIRED MATCHER CODE

//=============================================================================

const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode))
    return false;

  bool ret_value = true;
  switch (opcode) {
    case Op_CacheWB:
    case Op_CacheWBPreSync:
    case Op_CacheWBPostSync:
      if (!VM_Version::supports_data_cache_line_flush()) {
        ret_value = false;
      }
      break;
  }

  return ret_value; // Per default match rules are supported.
}

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen) and element type (bt).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen, BasicType bt) {
  if (!match_rule_supported(opcode)) {
    return false;
  }

  // Special cases which require vector length
  switch (opcode) {
    case Op_MulAddVS2VI: {
      if (vlen != 4) {
        return false;
      }
      break;
    }
  }

  return true; // Per default match rules are supported.
}

const bool Matcher::has_predicated_vectors(void) {
  return false;
}

const int Matcher::float_pressure(int default_pressure_threshold) {
  return default_pressure_threshold;
}

int Matcher::regnum_to_fpu_offset(int regnum)
{
  Unimplemented();
  return 0;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.

  return (-32768 <= offset && offset < 32768);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  // Probably always true, even if a temp register is required.
  return true;
}

// true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  int size = MIN2(16,(int)MaxVectorSize);
  // Minimum 2 values in vector
  if (size < 2*type2aelembytes(bt)) size = 0;
  // But never < 4
  if (size < 4) size = 0;
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
//  For the moment limit the vector size to 8 bytes
    int size = 8 / type2aelembytes(bt);
    if (size < 2) size = 2;
    return size;
}

// Vector ideal reg.
const uint Matcher::vector_ideal_reg(int len) {
  switch(len) {
    case  8: return Op_VecD;
    case 16: return Op_VecX;
  }
  ShouldNotReachHere();
  return 0;
}

// AES support not yet implemented
const bool Matcher::pass_original_key_for_aes() {
  return false;
}

// aarch64 supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return true;
}

// false => size gets scaled to BytesPerLong, ok.
const bool Matcher::init_array_count_is_in_bytes = false;

// Use conditional move (CMOVL)
const int Matcher::long_cmove_cost() {
  // long cmoves are no more expensive than int cmoves
  return 0;
}

const int Matcher::float_cmove_cost() {
  // float cmoves are no more expensive than int cmoves
  return 0;
}

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_postalloc_expand = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

// No support for generic vector operands.
const bool Matcher::supports_generic_vector_operands  = false;

MachOper* Matcher::pd_specialize_generic_vector_operand(MachOper* original_opnd, uint ideal_reg, bool is_temp) {
  ShouldNotReachHere(); // generic vector operands not supported
  return NULL;
}

bool Matcher::is_generic_reg2reg_move(MachNode* m) {
  ShouldNotReachHere();  // generic vector operands not supported
  return false;
}

bool Matcher::is_generic_vector(MachOper* opnd)  {
  ShouldNotReachHere();  // generic vector operands not supported
  return false;
}

// This affects two different things:
//  - how Decode nodes are matched
//  - how ImplicitNullCheck opportunities are recognized
// If true, the matcher will try to remove all Decodes and match them
// (as operands) into nodes. NullChecks are not prepared to deal with
// Decodes by final_graph_reshaping().
// If false, final_graph_reshaping() forces the decode behind the Cmp
// for a NullCheck. The matcher matches the Decode node into a register.
// Implicit_null_check optimization moves the Decode along with the
// memory operation back up before the NullCheck.
bool Matcher::narrow_oop_use_complex_address() {
  return CompressedOops::shift() == 0;
}

bool Matcher::narrow_klass_use_complex_address() {
// TODO
// decide whether we need to set this to true
  return false;
}

bool Matcher::const_oop_prefer_decode() {
  // Prefer ConN+DecodeN over ConP in simple compressed oops mode.
  return CompressedOops::base() == NULL;
}

bool Matcher::const_klass_prefer_decode() {
  // Prefer ConNKlass+DecodeNKlass over ConP in simple compressed klass mode.
  return CompressedKlassPointers::base() == NULL;
}

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = false;

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
  Unimplemented();
}

// Advertise here if the CPU requires explicit rounding operations to implement strictfp mode.
const bool Matcher::strict_fp_requires_explicit_rounding = false;

// Are floats converted to double when stored to stack during
// deoptimization?
bool Matcher::float_in_double() { return false; }

// Do ints take an entire long register or just half?
// The relevant question is how the int is callee-saved:
// the whole long is written but de-opt'ing will have to extract
// the relevant 32 bits.
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  R0_num || reg == R0_H_num ||
    reg ==  R1_num || reg == R1_H_num ||
    reg ==  R2_num || reg == R2_H_num ||
    reg ==  R3_num || reg == R3_H_num ||
    reg ==  R4_num || reg == R4_H_num ||
    reg ==  R5_num || reg == R5_H_num ||
    reg ==  R6_num || reg == R6_H_num ||
    reg ==  R7_num || reg == R7_H_num ||
    reg ==  V0_num || reg == V0_H_num ||
    reg ==  V1_num || reg == V1_H_num ||
    reg ==  V2_num || reg == V2_H_num ||
    reg ==  V3_num || reg == V3_H_num ||
    reg ==  V4_num || reg == V4_H_num ||
    reg ==  V5_num || reg == V5_H_num ||
    reg ==  V6_num || reg == V6_H_num ||
    reg ==  V7_num || reg == V7_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con(jlong divisor) {
  return false;
}

RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI.
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL.
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL.
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return FP_REG_mask();
}

bool size_fits_all_mem_uses(AddPNode* addp, int shift) {
  for (DUIterator_Fast imax, i = addp->fast_outs(imax); i < imax; i++) {
    Node* u = addp->fast_out(i);
    if (u->is_Mem()) {
      int opsize = u->as_Mem()->memory_size();
      assert(opsize > 0, "unexpected memory operand size");
      if (u->as_Mem()->memory_size() != (1<<shift)) {
        return false;
      }
    }
  }
  return true;
}

const bool Matcher::convi2l_type_required = false;

// Should the matcher clone input 'm' of node 'n'?
bool Matcher::pd_clone_node(Node* n, Node* m, Matcher::MStack& mstack) {
  if (is_vshift_con_pattern(n, m)) { // ShiftV src (ShiftCntV con)
    mstack.push(m, Visit);           // m = ShiftCntV
    return true;
  }
  return false;
}

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::pd_clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  if (clone_base_plus_offset_address(m, mstack, address_visited)) {
    return true;
  }

  Node *off = m->in(AddPNode::Offset);
  if (off->Opcode() == Op_LShiftL && off->in(2)->is_Con() &&
      size_fits_all_mem_uses(m, off->in(2)->get_int()) &&
      // Are there other uses besides address expressions?
      !is_visited(off)) {
    address_visited.set(off->_idx); // Flag as address_visited
    mstack.push(off->in(2), Visit);
    Node *conv = off->in(1);
    if (conv->Opcode() == Op_ConvI2L &&
        // Are there other uses besides address expressions?
        !is_visited(conv)) {
      address_visited.set(conv->_idx); // Flag as address_visited
      mstack.push(conv->in(1), Pre_Visit);
    } else {
      mstack.push(conv, Pre_Visit);
    }
    address_visited.test_set(m->_idx); // Flag as address_visited
    mstack.push(m->in(AddPNode::Address), Pre_Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  } else if (off->Opcode() == Op_ConvI2L &&
             // Are there other uses besides address expressions?
             !is_visited(off)) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    address_visited.set(off->_idx); // Flag as address_visited
    mstack.push(off->in(1), Pre_Visit);
    mstack.push(m->in(AddPNode::Address), Pre_Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  }
  return false;
}

void Compile::reshape_address(AddPNode* addp) {
}


#define MOV_VOLATILE(REG, BASE, INDEX, SCALE, DISP, SCRATCH, INSN)      \
  C2_MacroAssembler _masm(&cbuf);                                       \
  {                                                                     \
    guarantee(INDEX == -1, "mode not permitted for volatile");          \
    guarantee(DISP == 0, "mode not permitted for volatile");            \
    guarantee(SCALE == 0, "mode not permitted for volatile");           \
    __ INSN(REG, as_Register(BASE));                                    \
  }


static Address mem2address(int opcode, Register base, int index, int size, int disp)
  {
    Address::extend scale;

    // Hooboy, this is fugly.  We need a way to communicate to the
    // encoder that the index needs to be sign extended, so we have to
    // enumerate all the cases.
    switch (opcode) {
    case INDINDEXSCALEDI2L:
    case INDINDEXSCALEDI2LN:
    case INDINDEXI2L:
    case INDINDEXI2LN:
      scale = Address::sxtw(size);
      break;
    default:
      scale = Address::lsl(size);
    }

    if (index == -1) {
      return Address(base, disp);
    } else {
      assert(disp == 0, "unsupported address mode: disp = %d", disp);
      return Address(base, as_Register(index), scale);
    }
  }


typedef void (MacroAssembler::* mem_insn)(Register Rt, const Address &adr);
typedef void (MacroAssembler::* mem_insn2)(Register Rt, Register adr);
typedef void (MacroAssembler::* mem_float_insn)(FloatRegister Rt, const Address &adr);
typedef void (MacroAssembler::* mem_vector_insn)(FloatRegister Rt,
                                  MacroAssembler::SIMD_RegVariant T, const Address &adr);

  // Used for all non-volatile memory accesses.  The use of
  // $mem->opcode() to discover whether this pattern uses sign-extended
  // offsets is something of a kludge.
  static void loadStore(C2_MacroAssembler masm, mem_insn insn,
                        Register reg, int opcode,
                        Register base, int index, int scale, int disp,
                        int size_in_memory)
  {
    Address addr = mem2address(opcode, base, index, scale, disp);
    if (addr.getMode() == Address::base_plus_offset) {
      /* If we get an out-of-range offset it is a bug in the compiler,
         so we assert here. */
      assert(Address::offset_ok_for_immed(addr.offset(), exact_log2(size_in_memory)),
             "c2 compiler bug");
      /* Fix up any out-of-range offsets. */
      assert_different_registers(rscratch1, base);
      assert_different_registers(rscratch1, reg);
      addr = masm.legitimize_address(addr, size_in_memory, rscratch1);
    }
    (masm.*insn)(reg, addr);
  }

  static void loadStore(C2_MacroAssembler masm, mem_float_insn insn,
                        FloatRegister reg, int opcode,
                        Register base, int index, int size, int disp,
                        int size_in_memory)
  {
    Address::extend scale;

    switch (opcode) {
    case INDINDEXSCALEDI2L:
    case INDINDEXSCALEDI2LN:
      scale = Address::sxtw(size);
      break;
    default:
      scale = Address::lsl(size);
    }

    if (index == -1) {
      /* If we get an out-of-range offset it is a bug in the compiler,
         so we assert here. */
      assert(Address::offset_ok_for_immed(disp, exact_log2(size_in_memory)), "c2 compiler bug");
      /* Fix up any out-of-range offsets. */
      assert_different_registers(rscratch1, base);
      Address addr = Address(base, disp);
      addr = masm.legitimize_address(addr, size_in_memory, rscratch1);
      (masm.*insn)(reg, addr);
    } else {
      assert(disp == 0, "unsupported address mode: disp = %d", disp);
      (masm.*insn)(reg, Address(base, as_Register(index), scale));
    }
  }

  static void loadStore(C2_MacroAssembler masm, mem_vector_insn insn,
                        FloatRegister reg, MacroAssembler::SIMD_RegVariant T,
                        int opcode, Register base, int index, int size, int disp)
  {
    if (index == -1) {
      (masm.*insn)(reg, T, Address(base, disp));
    } else {
      assert(disp == 0, "unsupported address mode");
      (masm.*insn)(reg, T, Address(base, as_Register(index), Address::lsl(size)));
    }
  }


#line 1515 "ad_aarch64.cpp"


//SourceForm

#line 31 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"


static void z_load_barrier(MacroAssembler& _masm, const MachNode* node, Address ref_addr, Register ref, Register tmp, bool weak) {
  ZLoadBarrierStubC2* const stub = ZLoadBarrierStubC2::create(node, ref_addr, ref, tmp, weak);
  __ ldr(tmp, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
  __ andr(tmp, tmp, ref);
  __ cbnz(tmp, *stub->entry());
  __ bind(*stub->continuation());
}

static void z_load_barrier_slow_path(MacroAssembler& _masm, const MachNode* node, Address ref_addr, Register ref, Register tmp) {
  ZLoadBarrierStubC2* const stub = ZLoadBarrierStubC2::create(node, ref_addr, ref, tmp, false /* weak */);
  __ b(*stub->entry());
  __ bind(*stub->continuation());
}


#line 1538 "ad_aarch64.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( INSN_COST == 100, "Expect (100) to equal 100");
  assert( BRANCH_COST == 200, "Expect (2 * INSN_COST) to equal 200");
  assert( CALL_COST == 200, "Expect (2 * INSN_COST) to equal 200");
  assert( VOLATILE_REF_COST == 1000, "Expect (10 * INSN_COST) to equal 1000");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R18
  'C', // R18_H
  'C', // R0
  'C', // R0_H
  'C', // R1
  'C', // R1_H
  'C', // R2
  'C', // R2_H
  'C', // R3
  'C', // R3_H
  'C', // R4
  'C', // R4_H
  'C', // R5
  'C', // R5_H
  'C', // R6
  'C', // R6_H
  'C', // R7
  'C', // R7_H
  'C', // R19
  'C', // R19_H
  'C', // R20
  'C', // R20_H
  'C', // R21
  'C', // R21_H
  'C', // R22
  'C', // R22_H
  'C', // R23
  'C', // R23_H
  'C', // R24
  'C', // R24_H
  'C', // R25
  'C', // R25_H
  'C', // R26
  'C', // R26_H
  'C', // R27
  'C', // R27_H
  'N', // R28
  'N', // R28_H
  'N', // R29
  'N', // R29_H
  'N', // R30
  'N', // R30_H
  'N', // R31
  'N', // R31_H
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R18
  'C', // R18_H
  'C', // R0
  'C', // R0_H
  'C', // R1
  'C', // R1_H
  'C', // R2
  'C', // R2_H
  'C', // R3
  'C', // R3_H
  'C', // R4
  'C', // R4_H
  'C', // R5
  'C', // R5_H
  'C', // R6
  'C', // R6_H
  'C', // R7
  'C', // R7_H
  'E', // R19
  'E', // R19_H
  'E', // R20
  'E', // R20_H
  'E', // R21
  'E', // R21_H
  'E', // R22
  'E', // R22_H
  'E', // R23
  'E', // R23_H
  'E', // R24
  'E', // R24_H
  'E', // R25
  'E', // R25_H
  'E', // R26
  'E', // R26_H
  'E', // R27
  'E', // R27_H
  'E', // R28
  'E', // R28_H
  'N', // R29
  'N', // R29_H
  'N', // R30
  'N', // R30_H
  'N', // R31
  'N', // R31_H
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immIExt_rule,
  /*   13 */  immI_le_4_rule,
  /*   14 */  immI_31_rule,
  /*   15 */  immI_8_rule,
  /*   16 */  immI_16_rule,
  /*   17 */  immI_24_rule,
  /*   18 */  immI_32_rule,
  /*   19 */  immI_48_rule,
  /*   20 */  immI_56_rule,
  /*   21 */  immI_63_rule,
  /*   22 */  immI_64_rule,
  /*   23 */  immI_255_rule,
  /*   24 */  immI_65535_rule,
  /*   25 */  immL_255_rule,
  /*   26 */  immL_65535_rule,
  /*   27 */  immL_4294967295_rule,
  /*   28 */  immL_bitmask_rule,
  /*   29 */  immI_bitmask_rule,
  /*   30 */  immIScale_rule,
  /*   31 */  immI26_rule,
  /*   32 */  immI19_rule,
  /*   33 */  immIU12_rule,
  /*   34 */  immLU12_rule,
  /*   35 */  immIOffset_rule,
  /*   36 */  immIOffset1_rule,
  /*   37 */  immIOffset2_rule,
  /*   38 */  immIOffset4_rule,
  /*   39 */  immIOffset8_rule,
  /*   40 */  immIOffset16_rule,
  /*   41 */  immLoffset_rule,
  /*   42 */  immLoffset1_rule,
  /*   43 */  immLoffset2_rule,
  /*   44 */  immLoffset4_rule,
  /*   45 */  immLoffset8_rule,
  /*   46 */  immLoffset16_rule,
  /*   47 */  immIAddSub_rule,
  /*   48 */  immILog_rule,
  /*   49 */  immL_rule,
  /*   50 */  immL0_rule,
  /*   51 */  immL_1_rule,
  /*   52 */  immL_M1_rule,
  /*   53 */  immL_pc_off_rule,
  /*   54 */  immLAddSub_rule,
  /*   55 */  immLLog_rule,
  /*   56 */  immL_32bits_rule,
  /*   57 */  immP_rule,
  /*   58 */  immP0_rule,
  /*   59 */  immP_1_rule,
  /*   60 */  immByteMapBase_rule,
  /*   61 */  immP_M1_rule,
  /*   62 */  immP_M2_rule,
  /*   63 */  immD_rule,
  /*   64 */  immD0_rule,
  /*   65 */  immDPacked_rule,
  /*   66 */  immF_rule,
  /*   67 */  immF0_rule,
  /*   68 */  immFPacked_rule,
  /*   69 */  immN_rule,
  /*   70 */  immN0_rule,
  /*   71 */  immNKlass_rule,
  /*   72 */  iRegI_rule,
  /*   73 */  iRegINoSp_rule,
  /*   74 */  iRegL_rule,
  /*   75 */  iRegLNoSp_rule,
  /*   76 */  iRegP_rule,
  /*   77 */  iRegPNoSp_rule,
  /*   78 */  iRegP_R0_rule,
  /*   79 */  iRegP_R1_rule,
  /*   80 */  iRegP_R2_rule,
  /*   81 */  iRegP_R3_rule,
  /*   82 */  iRegP_R4_rule,
  /*   83 */  iRegP_R5_rule,
  /*   84 */  iRegP_R10_rule,
  /*   85 */  iRegL_R0_rule,
  /*   86 */  iRegL_R2_rule,
  /*   87 */  iRegL_R3_rule,
  /*   88 */  iRegL_R11_rule,
  /*   89 */  iRegP_FP_rule,
  /*   90 */  iRegI_R0_rule,
  /*   91 */  iRegI_R2_rule,
  /*   92 */  iRegI_R3_rule,
  /*   93 */  iRegI_R4_rule,
  /*   94 */  iRegN_rule,
  /*   95 */  iRegN_R0_rule,
  /*   96 */  iRegN_R2_rule,
  /*   97 */  iRegN_R3_rule,
  /*   98 */  iRegNNoSp_rule,
  /*   99 */  iRegIHeapbase_rule,
  /*  100 */  vRegF_rule,
  /*  101 */  vRegD_rule,
  /*  102 */  vecD_rule,
  /*  103 */  vecX_rule,
  /*  104 */  vRegD_V0_rule,
  /*  105 */  vRegD_V1_rule,
  /*  106 */  vRegD_V2_rule,
  /*  107 */  vRegD_V3_rule,
  /*  108 */  vRegD_V4_rule,
  /*  109 */  vRegD_V5_rule,
  /*  110 */  vRegD_V6_rule,
  /*  111 */  vRegD_V7_rule,
  /*  112 */  vRegD_V8_rule,
  /*  113 */  vRegD_V9_rule,
  /*  114 */  vRegD_V10_rule,
  /*  115 */  vRegD_V11_rule,
  /*  116 */  vRegD_V12_rule,
  /*  117 */  vRegD_V13_rule,
  /*  118 */  vRegD_V14_rule,
  /*  119 */  vRegD_V15_rule,
  /*  120 */  vRegD_V16_rule,
  /*  121 */  vRegD_V17_rule,
  /*  122 */  vRegD_V18_rule,
  /*  123 */  vRegD_V19_rule,
  /*  124 */  vRegD_V20_rule,
  /*  125 */  vRegD_V21_rule,
  /*  126 */  vRegD_V22_rule,
  /*  127 */  vRegD_V23_rule,
  /*  128 */  vRegD_V24_rule,
  /*  129 */  vRegD_V25_rule,
  /*  130 */  vRegD_V26_rule,
  /*  131 */  vRegD_V27_rule,
  /*  132 */  vRegD_V28_rule,
  /*  133 */  vRegD_V29_rule,
  /*  134 */  vRegD_V30_rule,
  /*  135 */  vRegD_V31_rule,
  /*  136 */  rFlagsReg_rule,
  /*  137 */  rFlagsRegU_rule,
  /*  138 */  inline_cache_RegP_rule,
  /*  139 */  interpreter_method_oop_RegP_rule,
  /*  140 */  thread_RegP_rule,
  /*  141 */  lr_RegP_rule,
  /*  142 */  indirect_rule,
  /*  143 */  indIndexScaledI2L_rule,
  /*  144 */  indIndexScaled_rule,
  /*  145 */  indIndexI2L_rule,
  /*  146 */  indIndex_rule,
  /*  147 */  indOffI_rule,
  /*  148 */  indOffI1_rule,
  /*  149 */  indOffI2_rule,
  /*  150 */  indOffI4_rule,
  /*  151 */  indOffI8_rule,
  /*  152 */  indOffI16_rule,
  /*  153 */  indOffL_rule,
  /*  154 */  indOffL1_rule,
  /*  155 */  indOffL2_rule,
  /*  156 */  indOffL4_rule,
  /*  157 */  indOffL8_rule,
  /*  158 */  indOffL16_rule,
  /*  159 */  indirectN_rule,
  /*  160 */  indIndexScaledI2LN_rule,
  /*  161 */  indIndexScaledN_rule,
  /*  162 */  indIndexI2LN_rule,
  /*  163 */  indIndexN_rule,
  /*  164 */  indOffIN_rule,
  /*  165 */  indOffLN_rule,
  /*  166 */  thread_anchor_pc_rule,
  /*  167 */  stackSlotP_rule,
  /*  168 */  stackSlotI_rule,
  /*  169 */  stackSlotF_rule,
  /*  170 */  stackSlotD_rule,
  /*  171 */  stackSlotL_rule,
  /*  172 */  cmpOp_rule,
  /*  173 */  cmpOpU_rule,
  /*  174 */  cmpOpEqNe_rule,
  /*  175 */  cmpOpLtGe_rule,
  /*  176 */  cmpOpUEqNeLtGe_rule,
  /*  177 */  iRegL2I_rule,
  // last operand
  /*  178 */  vmem4_rule,
  /*  179 */  vmem8_rule,
  /*  180 */  vmem16_rule,
  /*  181 */  memory1_rule,
  /*  182 */  memory2_rule,
  /*  183 */  memory4_rule,
  /*  184 */  memory8_rule,
  /*  185 */  memory_rule,
  /*  186 */  iRegIorL2I_rule,
  // last operand class
  /*  187 */  _ConvI2L_iRegI__rule,
  /*  188 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  189 */  _LShiftL_iRegL_immIScale_rule,
  /*  190 */  _DecodeN_iRegN__rule,
  /*  191 */  _LoadB_memory1__rule,
  /*  192 */  _LoadUB_memory1__rule,
  /*  193 */  _LoadS_memory2__rule,
  /*  194 */  _LoadUS_memory2__rule,
  /*  195 */  _LoadI_memory4__rule,
  /*  196 */  _ConvI2L__LoadI_memory4___rule,
  /*  197 */  _LoadB_indirect__rule,
  /*  198 */  _LoadUB_indirect__rule,
  /*  199 */  _LoadUS_indirect__rule,
  /*  200 */  _LoadS_indirect__rule,
  /*  201 */  _LoadI_indirect__rule,
  /*  202 */  _ConvI2L__LoadI_indirect___rule,
  /*  203 */  _LoadL_memory8__rule,
  /*  204 */  _CastP2X_iRegP__rule,
  /*  205 */  _CastP2X__DecodeN_iRegN___rule,
  /*  206 */  _Binary_iRegP_iRegP_rule,
  /*  207 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  208 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  209 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  210 */  _Binary_iRegI_iRegI_rule,
  /*  211 */  _Binary_iRegL_iRegL_rule,
  /*  212 */  _Binary_iRegN_iRegN_rule,
  /*  213 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  214 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  215 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  216 */  _Binary_immI0_iRegIorL2I_rule,
  /*  217 */  _Binary_iRegIorL2I_immI0_rule,
  /*  218 */  _Binary_immI_1_immI0_rule,
  /*  219 */  _Binary_iRegL_immL0_rule,
  /*  220 */  _Binary_immL0_iRegL_rule,
  /*  221 */  _Binary_iRegP_immP0_rule,
  /*  222 */  _Binary_immP0_iRegP_rule,
  /*  223 */  _Binary_iRegN_immN0_rule,
  /*  224 */  _Binary_immN0_iRegN_rule,
  /*  225 */  _Binary_vRegF_vRegF_rule,
  /*  226 */  _Binary_vRegD_vRegD_rule,
  /*  227 */  _ConvL2I_iRegL__rule,
  /*  228 */  _ConvI2L_iRegIorL2I__rule,
  /*  229 */  _LShiftL__ConvI2L_iRegIorL2I__immIScale_rule,
  /*  230 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  231 */  _SubI_immI0_iRegIorL2I_rule,
  /*  232 */  _MulL_iRegL_iRegL_rule,
  /*  233 */  _SubL_immL0_iRegL_rule,
  /*  234 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  235 */  _SubL_immL0__ConvI2L_iRegIorL2I__rule,
  /*  236 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  237 */  _XorL_iRegL_immL_M1_rule,
  /*  238 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  239 */  _XorL_iRegL_iRegL_rule,
  /*  240 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  241 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  242 */  _URShiftL_iRegL_immI_rule,
  /*  243 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  244 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  245 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  246 */  _RShiftL_iRegL_immI_rule,
  /*  247 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  248 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  249 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  250 */  _LShiftL_iRegL_immI_rule,
  /*  251 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  252 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  253 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  254 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  255 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  256 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  257 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  258 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  259 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  260 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  261 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  262 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  263 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  264 */  _AndI__URShiftI_iRegIorL2I_immI_immI_bitmask_rule,
  /*  265 */  _AndI_iRegIorL2I_immI_bitmask_rule,
  /*  266 */  _AndL_iRegL_immL_bitmask_rule,
  /*  267 */  _ConvI2L__AndI_iRegIorL2I_immI_bitmask__rule,
  /*  268 */  _LShiftL_iRegL_iRegI_rule,
  /*  269 */  _SubI_immI_64_iRegI_rule,
  /*  270 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  271 */  _SubI_immI0_iRegI_rule,
  /*  272 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  273 */  _LShiftI_iRegI_iRegI_rule,
  /*  274 */  _SubI_immI_32_iRegI_rule,
  /*  275 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  276 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  277 */  _URShiftL_iRegL_iRegI_rule,
  /*  278 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  279 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  280 */  _URShiftI_iRegI_iRegI_rule,
  /*  281 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  282 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  283 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  284 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  285 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  286 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  287 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  288 */  _LShiftL_iRegL_immI_48_rule,
  /*  289 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  290 */  _LShiftL_iRegL_immI_32_rule,
  /*  291 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  292 */  _LShiftL_iRegL_immI_56_rule,
  /*  293 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  294 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  295 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  296 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  297 */  _AndL_iRegL_immL_255_rule,
  /*  298 */  _AndL_iRegL_immL_65535_rule,
  /*  299 */  _AndL_iRegL_immL_4294967295_rule,
  /*  300 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_56_immI_56_immIExt_rule,
  /*  301 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_48_immI_48_immIExt_rule,
  /*  302 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_32_immI_32_immIExt_rule,
  /*  303 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_immIExt_rule,
  /*  304 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_immIExt_rule,
  /*  305 */  _LShiftL__ConvI2L_iRegIorL2I__immIExt_rule,
  /*  306 */  _LShiftL__AndL_iRegL_immL_255_immIExt_rule,
  /*  307 */  _LShiftL__AndL_iRegL_immL_65535_immIExt_rule,
  /*  308 */  _LShiftL__AndL_iRegL_immL_4294967295_immIExt_rule,
  /*  309 */  _LShiftI__AndI_iRegIorL2I_immI_255_immIExt_rule,
  /*  310 */  _LShiftI__AndI_iRegIorL2I_immI_65535_immIExt_rule,
  /*  311 */  _NegF_vRegF__rule,
  /*  312 */  _Binary__NegF_vRegF__vRegF_rule,
  /*  313 */  _Binary_vRegF__NegF_vRegF__rule,
  /*  314 */  _NegD_vRegD__rule,
  /*  315 */  _Binary__NegD_vRegD__vRegD_rule,
  /*  316 */  _Binary_vRegD__NegD_vRegD__rule,
  /*  317 */  _OverflowMulI_iRegIorL2I_iRegIorL2I_rule,
  /*  318 */  _OverflowMulL_iRegL_iRegL_rule,
  /*  319 */  _CmpI_iRegIorL2I_immI0_rule,
  /*  320 */  _CmpL_iRegL_immL0_rule,
  /*  321 */  _CmpP_iRegP_immP0_rule,
  /*  322 */  _CmpN_iRegN_immN0_rule,
  /*  323 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  324 */  _CmpU_iRegIorL2I_immI0_rule,
  /*  325 */  _CmpUL_iRegL_immL0_rule,
  /*  326 */  _AndL_iRegL_immL_rule,
  /*  327 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /*  328 */  _AndI_iRegIorL2I_immI_rule,
  /*  329 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /*  330 */  _AndL_iRegL_iRegL_rule,
  /*  331 */  _AndI_iRegIorL2I_iRegIorL2I_rule,
  /*  332 */  _PartialSubtypeCheck_iRegP_R4_iRegP_R0_rule,
  /*  333 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /*  334 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /*  335 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /*  336 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /*  337 */  _Binary_iRegP_R3_immI_le_4_rule,
  /*  338 */  _Binary_iRegP_R3_immI_1_rule,
  /*  339 */  _Binary_iRegP_R1_iRegP_R3_rule,
  /*  340 */  _Binary_iRegP_R1_iRegI_R3_rule,
  /*  341 */  _MulVS_vecD_vecD_rule,
  /*  342 */  _MulVS_vecX_vecX_rule,
  /*  343 */  _MulVI_vecD_vecD_rule,
  /*  344 */  _MulVI_vecX_vecX_rule,
  /*  345 */  _Binary_vecD_vecD_rule,
  /*  346 */  _Binary_vecX_vecX_rule,
  /*  347 */  _NegVF_vecD__rule,
  /*  348 */  _Binary__NegVF_vecD__vecD_rule,
  /*  349 */  _Binary_vecD__NegVF_vecD__rule,
  /*  350 */  _NegVF_vecX__rule,
  /*  351 */  _Binary__NegVF_vecX__vecX_rule,
  /*  352 */  _Binary_vecX__NegVF_vecX__rule,
  /*  353 */  _NegVD_vecX__rule,
  /*  354 */  _Binary__NegVD_vecX__vecX_rule,
  /*  355 */  _Binary_vecX__NegVD_vecX__rule,
  /*  356 */  _LShiftCntV_immI__rule,
  /*  357 */  _RShiftCntV_immI__rule,
  // last internally defined operand
  /*  358 */  iRegINoSp_rule,
  /*  359 */  iRegLNoSp_rule,
  /*  360 */  iRegPNoSp_rule,
  /*  361 */  iRegPNoSp_rule,
  /*  362 */  iRegPNoSp_rule,
  /*  363 */  iRegPNoSp_rule,
  /*  364 */  iRegNNoSp_rule,
  /*  365 */  iRegNNoSp_rule,
  /*  366 */  iRegNNoSp_rule,
  /*  367 */  vRegF_rule,
  /*  368 */  vRegF_rule,
  /*  369 */  vRegD_rule,
  /*  370 */  vRegD_rule,
  /*  371 */  iRegINoSp_rule,
  /*  372 */  iRegINoSp_rule,
  /*  373 */  iRegINoSp_rule,
  /*  374 */  iRegINoSp_rule,
  /*  375 */  iRegPNoSp_rule,
  /*  376 */  iRegLNoSp_rule,
  /*  377 */  iRegPNoSp_rule,
  /*  378 */  iRegPNoSp_rule,
  /*  379 */  iRegI_rule,
  /*  380 */  rFlagsReg_rule,
  /*  381 */  rFlagsReg_rule,
  /*  382 */  rFlagsReg_rule,
  /*  383 */  vRegF_rule,
  /*  384 */  vRegD_rule,
  /*  385 */  vRegF_rule,
  /*  386 */  vRegD_rule,
  /*  387 */  iRegINoSp_rule,
  /*  388 */  vRegF_rule,
  /*  389 */  iRegLNoSp_rule,
  /*  390 */  vRegD_rule,
  /*  391 */  iRegINoSp_rule,
  /*  392 */  vRegF_rule,
  /*  393 */  iRegLNoSp_rule,
  /*  394 */  vRegD_rule,
  /*  395 */  rFlagsReg_rule,
  /*  396 */  rFlagsReg_rule,
  /*  397 */  rFlagsReg_rule,
  /*  398 */  rFlagsReg_rule,
  /*  399 */  rFlagsReg_rule,
  /*  400 */  rFlagsReg_rule,
  /*  401 */  rFlagsReg_rule,
  /*  402 */  rFlagsReg_rule,
  /*  403 */  rFlagsReg_rule,
  /*  404 */  rFlagsReg_rule,
  /*  405 */  rFlagsReg_rule,
  /*  406 */  rFlagsReg_rule,
  /*  407 */  rFlagsReg_rule,
  /*  408 */  rFlagsReg_rule,
  /*  409 */  rFlagsReg_rule,
  /*  410 */  rFlagsReg_rule,
  /*  411 */  rFlagsRegU_rule,
  /*  412 */  rFlagsRegU_rule,
  /*  413 */  rFlagsRegU_rule,
  /*  414 */  rFlagsRegU_rule,
  /*  415 */  rFlagsReg_rule,
  /*  416 */  rFlagsReg_rule,
  /*  417 */  rFlagsReg_rule,
  /*  418 */  rFlagsReg_rule,
  /*  419 */  rFlagsRegU_rule,
  /*  420 */  rFlagsRegU_rule,
  /*  421 */  rFlagsRegU_rule,
  /*  422 */  rFlagsRegU_rule,
  /*  423 */  rFlagsRegU_rule,
  /*  424 */  rFlagsRegU_rule,
  /*  425 */  rFlagsRegU_rule,
  /*  426 */  rFlagsRegU_rule,
  /*  427 */  rFlagsReg_rule,
  /*  428 */  rFlagsReg_rule,
  /*  429 */  rFlagsReg_rule,
  /*  430 */  rFlagsReg_rule,
  /*  431 */  rFlagsReg_rule,
  /*  432 */  rFlagsReg_rule,
  /*  433 */  rFlagsReg_rule,
  /*  434 */  rFlagsReg_rule,
  /*  435 */  rFlagsReg_rule,
  /*  436 */  rFlagsReg_rule,
  /*  437 */  rFlagsReg_rule,
  /*  438 */  thread_RegP_rule,
  /*  439 */  vecD_rule,
  /*  440 */  vecX_rule,
  /*  441 */  vecX_rule,
  /*  442 */  vecD_rule,
  /*  443 */  vecX_rule,
  /*  444 */  vecD_rule,
  /*  445 */  vecX_rule,
  /*  446 */  vecD_rule,
  /*  447 */  vecX_rule,
  /*  448 */  vecX_rule,
  /*  449 */  vecD_rule,
  /*  450 */  vecX_rule,
  /*  451 */  vecX_rule,
  /*  452 */  vecD_rule,
  /*  453 */  vecX_rule,
  /*  454 */  vecX_rule,
  /*  455 */  vecD_rule,
  /*  456 */  vecD_rule,
  /*  457 */  vecX_rule,
  /*  458 */  vecX_rule,
  /*  459 */  vecX_rule,
  /*  460 */  vecD_rule,
  /*  461 */  iRegINoSp_rule,
  /*  462 */  iRegLNoSp_rule,
  /*  463 */  iRegINoSp_rule,
  /*  464 */  iRegLNoSp_rule,
  /*  465 */  iRegINoSp_rule,
  /*  466 */  iRegLNoSp_rule,
  /*  467 */  iRegINoSp_rule,
  /*  468 */  iRegLNoSp_rule,
  /*  469 */  iRegINoSp_rule,
  /*  470 */  iRegLNoSp_rule,
  /*  471 */  iRegLNoSp_rule,
  /*  472 */  iRegLNoSp_rule,
  /*  473 */  iRegINoSp_rule,
  /*  474 */  iRegPNoSp_rule,
  /*  475 */  iRegNNoSp_rule,
  /*  476 */  iRegPNoSp_rule,
  /*  477 */  iRegNNoSp_rule,
  /*  478 */  vRegF_rule,
  /*  479 */  vRegD_rule,
  /*  480 */  Universe_rule,
  /*  481 */  Universe_rule,
  /*  482 */  Universe_rule,
  /*  483 */  Universe_rule,
  /*  484 */  Universe_rule,
  /*  485 */  Universe_rule,
  /*  486 */  Universe_rule,
  /*  487 */  Universe_rule,
  /*  488 */  Universe_rule,
  /*  489 */  Universe_rule,
  /*  490 */  Universe_rule,
  /*  491 */  Universe_rule,
  /*  492 */  Universe_rule,
  /*  493 */  Universe_rule,
  /*  494 */  Universe_rule,
  /*  495 */  Universe_rule,
  /*  496 */  Universe_rule,
  /*  497 */  Universe_rule,
  /*  498 */  iRegINoSp_rule,
  /*  499 */  iRegLNoSp_rule,
  /*  500 */  iRegINoSp_rule,
  /*  501 */  iRegLNoSp_rule,
  /*  502 */  iRegINoSp_rule,
  /*  503 */  iRegINoSp_rule,
  /*  504 */  iRegLNoSp_rule,
  /*  505 */  iRegLNoSp_rule,
  /*  506 */  iRegINoSp_rule,
  /*  507 */  iRegLNoSp_rule,
  /*  508 */  iRegLNoSp_rule,
  /*  509 */  iRegPNoSp_rule,
  /*  510 */  iRegNNoSp_rule,
  /*  511 */  vRegF_rule,
  /*  512 */  vRegD_rule,
  /*  513 */  Universe_rule,
  /*  514 */  Universe_rule,
  /*  515 */  Universe_rule,
  /*  516 */  Universe_rule,
  /*  517 */  Universe_rule,
  /*  518 */  Universe_rule,
  /*  519 */  Universe_rule,
  /*  520 */  Universe_rule,
  /*  521 */  Universe_rule,
  /*  522 */  Universe_rule,
  /*  523 */  Universe_rule,
  /*  524 */  iRegINoSp_rule,
  /*  525 */  iRegLNoSp_rule,
  /*  526 */  iRegINoSp_rule,
  /*  527 */  iRegINoSp_rule,
  /*  528 */  iRegINoSp_rule,
  /*  529 */  iRegINoSp_rule,
  /*  530 */  iRegINoSp_rule,
  /*  531 */  iRegINoSp_rule,
  /*  532 */  Universe_rule,
  /*  533 */  Universe_rule,
  /*  534 */  Universe_rule,
  /*  535 */  Universe_rule,
  /*  536 */  Universe_rule,
  /*  537 */  Universe_rule,
  /*  538 */  Universe_rule,
  /*  539 */  Universe_rule,
  /*  540 */  Universe_rule,
  /*  541 */  Universe_rule,
  /*  542 */  Universe_rule,
  /*  543 */  iRegINoSp_rule,
  /*  544 */  iRegINoSp_rule,
  /*  545 */  iRegNNoSp_rule,
  /*  546 */  iRegNNoSp_rule,
  /*  547 */  iRegPNoSp_rule,
  /*  548 */  iRegPNoSp_rule,
  /*  549 */  iRegNNoSp_rule,
  /*  550 */  iRegPNoSp_rule,
  /*  551 */  iRegPNoSp_rule,
  /*  552 */  iRegINoSp_rule,
  /*  553 */  iRegINoSp_rule,
  /*  554 */  iRegINoSp_rule,
  /*  555 */  iRegINoSp_rule,
  /*  556 */  iRegINoSp_rule,
  /*  557 */  iRegINoSp_rule,
  /*  558 */  iRegINoSp_rule,
  /*  559 */  iRegINoSp_rule,
  /*  560 */  iRegINoSp_rule,
  /*  561 */  iRegINoSp_rule,
  /*  562 */  iRegINoSp_rule,
  /*  563 */  iRegINoSp_rule,
  /*  564 */  iRegINoSp_rule,
  /*  565 */  iRegINoSp_rule,
  /*  566 */  iRegINoSp_rule,
  /*  567 */  iRegLNoSp_rule,
  /*  568 */  iRegNNoSp_rule,
  /*  569 */  iRegPNoSp_rule,
  /*  570 */  iRegINoSp_rule,
  /*  571 */  iRegINoSp_rule,
  /*  572 */  iRegINoSp_rule,
  /*  573 */  iRegLNoSp_rule,
  /*  574 */  iRegNNoSp_rule,
  /*  575 */  iRegPNoSp_rule,
  /*  576 */  iRegINoSp_rule,
  /*  577 */  iRegINoSp_rule,
  /*  578 */  iRegINoSp_rule,
  /*  579 */  iRegINoSp_rule,
  /*  580 */  iRegINoSp_rule,
  /*  581 */  iRegINoSp_rule,
  /*  582 */  iRegINoSp_rule,
  /*  583 */  iRegINoSp_rule,
  /*  584 */  iRegINoSp_rule,
  /*  585 */  iRegINoSp_rule,
  /*  586 */  iRegINoSp_rule,
  /*  587 */  iRegINoSp_rule,
  /*  588 */  iRegINoSp_rule,
  /*  589 */  iRegLNoSp_rule,
  /*  590 */  iRegINoSp_rule,
  /*  591 */  iRegPNoSp_rule,
  /*  592 */  iRegINoSp_rule,
  /*  593 */  iRegLNoSp_rule,
  /*  594 */  iRegINoSp_rule,
  /*  595 */  iRegPNoSp_rule,
  /*  596 */  iRegLNoSp_rule,
  /*  597 */  Universe_rule,
  /*  598 */  iRegLNoSp_rule,
  /*  599 */  Universe_rule,
  /*  600 */  iRegINoSp_rule,
  /*  601 */  Universe_rule,
  /*  602 */  iRegINoSp_rule,
  /*  603 */  Universe_rule,
  /*  604 */  iRegLNoSp_rule,
  /*  605 */  Universe_rule,
  /*  606 */  iRegLNoSp_rule,
  /*  607 */  Universe_rule,
  /*  608 */  iRegINoSp_rule,
  /*  609 */  Universe_rule,
  /*  610 */  iRegINoSp_rule,
  /*  611 */  Universe_rule,
  /*  612 */  iRegINoSp_rule,
  /*  613 */  iRegINoSp_rule,
  /*  614 */  iRegINoSp_rule,
  /*  615 */  iRegINoSp_rule,
  /*  616 */  iRegINoSp_rule,
  /*  617 */  iRegINoSp_rule,
  /*  618 */  iRegINoSp_rule,
  /*  619 */  iRegINoSp_rule,
  /*  620 */  iRegINoSp_rule,
  /*  621 */  iRegINoSp_rule,
  /*  622 */  iRegLNoSp_rule,
  /*  623 */  iRegLNoSp_rule,
  /*  624 */  iRegLNoSp_rule,
  /*  625 */  iRegLNoSp_rule,
  /*  626 */  iRegLNoSp_rule,
  /*  627 */  iRegLNoSp_rule,
  /*  628 */  iRegPNoSp_rule,
  /*  629 */  iRegPNoSp_rule,
  /*  630 */  iRegPNoSp_rule,
  /*  631 */  iRegPNoSp_rule,
  /*  632 */  iRegPNoSp_rule,
  /*  633 */  iRegPNoSp_rule,
  /*  634 */  iRegNNoSp_rule,
  /*  635 */  iRegNNoSp_rule,
  /*  636 */  iRegNNoSp_rule,
  /*  637 */  iRegNNoSp_rule,
  /*  638 */  iRegNNoSp_rule,
  /*  639 */  iRegNNoSp_rule,
  /*  640 */  vRegF_rule,
  /*  641 */  vRegF_rule,
  /*  642 */  vRegD_rule,
  /*  643 */  vRegD_rule,
  /*  644 */  iRegINoSp_rule,
  /*  645 */  iRegINoSp_rule,
  /*  646 */  iRegINoSp_rule,
  /*  647 */  iRegPNoSp_rule,
  /*  648 */  iRegPNoSp_rule,
  /*  649 */  iRegPNoSp_rule,
  /*  650 */  iRegPNoSp_rule,
  /*  651 */  iRegLNoSp_rule,
  /*  652 */  iRegPNoSp_rule,
  /*  653 */  iRegLNoSp_rule,
  /*  654 */  iRegLNoSp_rule,
  /*  655 */  iRegINoSp_rule,
  /*  656 */  iRegINoSp_rule,
  /*  657 */  iRegLNoSp_rule,
  /*  658 */  iRegLNoSp_rule,
  /*  659 */  iRegINoSp_rule,
  /*  660 */  iRegLNoSp_rule,
  /*  661 */  iRegINoSp_rule,
  /*  662 */  iRegLNoSp_rule,
  /*  663 */  iRegLNoSp_rule,
  /*  664 */  iRegLNoSp_rule,
  /*  665 */  iRegLNoSp_rule,
  /*  666 */  iRegINoSp_rule,
  /*  667 */  iRegINoSp_rule,
  /*  668 */  iRegINoSp_rule,
  /*  669 */  iRegINoSp_rule,
  /*  670 */  iRegINoSp_rule,
  /*  671 */  iRegINoSp_rule,
  /*  672 */  iRegINoSp_rule,
  /*  673 */  iRegLNoSp_rule,
  /*  674 */  iRegLNoSp_rule,
  /*  675 */  iRegLNoSp_rule,
  /*  676 */  iRegLNoSp_rule,
  /*  677 */  iRegLNoSp_rule,
  /*  678 */  iRegLNoSp_rule,
  /*  679 */  iRegLNoSp_rule,
  /*  680 */  iRegLNoSp_rule,
  /*  681 */  iRegLNoSp_rule,
  /*  682 */  iRegLNoSp_rule,
  /*  683 */  iRegLNoSp_rule,
  /*  684 */  iRegLNoSp_rule,
  /*  685 */  iRegLNoSp_rule,
  /*  686 */  iRegLNoSp_rule,
  /*  687 */  iRegLNoSp_rule,
  /*  688 */  iRegLNoSp_rule,
  /*  689 */  iRegLNoSp_rule,
  /*  690 */  iRegINoSp_rule,
  /*  691 */  iRegINoSp_rule,
  /*  692 */  iRegLNoSp_rule,
  /*  693 */  iRegINoSp_rule,
  /*  694 */  iRegLNoSp_rule,
  /*  695 */  iRegINoSp_rule,
  /*  696 */  iRegINoSp_rule,
  /*  697 */  iRegINoSp_rule,
  /*  698 */  iRegINoSp_rule,
  /*  699 */  iRegINoSp_rule,
  /*  700 */  iRegINoSp_rule,
  /*  701 */  iRegLNoSp_rule,
  /*  702 */  iRegLNoSp_rule,
  /*  703 */  iRegLNoSp_rule,
  /*  704 */  iRegLNoSp_rule,
  /*  705 */  iRegLNoSp_rule,
  /*  706 */  iRegLNoSp_rule,
  /*  707 */  iRegLNoSp_rule,
  /*  708 */  iRegLNoSp_rule,
  /*  709 */  iRegINoSp_rule,
  /*  710 */  iRegINoSp_rule,
  /*  711 */  iRegINoSp_rule,
  /*  712 */  iRegLNoSp_rule,
  /*  713 */  iRegLNoSp_rule,
  /*  714 */  iRegINoSp_rule,
  /*  715 */  iRegINoSp_rule,
  /*  716 */  iRegLNoSp_rule,
  /*  717 */  iRegLNoSp_rule,
  /*  718 */  iRegINoSp_rule,
  /*  719 */  iRegINoSp_rule,
  /*  720 */  iRegLNoSp_rule,
  /*  721 */  iRegLNoSp_rule,
  /*  722 */  iRegINoSp_rule,
  /*  723 */  iRegINoSp_rule,
  /*  724 */  iRegLNoSp_rule,
  /*  725 */  iRegLNoSp_rule,
  /*  726 */  iRegINoSp_rule,
  /*  727 */  iRegINoSp_rule,
  /*  728 */  iRegLNoSp_rule,
  /*  729 */  iRegLNoSp_rule,
  /*  730 */  iRegINoSp_rule,
  /*  731 */  iRegINoSp_rule,
  /*  732 */  iRegLNoSp_rule,
  /*  733 */  iRegLNoSp_rule,
  /*  734 */  iRegINoSp_rule,
  /*  735 */  iRegINoSp_rule,
  /*  736 */  iRegINoSp_rule,
  /*  737 */  iRegINoSp_rule,
  /*  738 */  iRegLNoSp_rule,
  /*  739 */  iRegLNoSp_rule,
  /*  740 */  iRegLNoSp_rule,
  /*  741 */  iRegLNoSp_rule,
  /*  742 */  iRegINoSp_rule,
  /*  743 */  iRegINoSp_rule,
  /*  744 */  iRegINoSp_rule,
  /*  745 */  iRegINoSp_rule,
  /*  746 */  iRegLNoSp_rule,
  /*  747 */  iRegLNoSp_rule,
  /*  748 */  iRegLNoSp_rule,
  /*  749 */  iRegLNoSp_rule,
  /*  750 */  iRegINoSp_rule,
  /*  751 */  iRegINoSp_rule,
  /*  752 */  iRegINoSp_rule,
  /*  753 */  iRegINoSp_rule,
  /*  754 */  iRegLNoSp_rule,
  /*  755 */  iRegLNoSp_rule,
  /*  756 */  iRegLNoSp_rule,
  /*  757 */  iRegLNoSp_rule,
  /*  758 */  iRegINoSp_rule,
  /*  759 */  iRegINoSp_rule,
  /*  760 */  iRegLNoSp_rule,
  /*  761 */  iRegLNoSp_rule,
  /*  762 */  iRegINoSp_rule,
  /*  763 */  iRegINoSp_rule,
  /*  764 */  iRegLNoSp_rule,
  /*  765 */  iRegLNoSp_rule,
  /*  766 */  iRegINoSp_rule,
  /*  767 */  iRegINoSp_rule,
  /*  768 */  iRegLNoSp_rule,
  /*  769 */  iRegLNoSp_rule,
  /*  770 */  iRegINoSp_rule,
  /*  771 */  iRegINoSp_rule,
  /*  772 */  iRegLNoSp_rule,
  /*  773 */  iRegLNoSp_rule,
  /*  774 */  iRegINoSp_rule,
  /*  775 */  iRegINoSp_rule,
  /*  776 */  iRegLNoSp_rule,
  /*  777 */  iRegLNoSp_rule,
  /*  778 */  iRegINoSp_rule,
  /*  779 */  iRegINoSp_rule,
  /*  780 */  iRegLNoSp_rule,
  /*  781 */  iRegLNoSp_rule,
  /*  782 */  iRegINoSp_rule,
  /*  783 */  iRegINoSp_rule,
  /*  784 */  iRegLNoSp_rule,
  /*  785 */  iRegLNoSp_rule,
  /*  786 */  iRegINoSp_rule,
  /*  787 */  iRegINoSp_rule,
  /*  788 */  iRegLNoSp_rule,
  /*  789 */  iRegLNoSp_rule,
  /*  790 */  iRegINoSp_rule,
  /*  791 */  iRegINoSp_rule,
  /*  792 */  iRegLNoSp_rule,
  /*  793 */  iRegLNoSp_rule,
  /*  794 */  iRegINoSp_rule,
  /*  795 */  iRegINoSp_rule,
  /*  796 */  iRegLNoSp_rule,
  /*  797 */  iRegLNoSp_rule,
  /*  798 */  iRegINoSp_rule,
  /*  799 */  iRegINoSp_rule,
  /*  800 */  iRegLNoSp_rule,
  /*  801 */  iRegLNoSp_rule,
  /*  802 */  iRegINoSp_rule,
  /*  803 */  iRegINoSp_rule,
  /*  804 */  iRegLNoSp_rule,
  /*  805 */  iRegLNoSp_rule,
  /*  806 */  iRegINoSp_rule,
  /*  807 */  iRegINoSp_rule,
  /*  808 */  iRegLNoSp_rule,
  /*  809 */  iRegLNoSp_rule,
  /*  810 */  iRegINoSp_rule,
  /*  811 */  iRegINoSp_rule,
  /*  812 */  iRegLNoSp_rule,
  /*  813 */  iRegLNoSp_rule,
  /*  814 */  iRegINoSp_rule,
  /*  815 */  iRegINoSp_rule,
  /*  816 */  iRegLNoSp_rule,
  /*  817 */  iRegLNoSp_rule,
  /*  818 */  iRegINoSp_rule,
  /*  819 */  iRegLNoSp_rule,
  /*  820 */  iRegINoSp_rule,
  /*  821 */  iRegLNoSp_rule,
  /*  822 */  iRegINoSp_rule,
  /*  823 */  iRegLNoSp_rule,
  /*  824 */  iRegLNoSp_rule,
  /*  825 */  iRegINoSp_rule,
  /*  826 */  iRegLNoSp_rule,
  /*  827 */  iRegINoSp_rule,
  /*  828 */  iRegINoSp_rule,
  /*  829 */  iRegLNoSp_rule,
  /*  830 */  iRegLNoSp_rule,
  /*  831 */  iRegINoSp_rule,
  /*  832 */  iRegLNoSp_rule,
  /*  833 */  iRegLNoSp_rule,
  /*  834 */  iRegLNoSp_rule,
  /*  835 */  iRegLNoSp_rule,
  /*  836 */  iRegINoSp_rule,
  /*  837 */  iRegINoSp_rule,
  /*  838 */  iRegLNoSp_rule,
  /*  839 */  iRegLNoSp_rule,
  /*  840 */  iRegINoSp_rule,
  /*  841 */  iRegINoSp_rule,
  /*  842 */  0,
  /*  843 */  0,
  /*  844 */  iRegLNoSp_rule,
  /*  845 */  iRegLNoSp_rule,
  /*  846 */  iRegLNoSp_rule,
  /*  847 */  iRegLNoSp_rule,
  /*  848 */  iRegINoSp_rule,
  /*  849 */  iRegINoSp_rule,
  /*  850 */  iRegINoSp_rule,
  /*  851 */  iRegINoSp_rule,
  /*  852 */  0,
  /*  853 */  0,
  /*  854 */  iRegLNoSp_rule,
  /*  855 */  iRegLNoSp_rule,
  /*  856 */  iRegLNoSp_rule,
  /*  857 */  iRegLNoSp_rule,
  /*  858 */  iRegINoSp_rule,
  /*  859 */  iRegINoSp_rule,
  /*  860 */  iRegINoSp_rule,
  /*  861 */  iRegINoSp_rule,
  /*  862 */  iRegLNoSp_rule,
  /*  863 */  iRegLNoSp_rule,
  /*  864 */  iRegLNoSp_rule,
  /*  865 */  iRegINoSp_rule,
  /*  866 */  iRegINoSp_rule,
  /*  867 */  iRegINoSp_rule,
  /*  868 */  iRegINoSp_rule,
  /*  869 */  iRegINoSp_rule,
  /*  870 */  iRegINoSp_rule,
  /*  871 */  iRegLNoSp_rule,
  /*  872 */  iRegLNoSp_rule,
  /*  873 */  iRegLNoSp_rule,
  /*  874 */  iRegLNoSp_rule,
  /*  875 */  iRegLNoSp_rule,
  /*  876 */  iRegLNoSp_rule,
  /*  877 */  iRegLNoSp_rule,
  /*  878 */  iRegLNoSp_rule,
  /*  879 */  iRegINoSp_rule,
  /*  880 */  iRegINoSp_rule,
  /*  881 */  iRegINoSp_rule,
  /*  882 */  iRegINoSp_rule,
  /*  883 */  iRegLNoSp_rule,
  /*  884 */  iRegLNoSp_rule,
  /*  885 */  iRegLNoSp_rule,
  /*  886 */  iRegLNoSp_rule,
  /*  887 */  iRegLNoSp_rule,
  /*  888 */  iRegLNoSp_rule,
  /*  889 */  iRegINoSp_rule,
  /*  890 */  iRegINoSp_rule,
  /*  891 */  iRegLNoSp_rule,
  /*  892 */  iRegLNoSp_rule,
  /*  893 */  iRegLNoSp_rule,
  /*  894 */  iRegLNoSp_rule,
  /*  895 */  iRegLNoSp_rule,
  /*  896 */  iRegLNoSp_rule,
  /*  897 */  iRegLNoSp_rule,
  /*  898 */  iRegLNoSp_rule,
  /*  899 */  iRegLNoSp_rule,
  /*  900 */  iRegLNoSp_rule,
  /*  901 */  iRegLNoSp_rule,
  /*  902 */  iRegLNoSp_rule,
  /*  903 */  iRegINoSp_rule,
  /*  904 */  iRegINoSp_rule,
  /*  905 */  iRegINoSp_rule,
  /*  906 */  iRegINoSp_rule,
  /*  907 */  iRegINoSp_rule,
  /*  908 */  iRegINoSp_rule,
  /*  909 */  iRegLNoSp_rule,
  /*  910 */  iRegLNoSp_rule,
  /*  911 */  iRegLNoSp_rule,
  /*  912 */  iRegLNoSp_rule,
  /*  913 */  iRegLNoSp_rule,
  /*  914 */  iRegLNoSp_rule,
  /*  915 */  iRegLNoSp_rule,
  /*  916 */  iRegLNoSp_rule,
  /*  917 */  iRegLNoSp_rule,
  /*  918 */  iRegLNoSp_rule,
  /*  919 */  iRegLNoSp_rule,
  /*  920 */  iRegLNoSp_rule,
  /*  921 */  iRegINoSp_rule,
  /*  922 */  iRegINoSp_rule,
  /*  923 */  iRegINoSp_rule,
  /*  924 */  iRegINoSp_rule,
  /*  925 */  iRegINoSp_rule,
  /*  926 */  iRegINoSp_rule,
  /*  927 */  vRegF_rule,
  /*  928 */  vRegD_rule,
  /*  929 */  vRegF_rule,
  /*  930 */  vRegD_rule,
  /*  931 */  vRegF_rule,
  /*  932 */  vRegD_rule,
  /*  933 */  vRegF_rule,
  /*  934 */  vRegD_rule,
  /*  935 */  vRegF_rule,
  /*  936 */  vRegF_rule,
  /*  937 */  vRegD_rule,
  /*  938 */  vRegD_rule,
  /*  939 */  vRegF_rule,
  /*  940 */  vRegF_rule,
  /*  941 */  vRegD_rule,
  /*  942 */  vRegD_rule,
  /*  943 */  vRegF_rule,
  /*  944 */  vRegD_rule,
  /*  945 */  vRegF_rule,
  /*  946 */  vRegF_rule,
  /*  947 */  vRegD_rule,
  /*  948 */  vRegD_rule,
  /*  949 */  vRegF_rule,
  /*  950 */  vRegD_rule,
  /*  951 */  iRegINoSp_rule,
  /*  952 */  iRegLNoSp_rule,
  /*  953 */  vRegD_rule,
  /*  954 */  vRegF_rule,
  /*  955 */  vRegD_rule,
  /*  956 */  iRegINoSp_rule,
  /*  957 */  iRegINoSp_rule,
  /*  958 */  iRegINoSp_rule,
  /*  959 */  iRegINoSp_rule,
  /*  960 */  iRegINoSp_rule,
  /*  961 */  iRegINoSp_rule,
  /*  962 */  iRegLNoSp_rule,
  /*  963 */  iRegLNoSp_rule,
  /*  964 */  iRegLNoSp_rule,
  /*  965 */  iRegLNoSp_rule,
  /*  966 */  iRegLNoSp_rule,
  /*  967 */  iRegLNoSp_rule,
  /*  968 */  iRegLNoSp_rule,
  /*  969 */  iRegLNoSp_rule,
  /*  970 */  iRegINoSp_rule,
  /*  971 */  iRegINoSp_rule,
  /*  972 */  iRegINoSp_rule,
  /*  973 */  vRegF_rule,
  /*  974 */  vRegD_rule,
  /*  975 */  iRegINoSp_rule,
  /*  976 */  iRegLNoSp_rule,
  /*  977 */  vRegF_rule,
  /*  978 */  vRegF_rule,
  /*  979 */  iRegINoSp_rule,
  /*  980 */  iRegLNoSp_rule,
  /*  981 */  vRegD_rule,
  /*  982 */  vRegD_rule,
  /*  983 */  stackSlotI_rule,
  /*  984 */  stackSlotF_rule,
  /*  985 */  stackSlotL_rule,
  /*  986 */  stackSlotD_rule,
  /*  987 */  Universe_rule,
  /*  988 */  Universe_rule,
  /*  989 */  Universe_rule,
  /*  990 */  Universe_rule,
  /*  991 */  iRegINoSp_rule,
  /*  992 */  iRegINoSp_rule,
  /*  993 */  iRegINoSp_rule,
  /*  994 */  iRegINoSp_rule,
  /*  995 */  iRegINoSp_rule,
  /*  996 */  iRegINoSp_rule,
  /*  997 */  0,
  /*  998 */  iRegINoSp_rule,
  /*  999 */  0,
  /* 1000 */  iRegINoSp_rule,
  /* 1001 */  Universe_rule,
  /* 1002 */  Universe_rule,
  /* 1003 */  Universe_rule,
  /* 1004 */  Universe_rule,
  /* 1005 */  Universe_rule,
  /* 1006 */  Universe_rule,
  /* 1007 */  Universe_rule,
  /* 1008 */  Universe_rule,
  /* 1009 */  Universe_rule,
  /* 1010 */  Universe_rule,
  /* 1011 */  Universe_rule,
  /* 1012 */  Universe_rule,
  /* 1013 */  Universe_rule,
  /* 1014 */  Universe_rule,
  /* 1015 */  Universe_rule,
  /* 1016 */  Universe_rule,
  /* 1017 */  Universe_rule,
  /* 1018 */  Universe_rule,
  /* 1019 */  Universe_rule,
  /* 1020 */  Universe_rule,
  /* 1021 */  Universe_rule,
  /* 1022 */  Universe_rule,
  /* 1023 */  Universe_rule,
  /* 1024 */  Universe_rule,
  /* 1025 */  Universe_rule,
  /* 1026 */  Universe_rule,
  /* 1027 */  Universe_rule,
  /* 1028 */  Universe_rule,
  /* 1029 */  iRegP_R0_rule,
  /* 1030 */  Universe_rule,
  /* 1031 */  Universe_rule,
  /* 1032 */  Universe_rule,
  /* 1033 */  iRegP_R5_rule,
  /* 1034 */  iRegI_R0_rule,
  /* 1035 */  iRegI_R0_rule,
  /* 1036 */  iRegI_R0_rule,
  /* 1037 */  iRegI_R0_rule,
  /* 1038 */  iRegI_R0_rule,
  /* 1039 */  iRegI_R0_rule,
  /* 1040 */  iRegI_R0_rule,
  /* 1041 */  iRegI_R0_rule,
  /* 1042 */  iRegI_R0_rule,
  /* 1043 */  iRegI_R0_rule,
  /* 1044 */  iRegI_R0_rule,
  /* 1045 */  iRegI_R0_rule,
  /* 1046 */  iRegI_R0_rule,
  /* 1047 */  iRegI_R0_rule,
  /* 1048 */  iRegI_R0_rule,
  /* 1049 */  iRegI_R0_rule,
  /* 1050 */  iRegI_R0_rule,
  /* 1051 */  Universe_rule,
  /* 1052 */  iRegI_R0_rule,
  /* 1053 */  vecD_rule,
  /* 1054 */  vecD_rule,
  /* 1055 */  vecX_rule,
  /* 1056 */  Universe_rule,
  /* 1057 */  Universe_rule,
  /* 1058 */  Universe_rule,
  /* 1059 */  vecD_rule,
  /* 1060 */  vecX_rule,
  /* 1061 */  vecD_rule,
  /* 1062 */  vecX_rule,
  /* 1063 */  vecD_rule,
  /* 1064 */  vecX_rule,
  /* 1065 */  vecD_rule,
  /* 1066 */  vecX_rule,
  /* 1067 */  vecD_rule,
  /* 1068 */  vecX_rule,
  /* 1069 */  vecD_rule,
  /* 1070 */  vecX_rule,
  /* 1071 */  vecX_rule,
  /* 1072 */  vecX_rule,
  /* 1073 */  vecD_rule,
  /* 1074 */  vecX_rule,
  /* 1075 */  vecX_rule,
  /* 1076 */  iRegINoSp_rule,
  /* 1077 */  iRegINoSp_rule,
  /* 1078 */  iRegINoSp_rule,
  /* 1079 */  iRegINoSp_rule,
  /* 1080 */  vRegF_rule,
  /* 1081 */  vRegF_rule,
  /* 1082 */  vRegF_rule,
  /* 1083 */  vRegF_rule,
  /* 1084 */  vRegD_rule,
  /* 1085 */  vRegD_rule,
  /* 1086 */  vRegF_rule,
  /* 1087 */  vRegF_rule,
  /* 1088 */  vRegD_rule,
  /* 1089 */  vRegF_rule,
  /* 1090 */  vRegF_rule,
  /* 1091 */  vRegD_rule,
  /* 1092 */  vecD_rule,
  /* 1093 */  vecX_rule,
  /* 1094 */  vecD_rule,
  /* 1095 */  vecX_rule,
  /* 1096 */  vecD_rule,
  /* 1097 */  vecX_rule,
  /* 1098 */  vecX_rule,
  /* 1099 */  vecD_rule,
  /* 1100 */  vecX_rule,
  /* 1101 */  vecX_rule,
  /* 1102 */  vecD_rule,
  /* 1103 */  vecX_rule,
  /* 1104 */  vecD_rule,
  /* 1105 */  vecX_rule,
  /* 1106 */  vecD_rule,
  /* 1107 */  vecX_rule,
  /* 1108 */  vecX_rule,
  /* 1109 */  vecD_rule,
  /* 1110 */  vecX_rule,
  /* 1111 */  vecX_rule,
  /* 1112 */  vecD_rule,
  /* 1113 */  vecX_rule,
  /* 1114 */  vecD_rule,
  /* 1115 */  vecX_rule,
  /* 1116 */  vecD_rule,
  /* 1117 */  vecX_rule,
  /* 1118 */  vecD_rule,
  /* 1119 */  vecX_rule,
  /* 1120 */  vecX_rule,
  /* 1121 */  vecD_rule,
  /* 1122 */  vecD_rule,
  /* 1123 */  vecX_rule,
  /* 1124 */  vecX_rule,
  /* 1125 */  vecD_rule,
  /* 1126 */  vecD_rule,
  /* 1127 */  vecX_rule,
  /* 1128 */  vecX_rule,
  /* 1129 */  vecD_rule,
  /* 1130 */  vecX_rule,
  /* 1131 */  vecX_rule,
  /* 1132 */  vecD_rule,
  /* 1133 */  vecX_rule,
  /* 1134 */  vecD_rule,
  /* 1135 */  vecX_rule,
  /* 1136 */  vecD_rule,
  /* 1137 */  vecD_rule,
  /* 1138 */  vecX_rule,
  /* 1139 */  vecX_rule,
  /* 1140 */  vecX_rule,
  /* 1141 */  vecX_rule,
  /* 1142 */  vecX_rule,
  /* 1143 */  vecD_rule,
  /* 1144 */  vecX_rule,
  /* 1145 */  vecX_rule,
  /* 1146 */  vecD_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  vecD_rule,
  /* 1149 */  vecX_rule,
  /* 1150 */  vecD_rule,
  /* 1151 */  vecX_rule,
  /* 1152 */  vecD_rule,
  /* 1153 */  vecX_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecD_rule,
  /* 1157 */  vecX_rule,
  /* 1158 */  vecD_rule,
  /* 1159 */  vecX_rule,
  /* 1160 */  vecD_rule,
  /* 1161 */  vecX_rule,
  /* 1162 */  vecD_rule,
  /* 1163 */  vecX_rule,
  /* 1164 */  vecD_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  vecD_rule,
  /* 1167 */  vecX_rule,
  /* 1168 */  vecD_rule,
  /* 1169 */  vecX_rule,
  /* 1170 */  vecD_rule,
  /* 1171 */  vecX_rule,
  /* 1172 */  vecD_rule,
  /* 1173 */  vecX_rule,
  /* 1174 */  vecD_rule,
  /* 1175 */  vecX_rule,
  /* 1176 */  vecD_rule,
  /* 1177 */  vecX_rule,
  /* 1178 */  vecD_rule,
  /* 1179 */  vecX_rule,
  /* 1180 */  vecD_rule,
  /* 1181 */  vecX_rule,
  /* 1182 */  vecD_rule,
  /* 1183 */  vecX_rule,
  /* 1184 */  vecD_rule,
  /* 1185 */  vecX_rule,
  /* 1186 */  vecD_rule,
  /* 1187 */  vecX_rule,
  /* 1188 */  vecX_rule,
  /* 1189 */  vecX_rule,
  /* 1190 */  vecX_rule,
  /* 1191 */  vecX_rule,
  /* 1192 */  vecX_rule,
  /* 1193 */  vecX_rule,
  /* 1194 */  vecD_rule,
  /* 1195 */  vecX_rule,
  /* 1196 */  vecX_rule,
  /* 1197 */  vecD_rule,
  /* 1198 */  vecX_rule,
  /* 1199 */  vecX_rule,
  /* 1200 */  vecX_rule,
  /* 1201 */  iRegINoSp_rule,
  /* 1202 */  iRegINoSp_rule,
  /* 1203 */  iRegINoSp_rule,
  /* 1204 */  iRegINoSp_rule,
  /* 1205 */  iRegNNoSp_rule,
  /* 1206 */  iRegPNoSp_rule,
  /* 1207 */  iRegNNoSp_rule,
  /* 1208 */  iRegPNoSp_rule,
  /* 1209 */  iRegINoSp_rule,
  /* 1210 */  iRegINoSp_rule,
  /* 1211 */  iRegINoSp_rule,
  /* 1212 */  iRegINoSp_rule,
  /* 1213 */  iRegPNoSp_rule,
  /* 1214 */  iRegPNoSp_rule,
  /* 1215 */  iRegPNoSp_rule,
  /* 1216 */  iRegINoSp_rule,
  /* 1217 */  iRegINoSp_rule,
  /* 1218 */  iRegINoSp_rule,
  /* 1219 */  iRegINoSp_rule,
  /* 1220 */  iRegPNoSp_rule,
  /* 1221 */  iRegPNoSp_rule,
  /* 1222 */  iRegPNoSp_rule,
  /* 1223 */  iRegPNoSp_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  /*  142 */  0,
  /*  143 */  iRegP_rule,
  /*  144 */  iRegP_rule,
  /*  145 */  iRegP_rule,
  /*  146 */  iRegP_rule,
  /*  147 */  iRegP_rule,
  /*  148 */  iRegP_rule,
  /*  149 */  iRegP_rule,
  /*  150 */  iRegP_rule,
  /*  151 */  iRegP_rule,
  /*  152 */  iRegP_rule,
  /*  153 */  iRegP_rule,
  /*  154 */  iRegP_rule,
  /*  155 */  iRegP_rule,
  /*  156 */  iRegP_rule,
  /*  157 */  iRegP_rule,
  /*  158 */  iRegP_rule,
  /*  159 */  iRegN_rule,
  /*  160 */  _DecodeN_iRegN__rule,
  /*  161 */  _DecodeN_iRegN__rule,
  /*  162 */  _DecodeN_iRegN__rule,
  /*  163 */  _DecodeN_iRegN__rule,
  /*  164 */  _DecodeN_iRegN__rule,
  /*  165 */  _DecodeN_iRegN__rule,
  /*  166 */  thread_RegP_rule,
  /*  167 */  0,
  /*  168 */  0,
  /*  169 */  0,
  /*  170 */  0,
  /*  171 */  0,
  /*  172 */  0,
  /*  173 */  0,
  /*  174 */  0,
  /*  175 */  0,
  /*  176 */  0,
  /*  177 */  iRegL_rule,
  // last operand
  /*  178 */  0,
  /*  179 */  0,
  /*  180 */  0,
  /*  181 */  0,
  /*  182 */  0,
  /*  183 */  0,
  /*  184 */  0,
  /*  185 */  0,
  /*  186 */  0,
  // last operand class
  /*  187 */  iRegI_rule,
  /*  188 */  _ConvI2L_iRegI__rule,
  /*  189 */  iRegL_rule,
  /*  190 */  iRegN_rule,
  /*  191 */  memory1_rule,
  /*  192 */  memory1_rule,
  /*  193 */  memory2_rule,
  /*  194 */  memory2_rule,
  /*  195 */  memory4_rule,
  /*  196 */  _LoadI_memory4__rule,
  /*  197 */  indirect_rule,
  /*  198 */  indirect_rule,
  /*  199 */  indirect_rule,
  /*  200 */  indirect_rule,
  /*  201 */  indirect_rule,
  /*  202 */  _LoadI_indirect__rule,
  /*  203 */  memory8_rule,
  /*  204 */  iRegP_rule,
  /*  205 */  _DecodeN_iRegN__rule,
  /*  206 */  iRegP_rule,
  /*  207 */  iRegLNoSp_rule,
  /*  208 */  iRegINoSp_rule,
  /*  209 */  iRegNNoSp_rule,
  /*  210 */  iRegI_rule,
  /*  211 */  iRegL_rule,
  /*  212 */  iRegN_rule,
  /*  213 */  cmpOp_rule,
  /*  214 */  iRegIorL2I_rule,
  /*  215 */  cmpOpU_rule,
  /*  216 */  immI0_rule,
  /*  217 */  iRegIorL2I_rule,
  /*  218 */  immI_1_rule,
  /*  219 */  iRegL_rule,
  /*  220 */  immL0_rule,
  /*  221 */  iRegP_rule,
  /*  222 */  immP0_rule,
  /*  223 */  iRegN_rule,
  /*  224 */  immN0_rule,
  /*  225 */  vRegF_rule,
  /*  226 */  vRegD_rule,
  /*  227 */  iRegL_rule,
  /*  228 */  iRegIorL2I_rule,
  /*  229 */  _ConvI2L_iRegIorL2I__rule,
  /*  230 */  iRegIorL2I_rule,
  /*  231 */  immI0_rule,
  /*  232 */  iRegL_rule,
  /*  233 */  immL0_rule,
  /*  234 */  _ConvI2L_iRegIorL2I__rule,
  /*  235 */  immL0_rule,
  /*  236 */  iRegIorL2I_rule,
  /*  237 */  iRegL_rule,
  /*  238 */  iRegIorL2I_rule,
  /*  239 */  iRegL_rule,
  /*  240 */  iRegIorL2I_rule,
  /*  241 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  242 */  iRegL_rule,
  /*  243 */  _URShiftL_iRegL_immI_rule,
  /*  244 */  iRegIorL2I_rule,
  /*  245 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  246 */  iRegL_rule,
  /*  247 */  _RShiftL_iRegL_immI_rule,
  /*  248 */  iRegIorL2I_rule,
  /*  249 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  250 */  iRegL_rule,
  /*  251 */  _LShiftL_iRegL_immI_rule,
  /*  252 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  253 */  iRegIorL2I_rule,
  /*  254 */  _URShiftL_iRegL_immI_rule,
  /*  255 */  iRegL_rule,
  /*  256 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  257 */  iRegIorL2I_rule,
  /*  258 */  _RShiftL_iRegL_immI_rule,
  /*  259 */  iRegL_rule,
  /*  260 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  261 */  iRegIorL2I_rule,
  /*  262 */  _LShiftL_iRegL_immI_rule,
  /*  263 */  iRegL_rule,
  /*  264 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  265 */  iRegIorL2I_rule,
  /*  266 */  iRegL_rule,
  /*  267 */  _AndI_iRegIorL2I_immI_bitmask_rule,
  /*  268 */  iRegL_rule,
  /*  269 */  immI_64_rule,
  /*  270 */  iRegL_rule,
  /*  271 */  immI0_rule,
  /*  272 */  iRegL_rule,
  /*  273 */  iRegI_rule,
  /*  274 */  immI_32_rule,
  /*  275 */  iRegI_rule,
  /*  276 */  iRegI_rule,
  /*  277 */  iRegL_rule,
  /*  278 */  iRegL_rule,
  /*  279 */  iRegL_rule,
  /*  280 */  iRegI_rule,
  /*  281 */  iRegI_rule,
  /*  282 */  iRegI_rule,
  /*  283 */  iRegIorL2I_rule,
  /*  284 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  285 */  iRegIorL2I_rule,
  /*  286 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  287 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  288 */  iRegL_rule,
  /*  289 */  _LShiftL_iRegL_immI_48_rule,
  /*  290 */  iRegL_rule,
  /*  291 */  _LShiftL_iRegL_immI_32_rule,
  /*  292 */  iRegL_rule,
  /*  293 */  _LShiftL_iRegL_immI_56_rule,
  /*  294 */  _LShiftL_iRegL_immI_56_rule,
  /*  295 */  iRegIorL2I_rule,
  /*  296 */  iRegIorL2I_rule,
  /*  297 */  iRegL_rule,
  /*  298 */  iRegL_rule,
  /*  299 */  iRegL_rule,
  /*  300 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  301 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  302 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  303 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  304 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  305 */  _ConvI2L_iRegIorL2I__rule,
  /*  306 */  _AndL_iRegL_immL_255_rule,
  /*  307 */  _AndL_iRegL_immL_65535_rule,
  /*  308 */  _AndL_iRegL_immL_4294967295_rule,
  /*  309 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  310 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  311 */  vRegF_rule,
  /*  312 */  _NegF_vRegF__rule,
  /*  313 */  vRegF_rule,
  /*  314 */  vRegD_rule,
  /*  315 */  _NegD_vRegD__rule,
  /*  316 */  vRegD_rule,
  /*  317 */  iRegIorL2I_rule,
  /*  318 */  iRegL_rule,
  /*  319 */  iRegIorL2I_rule,
  /*  320 */  iRegL_rule,
  /*  321 */  iRegP_rule,
  /*  322 */  iRegN_rule,
  /*  323 */  _DecodeN_iRegN__rule,
  /*  324 */  iRegIorL2I_rule,
  /*  325 */  iRegL_rule,
  /*  326 */  iRegL_rule,
  /*  327 */  _AndL_iRegL_immL_rule,
  /*  328 */  iRegIorL2I_rule,
  /*  329 */  _AndI_iRegIorL2I_immI_rule,
  /*  330 */  iRegL_rule,
  /*  331 */  iRegIorL2I_rule,
  /*  332 */  iRegP_R4_rule,
  /*  333 */  iRegP_R1_rule,
  /*  334 */  iRegP_R3_rule,
  /*  335 */  iRegP_R1_rule,
  /*  336 */  iRegP_R3_rule,
  /*  337 */  iRegP_R3_rule,
  /*  338 */  iRegP_R3_rule,
  /*  339 */  iRegP_R1_rule,
  /*  340 */  iRegP_R1_rule,
  /*  341 */  vecD_rule,
  /*  342 */  vecX_rule,
  /*  343 */  vecD_rule,
  /*  344 */  vecX_rule,
  /*  345 */  vecD_rule,
  /*  346 */  vecX_rule,
  /*  347 */  vecD_rule,
  /*  348 */  _NegVF_vecD__rule,
  /*  349 */  vecD_rule,
  /*  350 */  vecX_rule,
  /*  351 */  _NegVF_vecX__rule,
  /*  352 */  vecX_rule,
  /*  353 */  vecX_rule,
  /*  354 */  _NegVD_vecX__rule,
  /*  355 */  vecX_rule,
  /*  356 */  immI_rule,
  /*  357 */  immI_rule,
  // last internally defined operand
  /*  358 */  immI_rule,
  /*  359 */  immL_rule,
  /*  360 */  immP_rule,
  /*  361 */  immP0_rule,
  /*  362 */  immP_1_rule,
  /*  363 */  immByteMapBase_rule,
  /*  364 */  immN_rule,
  /*  365 */  immN0_rule,
  /*  366 */  immNKlass_rule,
  /*  367 */  immFPacked_rule,
  /*  368 */  immF_rule,
  /*  369 */  immDPacked_rule,
  /*  370 */  immD_rule,
  /*  371 */  iRegIorL2I_rule,
  /*  372 */  iRegL_rule,
  /*  373 */  iRegIorL2I_rule,
  /*  374 */  iRegL_rule,
  /*  375 */  iRegL_rule,
  /*  376 */  iRegP_rule,
  /*  377 */  iRegPNoSp_rule,
  /*  378 */  iRegPNoSp_rule,
  /*  379 */  iRegI_rule,
  /*  380 */  memory8_rule,
  /*  381 */  indirect_rule,
  /*  382 */  indirect_rule,
  /*  383 */  vRegF_rule,
  /*  384 */  vRegD_rule,
  /*  385 */  vRegF_rule,
  /*  386 */  vRegD_rule,
  /*  387 */  stackSlotF_rule,
  /*  388 */  stackSlotI_rule,
  /*  389 */  stackSlotD_rule,
  /*  390 */  stackSlotL_rule,
  /*  391 */  vRegF_rule,
  /*  392 */  iRegI_rule,
  /*  393 */  vRegD_rule,
  /*  394 */  iRegL_rule,
  /*  395 */  iRegIorL2I_rule,
  /*  396 */  iRegIorL2I_rule,
  /*  397 */  iRegL_rule,
  /*  398 */  iRegL_rule,
  /*  399 */  iRegIorL2I_rule,
  /*  400 */  iRegIorL2I_rule,
  /*  401 */  iRegL_rule,
  /*  402 */  iRegL_rule,
  /*  403 */  immI0_rule,
  /*  404 */  immI0_rule,
  /*  405 */  iRegIorL2I_rule,
  /*  406 */  iRegL_rule,
  /*  407 */  iRegI_rule,
  /*  408 */  iRegI_rule,
  /*  409 */  iRegI_rule,
  /*  410 */  iRegI_rule,
  /*  411 */  iRegI_rule,
  /*  412 */  iRegI_rule,
  /*  413 */  iRegI_rule,
  /*  414 */  iRegI_rule,
  /*  415 */  iRegL_rule,
  /*  416 */  iRegL_rule,
  /*  417 */  iRegL_rule,
  /*  418 */  iRegL_rule,
  /*  419 */  iRegL_rule,
  /*  420 */  iRegL_rule,
  /*  421 */  iRegL_rule,
  /*  422 */  iRegL_rule,
  /*  423 */  iRegP_rule,
  /*  424 */  iRegN_rule,
  /*  425 */  iRegP_rule,
  /*  426 */  iRegN_rule,
  /*  427 */  vRegF_rule,
  /*  428 */  vRegF_rule,
  /*  429 */  vRegD_rule,
  /*  430 */  vRegD_rule,
  /*  431 */  _AndL_iRegL_immL_rule,
  /*  432 */  _AndI_iRegIorL2I_immI_rule,
  /*  433 */  _AndL_iRegL_iRegL_rule,
  /*  434 */  _AndI_iRegIorL2I_iRegIorL2I_rule,
  /*  435 */  iRegP_rule,
  /*  436 */  iRegP_rule,
  /*  437 */  _PartialSubtypeCheck_iRegP_R4_iRegP_R0_rule,
  /*  438 */  0,
  /*  439 */  vecD_rule,
  /*  440 */  vecX_rule,
  /*  441 */  vecX_rule,
  /*  442 */  vecD_rule,
  /*  443 */  vecX_rule,
  /*  444 */  vecD_rule,
  /*  445 */  vecX_rule,
  /*  446 */  vecD_rule,
  /*  447 */  vecX_rule,
  /*  448 */  vecX_rule,
  /*  449 */  vecD_rule,
  /*  450 */  vecX_rule,
  /*  451 */  vecX_rule,
  /*  452 */  vecD_rule,
  /*  453 */  vecX_rule,
  /*  454 */  vecX_rule,
  /*  455 */  iRegIorL2I_rule,
  /*  456 */  iRegIorL2I_rule,
  /*  457 */  iRegIorL2I_rule,
  /*  458 */  iRegIorL2I_rule,
  /*  459 */  vecX_rule,
  /*  460 */  vecD_rule,
  /*  461 */  memory1_rule,
  /*  462 */  _LoadB_memory1__rule,
  /*  463 */  memory1_rule,
  /*  464 */  _LoadUB_memory1__rule,
  /*  465 */  memory2_rule,
  /*  466 */  _LoadS_memory2__rule,
  /*  467 */  memory2_rule,
  /*  468 */  _LoadUS_memory2__rule,
  /*  469 */  memory4_rule,
  /*  470 */  _LoadI_memory4__rule,
  /*  471 */  _ConvI2L__LoadI_memory4___rule,
  /*  472 */  memory8_rule,
  /*  473 */  memory4_rule,
  /*  474 */  memory8_rule,
  /*  475 */  memory4_rule,
  /*  476 */  memory8_rule,
  /*  477 */  memory4_rule,
  /*  478 */  memory4_rule,
  /*  479 */  memory8_rule,
  /*  480 */  memory1_rule,
  /*  481 */  memory1_rule,
  /*  482 */  memory1_rule,
  /*  483 */  memory1_rule,
  /*  484 */  memory2_rule,
  /*  485 */  memory2_rule,
  /*  486 */  memory4_rule,
  /*  487 */  memory4_rule,
  /*  488 */  memory8_rule,
  /*  489 */  memory8_rule,
  /*  490 */  memory8_rule,
  /*  491 */  memory8_rule,
  /*  492 */  memory4_rule,
  /*  493 */  memory4_rule,
  /*  494 */  memory4_rule,
  /*  495 */  memory8_rule,
  /*  496 */  memory4_rule,
  /*  497 */  memory8_rule,
  /*  498 */  indirect_rule,
  /*  499 */  _LoadB_indirect__rule,
  /*  500 */  indirect_rule,
  /*  501 */  _LoadUB_indirect__rule,
  /*  502 */  indirect_rule,
  /*  503 */  indirect_rule,
  /*  504 */  _LoadUS_indirect__rule,
  /*  505 */  _LoadS_indirect__rule,
  /*  506 */  indirect_rule,
  /*  507 */  _ConvI2L__LoadI_indirect___rule,
  /*  508 */  indirect_rule,
  /*  509 */  indirect_rule,
  /*  510 */  indirect_rule,
  /*  511 */  indirect_rule,
  /*  512 */  indirect_rule,
  /*  513 */  indirect_rule,
  /*  514 */  indirect_rule,
  /*  515 */  indirect_rule,
  /*  516 */  indirect_rule,
  /*  517 */  indirect_rule,
  /*  518 */  indirect_rule,
  /*  519 */  indirect_rule,
  /*  520 */  indirect_rule,
  /*  521 */  indirect_rule,
  /*  522 */  0,
  /*  523 */  0,
  /*  524 */  iRegIorL2I_rule,
  /*  525 */  iRegL_rule,
  /*  526 */  iRegIorL2I_rule,
  /*  527 */  iRegIorL2I_rule,
  /*  528 */  iRegIorL2I_rule,
  /*  529 */  _LoadI_memory4__rule,
  /*  530 */  iRegL_rule,
  /*  531 */  _LoadL_memory8__rule,
  /*  532 */  0,
  /*  533 */  0,
  /*  534 */  0,
  /*  535 */  0,
  /*  536 */  0,
  /*  537 */  0,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  0,
  /*  542 */  0,
  /*  543 */  _CastP2X_iRegP__rule,
  /*  544 */  _CastP2X__DecodeN_iRegN___rule,
  /*  545 */  iRegP_rule,
  /*  546 */  iRegP_rule,
  /*  547 */  iRegN_rule,
  /*  548 */  iRegN_rule,
  /*  549 */  iRegP_rule,
  /*  550 */  iRegN_rule,
  /*  551 */  indirect_rule,
  /*  552 */  indirect_rule,
  /*  553 */  indirect_rule,
  /*  554 */  indirect_rule,
  /*  555 */  indirect_rule,
  /*  556 */  indirect_rule,
  /*  557 */  indirect_rule,
  /*  558 */  indirect_rule,
  /*  559 */  indirect_rule,
  /*  560 */  indirect_rule,
  /*  561 */  indirect_rule,
  /*  562 */  indirect_rule,
  /*  563 */  indirect_rule,
  /*  564 */  indirect_rule,
  /*  565 */  indirect_rule,
  /*  566 */  indirect_rule,
  /*  567 */  indirect_rule,
  /*  568 */  indirect_rule,
  /*  569 */  indirect_rule,
  /*  570 */  indirect_rule,
  /*  571 */  indirect_rule,
  /*  572 */  indirect_rule,
  /*  573 */  indirect_rule,
  /*  574 */  indirect_rule,
  /*  575 */  indirect_rule,
  /*  576 */  indirect_rule,
  /*  577 */  indirect_rule,
  /*  578 */  indirect_rule,
  /*  579 */  indirect_rule,
  /*  580 */  indirect_rule,
  /*  581 */  indirect_rule,
  /*  582 */  indirect_rule,
  /*  583 */  indirect_rule,
  /*  584 */  indirect_rule,
  /*  585 */  indirect_rule,
  /*  586 */  indirect_rule,
  /*  587 */  indirect_rule,
  /*  588 */  indirect_rule,
  /*  589 */  indirect_rule,
  /*  590 */  indirect_rule,
  /*  591 */  indirect_rule,
  /*  592 */  indirect_rule,
  /*  593 */  indirect_rule,
  /*  594 */  indirect_rule,
  /*  595 */  indirect_rule,
  /*  596 */  indirect_rule,
  /*  597 */  indirect_rule,
  /*  598 */  indirect_rule,
  /*  599 */  indirect_rule,
  /*  600 */  indirect_rule,
  /*  601 */  indirect_rule,
  /*  602 */  indirect_rule,
  /*  603 */  indirect_rule,
  /*  604 */  indirect_rule,
  /*  605 */  indirect_rule,
  /*  606 */  indirect_rule,
  /*  607 */  indirect_rule,
  /*  608 */  indirect_rule,
  /*  609 */  indirect_rule,
  /*  610 */  indirect_rule,
  /*  611 */  indirect_rule,
  /*  612 */  iRegL_rule,
  /*  613 */  iRegL_rule,
  /*  614 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  615 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  616 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  617 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  618 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  619 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  620 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  621 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  622 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  623 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  624 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  625 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  626 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  627 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  628 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  629 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  630 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  631 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  632 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  633 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  634 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  635 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  636 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  637 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  638 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  639 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  640 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  641 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  642 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  643 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  644 */  iRegIorL2I_rule,
  /*  645 */  iRegIorL2I_rule,
  /*  646 */  _ConvL2I_iRegL__rule,
  /*  647 */  iRegP_rule,
  /*  648 */  iRegP_rule,
  /*  649 */  iRegP_rule,
  /*  650 */  iRegP_rule,
  /*  651 */  _ConvI2L_iRegIorL2I__rule,
  /*  652 */  iRegP_rule,
  /*  653 */  iRegL_rule,
  /*  654 */  iRegL_rule,
  /*  655 */  iRegIorL2I_rule,
  /*  656 */  iRegIorL2I_rule,
  /*  657 */  iRegL_rule,
  /*  658 */  iRegL_rule,
  /*  659 */  immI0_rule,
  /*  660 */  immL0_rule,
  /*  661 */  iRegIorL2I_rule,
  /*  662 */  _ConvI2L_iRegIorL2I__rule,
  /*  663 */  _ConvI2L_iRegIorL2I__rule,
  /*  664 */  iRegL_rule,
  /*  665 */  iRegL_rule,
  /*  666 */  iRegIorL2I_rule,
  /*  667 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  668 */  iRegIorL2I_rule,
  /*  669 */  _SubI_immI0_iRegIorL2I_rule,
  /*  670 */  iRegIorL2I_rule,
  /*  671 */  iRegIorL2I_rule,
  /*  672 */  _SubI_immI0_iRegIorL2I_rule,
  /*  673 */  iRegL_rule,
  /*  674 */  _MulL_iRegL_iRegL_rule,
  /*  675 */  iRegL_rule,
  /*  676 */  _SubL_immL0_iRegL_rule,
  /*  677 */  iRegL_rule,
  /*  678 */  iRegL_rule,
  /*  679 */  _SubL_immL0_iRegL_rule,
  /*  680 */  iRegLNoSp_rule,
  /*  681 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  682 */  iRegLNoSp_rule,
  /*  683 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  684 */  iRegLNoSp_rule,
  /*  685 */  iRegLNoSp_rule,
  /*  686 */  _SubL_immL0__ConvI2L_iRegIorL2I__rule,
  /*  687 */  _ConvI2L_iRegIorL2I__rule,
  /*  688 */  _ConvI2L_iRegIorL2I__rule,
  /*  689 */  _SubL_immL0__ConvI2L_iRegIorL2I__rule,
  /*  690 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  691 */  iRegIorL2I_rule,
  /*  692 */  iRegL_rule,
  /*  693 */  iRegIorL2I_rule,
  /*  694 */  iRegL_rule,
  /*  695 */  iRegIorL2I_rule,
  /*  696 */  iRegIorL2I_rule,
  /*  697 */  iRegIorL2I_rule,
  /*  698 */  iRegIorL2I_rule,
  /*  699 */  iRegIorL2I_rule,
  /*  700 */  iRegIorL2I_rule,
  /*  701 */  iRegL_rule,
  /*  702 */  iRegL_rule,
  /*  703 */  iRegL_rule,
  /*  704 */  iRegL_rule,
  /*  705 */  _CastP2X_iRegP__rule,
  /*  706 */  iRegL_rule,
  /*  707 */  iRegL_rule,
  /*  708 */  iRegL_rule,
  /*  709 */  iRegIorL2I_rule,
  /*  710 */  iRegIorL2I_rule,
  /*  711 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  712 */  iRegL_rule,
  /*  713 */  _XorL_iRegL_immL_M1_rule,
  /*  714 */  iRegIorL2I_rule,
  /*  715 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  716 */  iRegL_rule,
  /*  717 */  _XorL_iRegL_immL_M1_rule,
  /*  718 */  immI_M1_rule,
  /*  719 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  720 */  immL_M1_rule,
  /*  721 */  _XorL_iRegL_iRegL_rule,
  /*  722 */  iRegIorL2I_rule,
  /*  723 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  724 */  iRegL_rule,
  /*  725 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  726 */  iRegIorL2I_rule,
  /*  727 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  728 */  iRegL_rule,
  /*  729 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  730 */  iRegIorL2I_rule,
  /*  731 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  732 */  iRegL_rule,
  /*  733 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  734 */  immI_M1_rule,
  /*  735 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  736 */  immI_M1_rule,
  /*  737 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  738 */  immL_M1_rule,
  /*  739 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  740 */  immL_M1_rule,
  /*  741 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  742 */  immI_M1_rule,
  /*  743 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  744 */  immI_M1_rule,
  /*  745 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  746 */  immL_M1_rule,
  /*  747 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  748 */  immL_M1_rule,
  /*  749 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  750 */  immI_M1_rule,
  /*  751 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  752 */  immI_M1_rule,
  /*  753 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  754 */  immL_M1_rule,
  /*  755 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  756 */  immL_M1_rule,
  /*  757 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  758 */  iRegIorL2I_rule,
  /*  759 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  760 */  iRegL_rule,
  /*  761 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  762 */  iRegIorL2I_rule,
  /*  763 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  764 */  iRegL_rule,
  /*  765 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  766 */  iRegIorL2I_rule,
  /*  767 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  768 */  iRegL_rule,
  /*  769 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  770 */  iRegIorL2I_rule,
  /*  771 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  772 */  iRegL_rule,
  /*  773 */  _URShiftL_iRegL_immI_rule,
  /*  774 */  iRegIorL2I_rule,
  /*  775 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  776 */  iRegL_rule,
  /*  777 */  _RShiftL_iRegL_immI_rule,
  /*  778 */  iRegIorL2I_rule,
  /*  779 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  780 */  iRegL_rule,
  /*  781 */  _LShiftL_iRegL_immI_rule,
  /*  782 */  iRegIorL2I_rule,
  /*  783 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  784 */  iRegL_rule,
  /*  785 */  _URShiftL_iRegL_immI_rule,
  /*  786 */  iRegIorL2I_rule,
  /*  787 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  788 */  iRegL_rule,
  /*  789 */  _RShiftL_iRegL_immI_rule,
  /*  790 */  iRegIorL2I_rule,
  /*  791 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  792 */  iRegL_rule,
  /*  793 */  _LShiftL_iRegL_immI_rule,
  /*  794 */  iRegIorL2I_rule,
  /*  795 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  796 */  iRegL_rule,
  /*  797 */  _URShiftL_iRegL_immI_rule,
  /*  798 */  iRegIorL2I_rule,
  /*  799 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  800 */  iRegL_rule,
  /*  801 */  _RShiftL_iRegL_immI_rule,
  /*  802 */  iRegIorL2I_rule,
  /*  803 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  804 */  iRegL_rule,
  /*  805 */  _LShiftL_iRegL_immI_rule,
  /*  806 */  iRegIorL2I_rule,
  /*  807 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  808 */  iRegL_rule,
  /*  809 */  _URShiftL_iRegL_immI_rule,
  /*  810 */  iRegIorL2I_rule,
  /*  811 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  812 */  iRegL_rule,
  /*  813 */  _RShiftL_iRegL_immI_rule,
  /*  814 */  iRegIorL2I_rule,
  /*  815 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  816 */  iRegL_rule,
  /*  817 */  _LShiftL_iRegL_immI_rule,
  /*  818 */  iRegIorL2I_rule,
  /*  819 */  iRegL_rule,
  /*  820 */  iRegIorL2I_rule,
  /*  821 */  iRegL_rule,
  /*  822 */  iRegIorL2I_rule,
  /*  823 */  iRegL_rule,
  /*  824 */  _LShiftL_iRegL_immI_rule,
  /*  825 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  826 */  _LShiftL_iRegL_immI_rule,
  /*  827 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  828 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  829 */  _URShiftL_iRegL_immI_rule,
  /*  830 */  _AndI__URShiftI_iRegIorL2I_immI_immI_bitmask_rule,
  /*  831 */  _AndI_iRegIorL2I_immI_bitmask_rule,
  /*  832 */  _AndL_iRegL_immL_bitmask_rule,
  /*  833 */  _ConvI2L__AndI_iRegIorL2I_immI_bitmask__rule,
  /*  834 */  _LShiftL_iRegL_immI_rule,
  /*  835 */  _URShiftL_iRegL_immI_rule,
  /*  836 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  837 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  838 */  _LShiftL_iRegL_immI_rule,
  /*  839 */  _URShiftL_iRegL_immI_rule,
  /*  840 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  841 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  842 */  0,
  /*  843 */  0,
  /*  844 */  _LShiftL_iRegL_iRegI_rule,
  /*  845 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  846 */  _LShiftL_iRegL_iRegI_rule,
  /*  847 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  848 */  _LShiftI_iRegI_iRegI_rule,
  /*  849 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  850 */  _LShiftI_iRegI_iRegI_rule,
  /*  851 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  852 */  0,
  /*  853 */  0,
  /*  854 */  _URShiftL_iRegL_iRegI_rule,
  /*  855 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  856 */  _URShiftL_iRegL_iRegI_rule,
  /*  857 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  858 */  _URShiftI_iRegI_iRegI_rule,
  /*  859 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  860 */  _URShiftI_iRegI_iRegI_rule,
  /*  861 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  862 */  iRegL_rule,
  /*  863 */  _ConvI2L_iRegIorL2I__rule,
  /*  864 */  iRegL_rule,
  /*  865 */  iRegIorL2I_rule,
  /*  866 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  867 */  iRegIorL2I_rule,
  /*  868 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  869 */  iRegIorL2I_rule,
  /*  870 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  871 */  iRegL_rule,
  /*  872 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  873 */  iRegL_rule,
  /*  874 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  875 */  iRegL_rule,
  /*  876 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  877 */  iRegL_rule,
  /*  878 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  879 */  iRegIorL2I_rule,
  /*  880 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  881 */  iRegIorL2I_rule,
  /*  882 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  883 */  iRegL_rule,
  /*  884 */  _AndL_iRegL_immL_255_rule,
  /*  885 */  iRegL_rule,
  /*  886 */  _AndL_iRegL_immL_65535_rule,
  /*  887 */  iRegL_rule,
  /*  888 */  _AndL_iRegL_immL_4294967295_rule,
  /*  889 */  iRegIorL2I_rule,
  /*  890 */  iRegIorL2I_rule,
  /*  891 */  iRegL_rule,
  /*  892 */  iRegL_rule,
  /*  893 */  iRegL_rule,
  /*  894 */  iRegL_rule,
  /*  895 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_56_immI_56_immIExt_rule,
  /*  896 */  iRegL_rule,
  /*  897 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_48_immI_48_immIExt_rule,
  /*  898 */  iRegL_rule,
  /*  899 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_32_immI_32_immIExt_rule,
  /*  900 */  iRegL_rule,
  /*  901 */  iRegL_rule,
  /*  902 */  iRegL_rule,
  /*  903 */  iRegIorL2I_rule,
  /*  904 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_immIExt_rule,
  /*  905 */  iRegIorL2I_rule,
  /*  906 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_immIExt_rule,
  /*  907 */  iRegIorL2I_rule,
  /*  908 */  iRegIorL2I_rule,
  /*  909 */  iRegL_rule,
  /*  910 */  _LShiftL__ConvI2L_iRegIorL2I__immIExt_rule,
  /*  911 */  iRegL_rule,
  /*  912 */  iRegL_rule,
  /*  913 */  _LShiftL__AndL_iRegL_immL_255_immIExt_rule,
  /*  914 */  iRegL_rule,
  /*  915 */  _LShiftL__AndL_iRegL_immL_65535_immIExt_rule,
  /*  916 */  iRegL_rule,
  /*  917 */  _LShiftL__AndL_iRegL_immL_4294967295_immIExt_rule,
  /*  918 */  iRegL_rule,
  /*  919 */  iRegL_rule,
  /*  920 */  iRegL_rule,
  /*  921 */  iRegIorL2I_rule,
  /*  922 */  _LShiftI__AndI_iRegIorL2I_immI_255_immIExt_rule,
  /*  923 */  iRegIorL2I_rule,
  /*  924 */  _LShiftI__AndI_iRegIorL2I_immI_65535_immIExt_rule,
  /*  925 */  iRegIorL2I_rule,
  /*  926 */  iRegIorL2I_rule,
  /*  927 */  vRegF_rule,
  /*  928 */  vRegD_rule,
  /*  929 */  vRegF_rule,
  /*  930 */  vRegD_rule,
  /*  931 */  vRegF_rule,
  /*  932 */  vRegD_rule,
  /*  933 */  vRegF_rule,
  /*  934 */  vRegD_rule,
  /*  935 */  vRegF_rule,
  /*  936 */  vRegF_rule,
  /*  937 */  vRegD_rule,
  /*  938 */  vRegD_rule,
  /*  939 */  _NegF_vRegF__rule,
  /*  940 */  _NegF_vRegF__rule,
  /*  941 */  _NegD_vRegD__rule,
  /*  942 */  _NegD_vRegD__rule,
  /*  943 */  _NegF_vRegF__rule,
  /*  944 */  _NegD_vRegD__rule,
  /*  945 */  vRegF_rule,
  /*  946 */  vRegF_rule,
  /*  947 */  vRegD_rule,
  /*  948 */  vRegD_rule,
  /*  949 */  vRegF_rule,
  /*  950 */  vRegD_rule,
  /*  951 */  iRegIorL2I_rule,
  /*  952 */  iRegL_rule,
  /*  953 */  vRegD_rule,
  /*  954 */  vRegF_rule,
  /*  955 */  vRegD_rule,
  /*  956 */  iRegIorL2I_rule,
  /*  957 */  iRegIorL2I_rule,
  /*  958 */  iRegIorL2I_rule,
  /*  959 */  iRegIorL2I_rule,
  /*  960 */  iRegIorL2I_rule,
  /*  961 */  iRegIorL2I_rule,
  /*  962 */  iRegL_rule,
  /*  963 */  iRegL_rule,
  /*  964 */  iRegL_rule,
  /*  965 */  iRegL_rule,
  /*  966 */  iRegL_rule,
  /*  967 */  iRegL_rule,
  /*  968 */  iRegIorL2I_rule,
  /*  969 */  _ConvI2L_iRegIorL2I__rule,
  /*  970 */  iRegL_rule,
  /*  971 */  iRegIorL2I_rule,
  /*  972 */  iRegP_rule,
  /*  973 */  vRegD_rule,
  /*  974 */  vRegF_rule,
  /*  975 */  vRegF_rule,
  /*  976 */  vRegF_rule,
  /*  977 */  iRegIorL2I_rule,
  /*  978 */  iRegL_rule,
  /*  979 */  vRegD_rule,
  /*  980 */  vRegD_rule,
  /*  981 */  iRegIorL2I_rule,
  /*  982 */  iRegL_rule,
  /*  983 */  vRegF_rule,
  /*  984 */  iRegI_rule,
  /*  985 */  vRegD_rule,
  /*  986 */  iRegL_rule,
  /*  987 */  iRegL_R11_rule,
  /*  988 */  immL_rule,
  /*  989 */  cmpOp_rule,
  /*  990 */  cmpOp_rule,
  /*  991 */  vRegF_rule,
  /*  992 */  vRegD_rule,
  /*  993 */  vRegF_rule,
  /*  994 */  vRegD_rule,
  /*  995 */  iRegIorL2I_rule,
  /*  996 */  iRegIorL2I_rule,
  /*  997 */  0,
  /*  998 */  iRegI_rule,
  /*  999 */  0,
  /* 1000 */  iRegI_rule,
  /* 1001 */  0,
  /* 1002 */  cmpOp_rule,
  /* 1003 */  cmpOpU_rule,
  /* 1004 */  cmpOpEqNe_rule,
  /* 1005 */  cmpOpEqNe_rule,
  /* 1006 */  cmpOpEqNe_rule,
  /* 1007 */  cmpOpEqNe_rule,
  /* 1008 */  cmpOpEqNe_rule,
  /* 1009 */  cmpOpUEqNeLtGe_rule,
  /* 1010 */  cmpOpUEqNeLtGe_rule,
  /* 1011 */  cmpOpLtGe_rule,
  /* 1012 */  cmpOpLtGe_rule,
  /* 1013 */  cmpOpEqNe_rule,
  /* 1014 */  cmpOpEqNe_rule,
  /* 1015 */  cmpOpLtGe_rule,
  /* 1016 */  cmpOpLtGe_rule,
  /* 1017 */  cmpOpEqNe_rule,
  /* 1018 */  cmpOpEqNe_rule,
  /* 1019 */  cmpOp_rule,
  /* 1020 */  cmpOpU_rule,
  /* 1021 */  iRegP_rule,
  /* 1022 */  0,
  /* 1023 */  0,
  /* 1024 */  0,
  /* 1025 */  0,
  /* 1026 */  0,
  /* 1027 */  iRegPNoSp_rule,
  /* 1028 */  iRegPNoSp_rule,
  /* 1029 */  0,
  /* 1030 */  0,
  /* 1031 */  0,
  /* 1032 */  0,
  /* 1033 */  iRegP_R4_rule,
  /* 1034 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1035 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1036 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1037 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1038 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1039 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1040 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1041 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1042 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1043 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /* 1044 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1045 */  _Binary_iRegP_R1_iRegP_R3_rule,
  /* 1046 */  _Binary_iRegP_R1_iRegP_R3_rule,
  /* 1047 */  iRegP_R1_rule,
  /* 1048 */  iRegP_R1_rule,
  /* 1049 */  iRegP_R1_rule,
  /* 1050 */  iRegP_R2_rule,
  /* 1051 */  iRegP_R0_rule,
  /* 1052 */  iRegP_R2_rule,
  /* 1053 */  vmem4_rule,
  /* 1054 */  vmem8_rule,
  /* 1055 */  vmem16_rule,
  /* 1056 */  vmem4_rule,
  /* 1057 */  vmem8_rule,
  /* 1058 */  vmem16_rule,
  /* 1059 */  iRegIorL2I_rule,
  /* 1060 */  iRegIorL2I_rule,
  /* 1061 */  immI_rule,
  /* 1062 */  immI_rule,
  /* 1063 */  iRegIorL2I_rule,
  /* 1064 */  iRegIorL2I_rule,
  /* 1065 */  immI_rule,
  /* 1066 */  immI_rule,
  /* 1067 */  iRegIorL2I_rule,
  /* 1068 */  iRegIorL2I_rule,
  /* 1069 */  immI_rule,
  /* 1070 */  immI_rule,
  /* 1071 */  iRegL_rule,
  /* 1072 */  immI0_rule,
  /* 1073 */  vRegF_rule,
  /* 1074 */  vRegF_rule,
  /* 1075 */  vRegD_rule,
  /* 1076 */  iRegIorL2I_rule,
  /* 1077 */  iRegIorL2I_rule,
  /* 1078 */  iRegIorL2I_rule,
  /* 1079 */  iRegIorL2I_rule,
  /* 1080 */  vRegF_rule,
  /* 1081 */  vRegF_rule,
  /* 1082 */  vRegF_rule,
  /* 1083 */  vRegF_rule,
  /* 1084 */  vRegD_rule,
  /* 1085 */  vRegD_rule,
  /* 1086 */  vRegF_rule,
  /* 1087 */  vRegF_rule,
  /* 1088 */  vRegD_rule,
  /* 1089 */  vRegF_rule,
  /* 1090 */  vRegF_rule,
  /* 1091 */  vRegD_rule,
  /* 1092 */  vecD_rule,
  /* 1093 */  vecX_rule,
  /* 1094 */  vecD_rule,
  /* 1095 */  vecX_rule,
  /* 1096 */  vecD_rule,
  /* 1097 */  vecX_rule,
  /* 1098 */  vecX_rule,
  /* 1099 */  vecD_rule,
  /* 1100 */  vecX_rule,
  /* 1101 */  vecX_rule,
  /* 1102 */  vecD_rule,
  /* 1103 */  vecX_rule,
  /* 1104 */  vecD_rule,
  /* 1105 */  vecX_rule,
  /* 1106 */  vecD_rule,
  /* 1107 */  vecX_rule,
  /* 1108 */  vecX_rule,
  /* 1109 */  vecD_rule,
  /* 1110 */  vecX_rule,
  /* 1111 */  vecX_rule,
  /* 1112 */  vecD_rule,
  /* 1113 */  vecX_rule,
  /* 1114 */  vecD_rule,
  /* 1115 */  vecX_rule,
  /* 1116 */  vecD_rule,
  /* 1117 */  vecX_rule,
  /* 1118 */  vecD_rule,
  /* 1119 */  vecX_rule,
  /* 1120 */  vecX_rule,
  /* 1121 */  vecD_rule,
  /* 1122 */  _MulVS_vecD_vecD_rule,
  /* 1123 */  vecX_rule,
  /* 1124 */  _MulVS_vecX_vecX_rule,
  /* 1125 */  vecD_rule,
  /* 1126 */  _MulVI_vecD_vecD_rule,
  /* 1127 */  vecX_rule,
  /* 1128 */  _MulVI_vecX_vecX_rule,
  /* 1129 */  vecD_rule,
  /* 1130 */  vecX_rule,
  /* 1131 */  vecX_rule,
  /* 1132 */  vecD_rule,
  /* 1133 */  vecX_rule,
  /* 1134 */  vecD_rule,
  /* 1135 */  vecX_rule,
  /* 1136 */  vecD_rule,
  /* 1137 */  vecD_rule,
  /* 1138 */  vecX_rule,
  /* 1139 */  vecX_rule,
  /* 1140 */  vecX_rule,
  /* 1141 */  vecX_rule,
  /* 1142 */  vecX_rule,
  /* 1143 */  vecD_rule,
  /* 1144 */  vecX_rule,
  /* 1145 */  vecX_rule,
  /* 1146 */  vecD_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  vecD_rule,
  /* 1149 */  vecX_rule,
  /* 1150 */  vecD_rule,
  /* 1151 */  vecX_rule,
  /* 1152 */  vecD_rule,
  /* 1153 */  vecX_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecD_rule,
  /* 1157 */  vecX_rule,
  /* 1158 */  vecD_rule,
  /* 1159 */  vecX_rule,
  /* 1160 */  vecD_rule,
  /* 1161 */  vecX_rule,
  /* 1162 */  vecD_rule,
  /* 1163 */  vecX_rule,
  /* 1164 */  vecD_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  vecD_rule,
  /* 1167 */  vecX_rule,
  /* 1168 */  vecD_rule,
  /* 1169 */  vecX_rule,
  /* 1170 */  vecD_rule,
  /* 1171 */  vecX_rule,
  /* 1172 */  vecD_rule,
  /* 1173 */  vecX_rule,
  /* 1174 */  vecD_rule,
  /* 1175 */  vecX_rule,
  /* 1176 */  vecD_rule,
  /* 1177 */  vecX_rule,
  /* 1178 */  vecD_rule,
  /* 1179 */  vecX_rule,
  /* 1180 */  vecD_rule,
  /* 1181 */  vecX_rule,
  /* 1182 */  vecD_rule,
  /* 1183 */  vecX_rule,
  /* 1184 */  vecD_rule,
  /* 1185 */  vecX_rule,
  /* 1186 */  vecD_rule,
  /* 1187 */  vecX_rule,
  /* 1188 */  vecX_rule,
  /* 1189 */  vecX_rule,
  /* 1190 */  vecX_rule,
  /* 1191 */  vecX_rule,
  /* 1192 */  vecX_rule,
  /* 1193 */  vecX_rule,
  /* 1194 */  vecD_rule,
  /* 1195 */  vecX_rule,
  /* 1196 */  vecX_rule,
  /* 1197 */  vecD_rule,
  /* 1198 */  vecX_rule,
  /* 1199 */  vecX_rule,
  /* 1200 */  vecX_rule,
  /* 1201 */  indirect_rule,
  /* 1202 */  indirect_rule,
  /* 1203 */  indirect_rule,
  /* 1204 */  indirect_rule,
  /* 1205 */  indirect_rule,
  /* 1206 */  indirect_rule,
  /* 1207 */  indirect_rule,
  /* 1208 */  indirect_rule,
  /* 1209 */  indirect_rule,
  /* 1210 */  indirect_rule,
  /* 1211 */  indirect_rule,
  /* 1212 */  indirect_rule,
  /* 1213 */  memory8_rule,
  /* 1214 */  memory8_rule,
  /* 1215 */  indirect_rule,
  /* 1216 */  indirect_rule,
  /* 1217 */  indirect_rule,
  /* 1218 */  indirect_rule,
  /* 1219 */  indirect_rule,
  /* 1220 */  indirect_rule,
  /* 1221 */  indirect_rule,
  /* 1222 */  indirect_rule,
  /* 1223 */  indirect_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  0,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  /*  142 */  0,
  /*  143 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  144 */  _LShiftL_iRegL_immIScale_rule,
  /*  145 */  _ConvI2L_iRegI__rule,
  /*  146 */  iRegL_rule,
  /*  147 */  immIOffset_rule,
  /*  148 */  immIOffset1_rule,
  /*  149 */  immIOffset2_rule,
  /*  150 */  immIOffset4_rule,
  /*  151 */  immIOffset8_rule,
  /*  152 */  immIOffset16_rule,
  /*  153 */  immLoffset_rule,
  /*  154 */  immLoffset1_rule,
  /*  155 */  immLoffset2_rule,
  /*  156 */  immLoffset4_rule,
  /*  157 */  immLoffset8_rule,
  /*  158 */  immLoffset16_rule,
  /*  159 */  0,
  /*  160 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  161 */  _LShiftL_iRegL_immIScale_rule,
  /*  162 */  _ConvI2L_iRegI__rule,
  /*  163 */  iRegL_rule,
  /*  164 */  immIOffset_rule,
  /*  165 */  immLoffset_rule,
  /*  166 */  immL_pc_off_rule,
  /*  167 */  0,
  /*  168 */  0,
  /*  169 */  0,
  /*  170 */  0,
  /*  171 */  0,
  /*  172 */  0,
  /*  173 */  0,
  /*  174 */  0,
  /*  175 */  0,
  /*  176 */  0,
  /*  177 */  0,
  // last operand
  /*  178 */  0,
  /*  179 */  0,
  /*  180 */  0,
  /*  181 */  0,
  /*  182 */  0,
  /*  183 */  0,
  /*  184 */  0,
  /*  185 */  0,
  /*  186 */  0,
  // last operand class
  /*  187 */  0,
  /*  188 */  immIScale_rule,
  /*  189 */  immIScale_rule,
  /*  190 */  0,
  /*  191 */  0,
  /*  192 */  0,
  /*  193 */  0,
  /*  194 */  0,
  /*  195 */  0,
  /*  196 */  0,
  /*  197 */  0,
  /*  198 */  0,
  /*  199 */  0,
  /*  200 */  0,
  /*  201 */  0,
  /*  202 */  0,
  /*  203 */  0,
  /*  204 */  0,
  /*  205 */  0,
  /*  206 */  iRegP_rule,
  /*  207 */  iRegLNoSp_rule,
  /*  208 */  iRegINoSp_rule,
  /*  209 */  iRegNNoSp_rule,
  /*  210 */  iRegI_rule,
  /*  211 */  iRegL_rule,
  /*  212 */  iRegN_rule,
  /*  213 */  rFlagsReg_rule,
  /*  214 */  iRegIorL2I_rule,
  /*  215 */  rFlagsRegU_rule,
  /*  216 */  iRegIorL2I_rule,
  /*  217 */  immI0_rule,
  /*  218 */  immI0_rule,
  /*  219 */  immL0_rule,
  /*  220 */  iRegL_rule,
  /*  221 */  immP0_rule,
  /*  222 */  iRegP_rule,
  /*  223 */  immN0_rule,
  /*  224 */  iRegN_rule,
  /*  225 */  vRegF_rule,
  /*  226 */  vRegD_rule,
  /*  227 */  0,
  /*  228 */  0,
  /*  229 */  immIScale_rule,
  /*  230 */  iRegIorL2I_rule,
  /*  231 */  iRegIorL2I_rule,
  /*  232 */  iRegL_rule,
  /*  233 */  iRegL_rule,
  /*  234 */  _ConvI2L_iRegIorL2I__rule,
  /*  235 */  _ConvI2L_iRegIorL2I__rule,
  /*  236 */  immI_M1_rule,
  /*  237 */  immL_M1_rule,
  /*  238 */  iRegIorL2I_rule,
  /*  239 */  iRegL_rule,
  /*  240 */  immI_rule,
  /*  241 */  immI_M1_rule,
  /*  242 */  immI_rule,
  /*  243 */  immL_M1_rule,
  /*  244 */  immI_rule,
  /*  245 */  immI_M1_rule,
  /*  246 */  immI_rule,
  /*  247 */  immL_M1_rule,
  /*  248 */  immI_rule,
  /*  249 */  immI_M1_rule,
  /*  250 */  immI_rule,
  /*  251 */  immL_M1_rule,
  /*  252 */  iRegIorL2I_rule,
  /*  253 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  254 */  iRegL_rule,
  /*  255 */  _URShiftL_iRegL_immI_rule,
  /*  256 */  iRegIorL2I_rule,
  /*  257 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  258 */  iRegL_rule,
  /*  259 */  _RShiftL_iRegL_immI_rule,
  /*  260 */  iRegIorL2I_rule,
  /*  261 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  262 */  iRegL_rule,
  /*  263 */  _LShiftL_iRegL_immI_rule,
  /*  264 */  immI_bitmask_rule,
  /*  265 */  immI_bitmask_rule,
  /*  266 */  immL_bitmask_rule,
  /*  267 */  0,
  /*  268 */  iRegI_rule,
  /*  269 */  iRegI_rule,
  /*  270 */  _SubI_immI_64_iRegI_rule,
  /*  271 */  iRegI_rule,
  /*  272 */  _SubI_immI0_iRegI_rule,
  /*  273 */  iRegI_rule,
  /*  274 */  iRegI_rule,
  /*  275 */  _SubI_immI_32_iRegI_rule,
  /*  276 */  _SubI_immI0_iRegI_rule,
  /*  277 */  iRegI_rule,
  /*  278 */  _SubI_immI_64_iRegI_rule,
  /*  279 */  _SubI_immI0_iRegI_rule,
  /*  280 */  iRegI_rule,
  /*  281 */  _SubI_immI_32_iRegI_rule,
  /*  282 */  _SubI_immI0_iRegI_rule,
  /*  283 */  immI_16_rule,
  /*  284 */  immI_16_rule,
  /*  285 */  immI_24_rule,
  /*  286 */  immI_24_rule,
  /*  287 */  immI_24_rule,
  /*  288 */  immI_48_rule,
  /*  289 */  immI_48_rule,
  /*  290 */  immI_32_rule,
  /*  291 */  immI_32_rule,
  /*  292 */  immI_56_rule,
  /*  293 */  immI_56_rule,
  /*  294 */  immI_56_rule,
  /*  295 */  immI_255_rule,
  /*  296 */  immI_65535_rule,
  /*  297 */  immL_255_rule,
  /*  298 */  immL_65535_rule,
  /*  299 */  immL_4294967295_rule,
  /*  300 */  immIExt_rule,
  /*  301 */  immIExt_rule,
  /*  302 */  immIExt_rule,
  /*  303 */  immIExt_rule,
  /*  304 */  immIExt_rule,
  /*  305 */  immIExt_rule,
  /*  306 */  immIExt_rule,
  /*  307 */  immIExt_rule,
  /*  308 */  immIExt_rule,
  /*  309 */  immIExt_rule,
  /*  310 */  immIExt_rule,
  /*  311 */  0,
  /*  312 */  vRegF_rule,
  /*  313 */  _NegF_vRegF__rule,
  /*  314 */  0,
  /*  315 */  vRegD_rule,
  /*  316 */  _NegD_vRegD__rule,
  /*  317 */  iRegIorL2I_rule,
  /*  318 */  iRegL_rule,
  /*  319 */  immI0_rule,
  /*  320 */  immL0_rule,
  /*  321 */  immP0_rule,
  /*  322 */  immN0_rule,
  /*  323 */  immP0_rule,
  /*  324 */  immI0_rule,
  /*  325 */  immL0_rule,
  /*  326 */  immL_rule,
  /*  327 */  immL0_rule,
  /*  328 */  immI_rule,
  /*  329 */  immI0_rule,
  /*  330 */  iRegL_rule,
  /*  331 */  iRegIorL2I_rule,
  /*  332 */  iRegP_R0_rule,
  /*  333 */  iRegI_R2_rule,
  /*  334 */  iRegI_R4_rule,
  /*  335 */  iRegI_R4_rule,
  /*  336 */  iRegI_R2_rule,
  /*  337 */  immI_le_4_rule,
  /*  338 */  immI_1_rule,
  /*  339 */  iRegP_R3_rule,
  /*  340 */  iRegI_R3_rule,
  /*  341 */  vecD_rule,
  /*  342 */  vecX_rule,
  /*  343 */  vecD_rule,
  /*  344 */  vecX_rule,
  /*  345 */  vecD_rule,
  /*  346 */  vecX_rule,
  /*  347 */  0,
  /*  348 */  vecD_rule,
  /*  349 */  _NegVF_vecD__rule,
  /*  350 */  0,
  /*  351 */  vecX_rule,
  /*  352 */  _NegVF_vecX__rule,
  /*  353 */  0,
  /*  354 */  vecX_rule,
  /*  355 */  _NegVD_vecX__rule,
  /*  356 */  0,
  /*  357 */  0,
  // last internally defined operand
  /*  358 */  0,
  /*  359 */  0,
  /*  360 */  0,
  /*  361 */  0,
  /*  362 */  0,
  /*  363 */  0,
  /*  364 */  0,
  /*  365 */  0,
  /*  366 */  0,
  /*  367 */  0,
  /*  368 */  0,
  /*  369 */  0,
  /*  370 */  0,
  /*  371 */  0,
  /*  372 */  0,
  /*  373 */  0,
  /*  374 */  0,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  0,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  _Binary_iRegP_iRegP_rule,
  /*  381 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  382 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  383 */  0,
  /*  384 */  0,
  /*  385 */  0,
  /*  386 */  0,
  /*  387 */  0,
  /*  388 */  0,
  /*  389 */  0,
  /*  390 */  0,
  /*  391 */  0,
  /*  392 */  0,
  /*  393 */  0,
  /*  394 */  0,
  /*  395 */  iRegIorL2I_rule,
  /*  396 */  immIAddSub_rule,
  /*  397 */  iRegL_rule,
  /*  398 */  immLAddSub_rule,
  /*  399 */  iRegIorL2I_rule,
  /*  400 */  immIAddSub_rule,
  /*  401 */  iRegL_rule,
  /*  402 */  immLAddSub_rule,
  /*  403 */  iRegIorL2I_rule,
  /*  404 */  iRegL_rule,
  /*  405 */  iRegIorL2I_rule,
  /*  406 */  iRegL_rule,
  /*  407 */  iRegI_rule,
  /*  408 */  immI0_rule,
  /*  409 */  immIAddSub_rule,
  /*  410 */  immI_rule,
  /*  411 */  iRegI_rule,
  /*  412 */  immI0_rule,
  /*  413 */  immIAddSub_rule,
  /*  414 */  immI_rule,
  /*  415 */  iRegL_rule,
  /*  416 */  immL0_rule,
  /*  417 */  immLAddSub_rule,
  /*  418 */  immL_rule,
  /*  419 */  iRegL_rule,
  /*  420 */  immL0_rule,
  /*  421 */  immLAddSub_rule,
  /*  422 */  immL_rule,
  /*  423 */  iRegP_rule,
  /*  424 */  iRegN_rule,
  /*  425 */  immP0_rule,
  /*  426 */  immN0_rule,
  /*  427 */  vRegF_rule,
  /*  428 */  immF0_rule,
  /*  429 */  vRegD_rule,
  /*  430 */  immD0_rule,
  /*  431 */  immL0_rule,
  /*  432 */  immI0_rule,
  /*  433 */  immL0_rule,
  /*  434 */  immI0_rule,
  /*  435 */  iRegP_rule,
  /*  436 */  iRegP_rule,
  /*  437 */  immP0_rule,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  0,
  /*  441 */  0,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  0,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  0,
  /*  462 */  0,
  /*  463 */  0,
  /*  464 */  0,
  /*  465 */  0,
  /*  466 */  0,
  /*  467 */  0,
  /*  468 */  0,
  /*  469 */  0,
  /*  470 */  0,
  /*  471 */  immL_32bits_rule,
  /*  472 */  0,
  /*  473 */  0,
  /*  474 */  0,
  /*  475 */  0,
  /*  476 */  0,
  /*  477 */  0,
  /*  478 */  0,
  /*  479 */  0,
  /*  480 */  immI0_rule,
  /*  481 */  immI0_rule,
  /*  482 */  iRegIorL2I_rule,
  /*  483 */  immI0_rule,
  /*  484 */  iRegIorL2I_rule,
  /*  485 */  immI0_rule,
  /*  486 */  iRegIorL2I_rule,
  /*  487 */  immI0_rule,
  /*  488 */  iRegL_rule,
  /*  489 */  immL0_rule,
  /*  490 */  iRegP_rule,
  /*  491 */  immP0_rule,
  /*  492 */  iRegN_rule,
  /*  493 */  immN0_rule,
  /*  494 */  vRegF_rule,
  /*  495 */  vRegD_rule,
  /*  496 */  iRegN_rule,
  /*  497 */  0,
  /*  498 */  0,
  /*  499 */  0,
  /*  500 */  0,
  /*  501 */  0,
  /*  502 */  0,
  /*  503 */  0,
  /*  504 */  0,
  /*  505 */  0,
  /*  506 */  0,
  /*  507 */  immL_32bits_rule,
  /*  508 */  0,
  /*  509 */  0,
  /*  510 */  0,
  /*  511 */  0,
  /*  512 */  0,
  /*  513 */  iRegIorL2I_rule,
  /*  514 */  iRegIorL2I_rule,
  /*  515 */  iRegIorL2I_rule,
  /*  516 */  iRegL_rule,
  /*  517 */  iRegP_rule,
  /*  518 */  iRegN_rule,
  /*  519 */  vRegF_rule,
  /*  520 */  vRegD_rule,
  /*  521 */  0,
  /*  522 */  0,
  /*  523 */  0,
  /*  524 */  0,
  /*  525 */  0,
  /*  526 */  0,
  /*  527 */  0,
  /*  528 */  0,
  /*  529 */  0,
  /*  530 */  0,
  /*  531 */  0,
  /*  532 */  0,
  /*  533 */  0,
  /*  534 */  0,
  /*  535 */  0,
  /*  536 */  0,
  /*  537 */  0,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  0,
  /*  542 */  0,
  /*  543 */  0,
  /*  544 */  0,
  /*  545 */  0,
  /*  546 */  0,
  /*  547 */  0,
  /*  548 */  0,
  /*  549 */  0,
  /*  550 */  0,
  /*  551 */  0,
  /*  552 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  553 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  554 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  555 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  556 */  _Binary_iRegP_iRegP_rule,
  /*  557 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  558 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  559 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  560 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  561 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  562 */  _Binary_iRegP_iRegP_rule,
  /*  563 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  564 */  _Binary_iRegI_iRegI_rule,
  /*  565 */  _Binary_iRegI_iRegI_rule,
  /*  566 */  _Binary_iRegI_iRegI_rule,
  /*  567 */  _Binary_iRegL_iRegL_rule,
  /*  568 */  _Binary_iRegN_iRegN_rule,
  /*  569 */  _Binary_iRegP_iRegP_rule,
  /*  570 */  _Binary_iRegI_iRegI_rule,
  /*  571 */  _Binary_iRegI_iRegI_rule,
  /*  572 */  _Binary_iRegI_iRegI_rule,
  /*  573 */  _Binary_iRegL_iRegL_rule,
  /*  574 */  _Binary_iRegN_iRegN_rule,
  /*  575 */  _Binary_iRegP_iRegP_rule,
  /*  576 */  _Binary_iRegI_iRegI_rule,
  /*  577 */  _Binary_iRegI_iRegI_rule,
  /*  578 */  _Binary_iRegI_iRegI_rule,
  /*  579 */  _Binary_iRegL_iRegL_rule,
  /*  580 */  _Binary_iRegN_iRegN_rule,
  /*  581 */  _Binary_iRegP_iRegP_rule,
  /*  582 */  _Binary_iRegI_iRegI_rule,
  /*  583 */  _Binary_iRegI_iRegI_rule,
  /*  584 */  _Binary_iRegI_iRegI_rule,
  /*  585 */  _Binary_iRegL_iRegL_rule,
  /*  586 */  _Binary_iRegN_iRegN_rule,
  /*  587 */  _Binary_iRegP_iRegP_rule,
  /*  588 */  iRegI_rule,
  /*  589 */  iRegL_rule,
  /*  590 */  iRegN_rule,
  /*  591 */  iRegP_rule,
  /*  592 */  iRegI_rule,
  /*  593 */  iRegL_rule,
  /*  594 */  iRegN_rule,
  /*  595 */  iRegP_rule,
  /*  596 */  iRegL_rule,
  /*  597 */  iRegL_rule,
  /*  598 */  immLAddSub_rule,
  /*  599 */  immLAddSub_rule,
  /*  600 */  iRegIorL2I_rule,
  /*  601 */  iRegIorL2I_rule,
  /*  602 */  immIAddSub_rule,
  /*  603 */  immIAddSub_rule,
  /*  604 */  iRegL_rule,
  /*  605 */  iRegL_rule,
  /*  606 */  immLAddSub_rule,
  /*  607 */  immLAddSub_rule,
  /*  608 */  iRegIorL2I_rule,
  /*  609 */  iRegIorL2I_rule,
  /*  610 */  immIAddSub_rule,
  /*  611 */  immIAddSub_rule,
  /*  612 */  iRegL_rule,
  /*  613 */  immLAddSub_rule,
  /*  614 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  615 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  616 */  _Binary_immI0_iRegIorL2I_rule,
  /*  617 */  _Binary_immI0_iRegIorL2I_rule,
  /*  618 */  _Binary_iRegIorL2I_immI0_rule,
  /*  619 */  _Binary_iRegIorL2I_immI0_rule,
  /*  620 */  _Binary_immI_1_immI0_rule,
  /*  621 */  _Binary_immI_1_immI0_rule,
  /*  622 */  _Binary_iRegL_iRegL_rule,
  /*  623 */  _Binary_iRegL_iRegL_rule,
  /*  624 */  _Binary_iRegL_immL0_rule,
  /*  625 */  _Binary_iRegL_immL0_rule,
  /*  626 */  _Binary_immL0_iRegL_rule,
  /*  627 */  _Binary_immL0_iRegL_rule,
  /*  628 */  _Binary_iRegP_iRegP_rule,
  /*  629 */  _Binary_iRegP_iRegP_rule,
  /*  630 */  _Binary_iRegP_immP0_rule,
  /*  631 */  _Binary_iRegP_immP0_rule,
  /*  632 */  _Binary_immP0_iRegP_rule,
  /*  633 */  _Binary_immP0_iRegP_rule,
  /*  634 */  _Binary_iRegN_iRegN_rule,
  /*  635 */  _Binary_iRegN_iRegN_rule,
  /*  636 */  _Binary_iRegN_immN0_rule,
  /*  637 */  _Binary_iRegN_immN0_rule,
  /*  638 */  _Binary_immN0_iRegN_rule,
  /*  639 */  _Binary_immN0_iRegN_rule,
  /*  640 */  _Binary_vRegF_vRegF_rule,
  /*  641 */  _Binary_vRegF_vRegF_rule,
  /*  642 */  _Binary_vRegD_vRegD_rule,
  /*  643 */  _Binary_vRegD_vRegD_rule,
  /*  644 */  iRegIorL2I_rule,
  /*  645 */  immIAddSub_rule,
  /*  646 */  immIAddSub_rule,
  /*  647 */  iRegL_rule,
  /*  648 */  _ConvI2L_iRegIorL2I__rule,
  /*  649 */  _LShiftL_iRegL_immIScale_rule,
  /*  650 */  _LShiftL__ConvI2L_iRegIorL2I__immIScale_rule,
  /*  651 */  immI_rule,
  /*  652 */  immLAddSub_rule,
  /*  653 */  iRegL_rule,
  /*  654 */  immLAddSub_rule,
  /*  655 */  iRegIorL2I_rule,
  /*  656 */  immIAddSub_rule,
  /*  657 */  iRegL_rule,
  /*  658 */  immLAddSub_rule,
  /*  659 */  iRegIorL2I_rule,
  /*  660 */  iRegL_rule,
  /*  661 */  iRegIorL2I_rule,
  /*  662 */  _ConvI2L_iRegIorL2I__rule,
  /*  663 */  _ConvI2L_iRegIorL2I__rule,
  /*  664 */  iRegL_rule,
  /*  665 */  iRegL_rule,
  /*  666 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  667 */  iRegIorL2I_rule,
  /*  668 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  669 */  iRegIorL2I_rule,
  /*  670 */  _SubI_immI0_iRegIorL2I_rule,
  /*  671 */  _SubI_immI0_iRegIorL2I_rule,
  /*  672 */  iRegIorL2I_rule,
  /*  673 */  _MulL_iRegL_iRegL_rule,
  /*  674 */  iRegL_rule,
  /*  675 */  _MulL_iRegL_iRegL_rule,
  /*  676 */  iRegL_rule,
  /*  677 */  _SubL_immL0_iRegL_rule,
  /*  678 */  _SubL_immL0_iRegL_rule,
  /*  679 */  iRegL_rule,
  /*  680 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  681 */  iRegLNoSp_rule,
  /*  682 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  683 */  iRegLNoSp_rule,
  /*  684 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  685 */  _MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I__rule,
  /*  686 */  _ConvI2L_iRegIorL2I__rule,
  /*  687 */  _SubL_immL0__ConvI2L_iRegIorL2I__rule,
  /*  688 */  _SubL_immL0__ConvI2L_iRegIorL2I__rule,
  /*  689 */  _ConvI2L_iRegIorL2I__rule,
  /*  690 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  691 */  iRegIorL2I_rule,
  /*  692 */  iRegL_rule,
  /*  693 */  iRegIorL2I_rule,
  /*  694 */  iRegL_rule,
  /*  695 */  iRegIorL2I_rule,
  /*  696 */  immI_rule,
  /*  697 */  iRegIorL2I_rule,
  /*  698 */  immI_rule,
  /*  699 */  iRegIorL2I_rule,
  /*  700 */  immI_rule,
  /*  701 */  iRegIorL2I_rule,
  /*  702 */  immI_rule,
  /*  703 */  iRegIorL2I_rule,
  /*  704 */  immI_rule,
  /*  705 */  immI_rule,
  /*  706 */  iRegIorL2I_rule,
  /*  707 */  immI_rule,
  /*  708 */  immL_M1_rule,
  /*  709 */  immI_M1_rule,
  /*  710 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  711 */  iRegIorL2I_rule,
  /*  712 */  _XorL_iRegL_immL_M1_rule,
  /*  713 */  iRegL_rule,
  /*  714 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  715 */  iRegIorL2I_rule,
  /*  716 */  _XorL_iRegL_immL_M1_rule,
  /*  717 */  iRegL_rule,
  /*  718 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  719 */  immI_M1_rule,
  /*  720 */  _XorL_iRegL_iRegL_rule,
  /*  721 */  immL_M1_rule,
  /*  722 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  723 */  iRegIorL2I_rule,
  /*  724 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  725 */  iRegL_rule,
  /*  726 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  727 */  iRegIorL2I_rule,
  /*  728 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  729 */  iRegL_rule,
  /*  730 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  731 */  iRegIorL2I_rule,
  /*  732 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  733 */  iRegL_rule,
  /*  734 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  735 */  immI_M1_rule,
  /*  736 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  737 */  immI_M1_rule,
  /*  738 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  739 */  immL_M1_rule,
  /*  740 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  741 */  immL_M1_rule,
  /*  742 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  743 */  immI_M1_rule,
  /*  744 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  745 */  immI_M1_rule,
  /*  746 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  747 */  immL_M1_rule,
  /*  748 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  749 */  immL_M1_rule,
  /*  750 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  751 */  immI_M1_rule,
  /*  752 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  753 */  immI_M1_rule,
  /*  754 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  755 */  immL_M1_rule,
  /*  756 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  757 */  immL_M1_rule,
  /*  758 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  759 */  iRegIorL2I_rule,
  /*  760 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  761 */  iRegL_rule,
  /*  762 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  763 */  iRegIorL2I_rule,
  /*  764 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  765 */  iRegL_rule,
  /*  766 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  767 */  iRegIorL2I_rule,
  /*  768 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  769 */  iRegL_rule,
  /*  770 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  771 */  iRegIorL2I_rule,
  /*  772 */  _URShiftL_iRegL_immI_rule,
  /*  773 */  iRegL_rule,
  /*  774 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  775 */  iRegIorL2I_rule,
  /*  776 */  _RShiftL_iRegL_immI_rule,
  /*  777 */  iRegL_rule,
  /*  778 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  779 */  iRegIorL2I_rule,
  /*  780 */  _LShiftL_iRegL_immI_rule,
  /*  781 */  iRegL_rule,
  /*  782 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  783 */  iRegIorL2I_rule,
  /*  784 */  _URShiftL_iRegL_immI_rule,
  /*  785 */  iRegL_rule,
  /*  786 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  787 */  iRegIorL2I_rule,
  /*  788 */  _RShiftL_iRegL_immI_rule,
  /*  789 */  iRegL_rule,
  /*  790 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  791 */  iRegIorL2I_rule,
  /*  792 */  _LShiftL_iRegL_immI_rule,
  /*  793 */  iRegL_rule,
  /*  794 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  795 */  iRegIorL2I_rule,
  /*  796 */  _URShiftL_iRegL_immI_rule,
  /*  797 */  iRegL_rule,
  /*  798 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  799 */  iRegIorL2I_rule,
  /*  800 */  _RShiftL_iRegL_immI_rule,
  /*  801 */  iRegL_rule,
  /*  802 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  803 */  iRegIorL2I_rule,
  /*  804 */  _LShiftL_iRegL_immI_rule,
  /*  805 */  iRegL_rule,
  /*  806 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  807 */  iRegIorL2I_rule,
  /*  808 */  _URShiftL_iRegL_immI_rule,
  /*  809 */  iRegL_rule,
  /*  810 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  811 */  iRegIorL2I_rule,
  /*  812 */  _RShiftL_iRegL_immI_rule,
  /*  813 */  iRegL_rule,
  /*  814 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  815 */  iRegIorL2I_rule,
  /*  816 */  _LShiftL_iRegL_immI_rule,
  /*  817 */  iRegL_rule,
  /*  818 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  819 */  _URShiftL_iRegL_immI_rule,
  /*  820 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  821 */  _RShiftL_iRegL_immI_rule,
  /*  822 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  823 */  _LShiftL_iRegL_immI_rule,
  /*  824 */  immI_rule,
  /*  825 */  immI_rule,
  /*  826 */  immI_rule,
  /*  827 */  immI_rule,
  /*  828 */  immI_bitmask_rule,
  /*  829 */  immL_bitmask_rule,
  /*  830 */  0,
  /*  831 */  immI_rule,
  /*  832 */  immI_rule,
  /*  833 */  immI_rule,
  /*  834 */  _URShiftL_iRegL_immI_rule,
  /*  835 */  _LShiftL_iRegL_immI_rule,
  /*  836 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  837 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  838 */  _URShiftL_iRegL_immI_rule,
  /*  839 */  _LShiftL_iRegL_immI_rule,
  /*  840 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  841 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  842 */  0,
  /*  843 */  0,
  /*  844 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  845 */  _LShiftL_iRegL_iRegI_rule,
  /*  846 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  847 */  _LShiftL_iRegL_iRegI_rule,
  /*  848 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  849 */  _LShiftI_iRegI_iRegI_rule,
  /*  850 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  851 */  _LShiftI_iRegI_iRegI_rule,
  /*  852 */  0,
  /*  853 */  0,
  /*  854 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  855 */  _URShiftL_iRegL_iRegI_rule,
  /*  856 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  857 */  _URShiftL_iRegL_iRegI_rule,
  /*  858 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  859 */  _URShiftI_iRegI_iRegI_rule,
  /*  860 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  861 */  _URShiftI_iRegI_iRegI_rule,
  /*  862 */  _ConvI2L_iRegIorL2I__rule,
  /*  863 */  iRegL_rule,
  /*  864 */  _ConvI2L_iRegIorL2I__rule,
  /*  865 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  866 */  iRegIorL2I_rule,
  /*  867 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  868 */  iRegIorL2I_rule,
  /*  869 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  870 */  iRegIorL2I_rule,
  /*  871 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  872 */  iRegL_rule,
  /*  873 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  874 */  iRegL_rule,
  /*  875 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  876 */  iRegL_rule,
  /*  877 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  878 */  iRegL_rule,
  /*  879 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  880 */  iRegIorL2I_rule,
  /*  881 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  882 */  iRegIorL2I_rule,
  /*  883 */  _AndL_iRegL_immL_255_rule,
  /*  884 */  iRegL_rule,
  /*  885 */  _AndL_iRegL_immL_65535_rule,
  /*  886 */  iRegL_rule,
  /*  887 */  _AndL_iRegL_immL_4294967295_rule,
  /*  888 */  iRegL_rule,
  /*  889 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  890 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  891 */  _AndL_iRegL_immL_255_rule,
  /*  892 */  _AndL_iRegL_immL_65535_rule,
  /*  893 */  _AndL_iRegL_immL_4294967295_rule,
  /*  894 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_56_immI_56_immIExt_rule,
  /*  895 */  iRegL_rule,
  /*  896 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_48_immI_48_immIExt_rule,
  /*  897 */  iRegL_rule,
  /*  898 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_32_immI_32_immIExt_rule,
  /*  899 */  iRegL_rule,
  /*  900 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_56_immI_56_immIExt_rule,
  /*  901 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_48_immI_48_immIExt_rule,
  /*  902 */  _LShiftL__RShiftL__LShiftL_iRegL_immI_32_immI_32_immIExt_rule,
  /*  903 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_immIExt_rule,
  /*  904 */  iRegIorL2I_rule,
  /*  905 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_immIExt_rule,
  /*  906 */  iRegIorL2I_rule,
  /*  907 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_immIExt_rule,
  /*  908 */  _LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_immIExt_rule,
  /*  909 */  _LShiftL__ConvI2L_iRegIorL2I__immIExt_rule,
  /*  910 */  iRegL_rule,
  /*  911 */  _LShiftL__ConvI2L_iRegIorL2I__immIExt_rule,
  /*  912 */  _LShiftL__AndL_iRegL_immL_255_immIExt_rule,
  /*  913 */  iRegL_rule,
  /*  914 */  _LShiftL__AndL_iRegL_immL_65535_immIExt_rule,
  /*  915 */  iRegL_rule,
  /*  916 */  _LShiftL__AndL_iRegL_immL_4294967295_immIExt_rule,
  /*  917 */  iRegL_rule,
  /*  918 */  _LShiftL__AndL_iRegL_immL_255_immIExt_rule,
  /*  919 */  _LShiftL__AndL_iRegL_immL_65535_immIExt_rule,
  /*  920 */  _LShiftL__AndL_iRegL_immL_4294967295_immIExt_rule,
  /*  921 */  _LShiftI__AndI_iRegIorL2I_immI_255_immIExt_rule,
  /*  922 */  iRegIorL2I_rule,
  /*  923 */  _LShiftI__AndI_iRegIorL2I_immI_65535_immIExt_rule,
  /*  924 */  iRegIorL2I_rule,
  /*  925 */  _LShiftI__AndI_iRegIorL2I_immI_255_immIExt_rule,
  /*  926 */  _LShiftI__AndI_iRegIorL2I_immI_65535_immIExt_rule,
  /*  927 */  vRegF_rule,
  /*  928 */  vRegD_rule,
  /*  929 */  vRegF_rule,
  /*  930 */  vRegD_rule,
  /*  931 */  vRegF_rule,
  /*  932 */  vRegD_rule,
  /*  933 */  _Binary_vRegF_vRegF_rule,
  /*  934 */  _Binary_vRegD_vRegD_rule,
  /*  935 */  _Binary__NegF_vRegF__vRegF_rule,
  /*  936 */  _Binary_vRegF__NegF_vRegF__rule,
  /*  937 */  _Binary__NegD_vRegD__vRegD_rule,
  /*  938 */  _Binary_vRegD__NegD_vRegD__rule,
  /*  939 */  _Binary__NegF_vRegF__vRegF_rule,
  /*  940 */  _Binary_vRegF__NegF_vRegF__rule,
  /*  941 */  _Binary__NegD_vRegD__vRegD_rule,
  /*  942 */  _Binary_vRegD__NegD_vRegD__rule,
  /*  943 */  _Binary_vRegF_vRegF_rule,
  /*  944 */  _Binary_vRegD_vRegD_rule,
  /*  945 */  vRegF_rule,
  /*  946 */  vRegF_rule,
  /*  947 */  vRegD_rule,
  /*  948 */  vRegD_rule,
  /*  949 */  vRegF_rule,
  /*  950 */  vRegD_rule,
  /*  951 */  0,
  /*  952 */  0,
  /*  953 */  0,
  /*  954 */  0,
  /*  955 */  immI_rule,
  /*  956 */  iRegIorL2I_rule,
  /*  957 */  immILog_rule,
  /*  958 */  iRegIorL2I_rule,
  /*  959 */  immILog_rule,
  /*  960 */  iRegIorL2I_rule,
  /*  961 */  immILog_rule,
  /*  962 */  iRegL_rule,
  /*  963 */  immLLog_rule,
  /*  964 */  iRegL_rule,
  /*  965 */  immLLog_rule,
  /*  966 */  iRegL_rule,
  /*  967 */  immLLog_rule,
  /*  968 */  0,
  /*  969 */  immL_32bits_rule,
  /*  970 */  0,
  /*  971 */  0,
  /*  972 */  0,
  /*  973 */  0,
  /*  974 */  0,
  /*  975 */  0,
  /*  976 */  0,
  /*  977 */  0,
  /*  978 */  0,
  /*  979 */  0,
  /*  980 */  0,
  /*  981 */  0,
  /*  982 */  0,
  /*  983 */  0,
  /*  984 */  0,
  /*  985 */  0,
  /*  986 */  0,
  /*  987 */  iRegP_R10_rule,
  /*  988 */  iRegP_R10_rule,
  /*  989 */  _OverflowMulI_iRegIorL2I_iRegIorL2I_rule,
  /*  990 */  _OverflowMulL_iRegL_iRegL_rule,
  /*  991 */  vRegF_rule,
  /*  992 */  vRegD_rule,
  /*  993 */  immF0_rule,
  /*  994 */  immD0_rule,
  /*  995 */  iRegIorL2I_rule,
  /*  996 */  immI0_rule,
  /*  997 */  0,
  /*  998 */  iRegI_rule,
  /*  999 */  0,
  /* 1000 */  iRegI_rule,
  /* 1001 */  0,
  /* 1002 */  rFlagsReg_rule,
  /* 1003 */  rFlagsRegU_rule,
  /* 1004 */  _CmpI_iRegIorL2I_immI0_rule,
  /* 1005 */  _CmpL_iRegL_immL0_rule,
  /* 1006 */  _CmpP_iRegP_immP0_rule,
  /* 1007 */  _CmpN_iRegN_immN0_rule,
  /* 1008 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /* 1009 */  _CmpU_iRegIorL2I_immI0_rule,
  /* 1010 */  _CmpUL_iRegL_immL0_rule,
  /* 1011 */  _CmpL_iRegL_immL0_rule,
  /* 1012 */  _CmpI_iRegIorL2I_immI0_rule,
  /* 1013 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /* 1014 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /* 1015 */  _CmpL_iRegL_immL0_rule,
  /* 1016 */  _CmpI_iRegIorL2I_immI0_rule,
  /* 1017 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /* 1018 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /* 1019 */  rFlagsReg_rule,
  /* 1020 */  rFlagsRegU_rule,
  /* 1021 */  0,
  /* 1022 */  0,
  /* 1023 */  0,
  /* 1024 */  0,
  /* 1025 */  0,
  /* 1026 */  0,
  /* 1027 */  inline_cache_RegP_rule,
  /* 1028 */  iRegP_R0_rule,
  /* 1029 */  0,
  /* 1030 */  0,
  /* 1031 */  0,
  /* 1032 */  0,
  /* 1033 */  iRegP_R0_rule,
  /* 1034 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /* 1035 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /* 1036 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /* 1037 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /* 1038 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /* 1039 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /* 1040 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /* 1041 */  _Binary_iRegP_R3_immI_le_4_rule,
  /* 1042 */  _Binary_iRegP_R3_immI_le_4_rule,
  /* 1043 */  _Binary_iRegP_R3_immI_1_rule,
  /* 1044 */  iRegI_R3_rule,
  /* 1045 */  iRegI_R4_rule,
  /* 1046 */  iRegI_R4_rule,
  /* 1047 */  iRegP_R2_rule,
  /* 1048 */  iRegP_R2_rule,
  /* 1049 */  iRegI_R2_rule,
  /* 1050 */  _Binary_iRegP_R1_iRegI_R3_rule,
  /* 1051 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /* 1052 */  _Binary_iRegP_R1_iRegI_R3_rule,
  /* 1053 */  0,
  /* 1054 */  0,
  /* 1055 */  0,
  /* 1056 */  vecD_rule,
  /* 1057 */  vecD_rule,
  /* 1058 */  vecX_rule,
  /* 1059 */  0,
  /* 1060 */  0,
  /* 1061 */  0,
  /* 1062 */  0,
  /* 1063 */  0,
  /* 1064 */  0,
  /* 1065 */  0,
  /* 1066 */  0,
  /* 1067 */  0,
  /* 1068 */  0,
  /* 1069 */  0,
  /* 1070 */  0,
  /* 1071 */  0,
  /* 1072 */  0,
  /* 1073 */  0,
  /* 1074 */  0,
  /* 1075 */  0,
  /* 1076 */  vecD_rule,
  /* 1077 */  vecX_rule,
  /* 1078 */  vecD_rule,
  /* 1079 */  vecX_rule,
  /* 1080 */  vecD_rule,
  /* 1081 */  vecX_rule,
  /* 1082 */  vecD_rule,
  /* 1083 */  vecX_rule,
  /* 1084 */  vecX_rule,
  /* 1085 */  vecX_rule,
  /* 1086 */  vecD_rule,
  /* 1087 */  vecX_rule,
  /* 1088 */  vecX_rule,
  /* 1089 */  vecD_rule,
  /* 1090 */  vecX_rule,
  /* 1091 */  vecX_rule,
  /* 1092 */  vecD_rule,
  /* 1093 */  vecX_rule,
  /* 1094 */  vecD_rule,
  /* 1095 */  vecX_rule,
  /* 1096 */  vecD_rule,
  /* 1097 */  vecX_rule,
  /* 1098 */  vecX_rule,
  /* 1099 */  vecD_rule,
  /* 1100 */  vecX_rule,
  /* 1101 */  vecX_rule,
  /* 1102 */  vecD_rule,
  /* 1103 */  vecX_rule,
  /* 1104 */  vecD_rule,
  /* 1105 */  vecX_rule,
  /* 1106 */  vecD_rule,
  /* 1107 */  vecX_rule,
  /* 1108 */  vecX_rule,
  /* 1109 */  vecD_rule,
  /* 1110 */  vecX_rule,
  /* 1111 */  vecX_rule,
  /* 1112 */  vecD_rule,
  /* 1113 */  vecX_rule,
  /* 1114 */  vecD_rule,
  /* 1115 */  vecX_rule,
  /* 1116 */  vecD_rule,
  /* 1117 */  vecX_rule,
  /* 1118 */  vecD_rule,
  /* 1119 */  vecX_rule,
  /* 1120 */  vecX_rule,
  /* 1121 */  _MulVS_vecD_vecD_rule,
  /* 1122 */  vecD_rule,
  /* 1123 */  _MulVS_vecX_vecX_rule,
  /* 1124 */  vecX_rule,
  /* 1125 */  _MulVI_vecD_vecD_rule,
  /* 1126 */  vecD_rule,
  /* 1127 */  _MulVI_vecX_vecX_rule,
  /* 1128 */  vecX_rule,
  /* 1129 */  _Binary_vecD_vecD_rule,
  /* 1130 */  _Binary_vecX_vecX_rule,
  /* 1131 */  _Binary_vecX_vecX_rule,
  /* 1132 */  _MulVS_vecD_vecD_rule,
  /* 1133 */  _MulVS_vecX_vecX_rule,
  /* 1134 */  _MulVI_vecD_vecD_rule,
  /* 1135 */  _MulVI_vecX_vecX_rule,
  /* 1136 */  _Binary__NegVF_vecD__vecD_rule,
  /* 1137 */  _Binary_vecD__NegVF_vecD__rule,
  /* 1138 */  _Binary__NegVF_vecX__vecX_rule,
  /* 1139 */  _Binary_vecX__NegVF_vecX__rule,
  /* 1140 */  _Binary__NegVD_vecX__vecX_rule,
  /* 1141 */  _Binary_vecX__NegVD_vecX__rule,
  /* 1142 */  vecX_rule,
  /* 1143 */  vecD_rule,
  /* 1144 */  vecX_rule,
  /* 1145 */  vecX_rule,
  /* 1146 */  vecD_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  vecD_rule,
  /* 1149 */  vecX_rule,
  /* 1150 */  vecD_rule,
  /* 1151 */  vecX_rule,
  /* 1152 */  vecD_rule,
  /* 1153 */  vecX_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecD_rule,
  /* 1157 */  vecX_rule,
  /* 1158 */  _LShiftCntV_immI__rule,
  /* 1159 */  _LShiftCntV_immI__rule,
  /* 1160 */  _RShiftCntV_immI__rule,
  /* 1161 */  _RShiftCntV_immI__rule,
  /* 1162 */  _RShiftCntV_immI__rule,
  /* 1163 */  _RShiftCntV_immI__rule,
  /* 1164 */  vecD_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  vecD_rule,
  /* 1167 */  vecX_rule,
  /* 1168 */  vecD_rule,
  /* 1169 */  vecX_rule,
  /* 1170 */  _LShiftCntV_immI__rule,
  /* 1171 */  _LShiftCntV_immI__rule,
  /* 1172 */  _RShiftCntV_immI__rule,
  /* 1173 */  _RShiftCntV_immI__rule,
  /* 1174 */  _RShiftCntV_immI__rule,
  /* 1175 */  _RShiftCntV_immI__rule,
  /* 1176 */  vecD_rule,
  /* 1177 */  vecX_rule,
  /* 1178 */  vecD_rule,
  /* 1179 */  vecX_rule,
  /* 1180 */  vecD_rule,
  /* 1181 */  vecX_rule,
  /* 1182 */  _LShiftCntV_immI__rule,
  /* 1183 */  _LShiftCntV_immI__rule,
  /* 1184 */  _RShiftCntV_immI__rule,
  /* 1185 */  _RShiftCntV_immI__rule,
  /* 1186 */  _RShiftCntV_immI__rule,
  /* 1187 */  _RShiftCntV_immI__rule,
  /* 1188 */  vecX_rule,
  /* 1189 */  vecX_rule,
  /* 1190 */  vecX_rule,
  /* 1191 */  _LShiftCntV_immI__rule,
  /* 1192 */  _RShiftCntV_immI__rule,
  /* 1193 */  _RShiftCntV_immI__rule,
  /* 1194 */  vecD_rule,
  /* 1195 */  vecX_rule,
  /* 1196 */  vecX_rule,
  /* 1197 */  vecD_rule,
  /* 1198 */  vecX_rule,
  /* 1199 */  vecX_rule,
  /* 1200 */  immI_rule,
  /* 1201 */  _Binary_iRegP_iRegP_rule,
  /* 1202 */  _Binary_iRegN_iRegN_rule,
  /* 1203 */  _Binary_iRegP_iRegP_rule,
  /* 1204 */  _Binary_iRegN_iRegN_rule,
  /* 1205 */  _Binary_iRegN_iRegN_rule,
  /* 1206 */  _Binary_iRegP_iRegP_rule,
  /* 1207 */  _Binary_iRegN_iRegN_rule,
  /* 1208 */  _Binary_iRegP_iRegP_rule,
  /* 1209 */  _Binary_iRegN_iRegN_rule,
  /* 1210 */  _Binary_iRegP_iRegP_rule,
  /* 1211 */  _Binary_iRegN_iRegN_rule,
  /* 1212 */  _Binary_iRegP_iRegP_rule,
  /* 1213 */  0,
  /* 1214 */  0,
  /* 1215 */  0,
  /* 1216 */  _Binary_iRegP_iRegP_rule,
  /* 1217 */  _Binary_iRegP_iRegP_rule,
  /* 1218 */  _Binary_iRegP_iRegP_rule,
  /* 1219 */  _Binary_iRegP_iRegP_rule,
  /* 1220 */  _Binary_iRegP_iRegP_rule,
  /* 1221 */  _Binary_iRegP_iRegP_rule,
  /* 1222 */  iRegP_rule,
  /* 1223 */  iRegP_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMIEXT",
  /*   13 */  "IMMI_LE_4",
  /*   14 */  "IMMI_31",
  /*   15 */  "IMMI_8",
  /*   16 */  "IMMI_16",
  /*   17 */  "IMMI_24",
  /*   18 */  "IMMI_32",
  /*   19 */  "IMMI_48",
  /*   20 */  "IMMI_56",
  /*   21 */  "IMMI_63",
  /*   22 */  "IMMI_64",
  /*   23 */  "IMMI_255",
  /*   24 */  "IMMI_65535",
  /*   25 */  "IMML_255",
  /*   26 */  "IMML_65535",
  /*   27 */  "IMML_4294967295",
  /*   28 */  "IMML_BITMASK",
  /*   29 */  "IMMI_BITMASK",
  /*   30 */  "IMMISCALE",
  /*   31 */  "IMMI26",
  /*   32 */  "IMMI19",
  /*   33 */  "IMMIU12",
  /*   34 */  "IMMLU12",
  /*   35 */  "IMMIOFFSET",
  /*   36 */  "IMMIOFFSET1",
  /*   37 */  "IMMIOFFSET2",
  /*   38 */  "IMMIOFFSET4",
  /*   39 */  "IMMIOFFSET8",
  /*   40 */  "IMMIOFFSET16",
  /*   41 */  "IMMLOFFSET",
  /*   42 */  "IMMLOFFSET1",
  /*   43 */  "IMMLOFFSET2",
  /*   44 */  "IMMLOFFSET4",
  /*   45 */  "IMMLOFFSET8",
  /*   46 */  "IMMLOFFSET16",
  /*   47 */  "IMMIADDSUB",
  /*   48 */  "IMMILOG",
  /*   49 */  "IMML",
  /*   50 */  "IMML0",
  /*   51 */  "IMML_1",
  /*   52 */  "IMML_M1",
  /*   53 */  "IMML_PC_OFF",
  /*   54 */  "IMMLADDSUB",
  /*   55 */  "IMMLLOG",
  /*   56 */  "IMML_32BITS",
  /*   57 */  "IMMP",
  /*   58 */  "IMMP0",
  /*   59 */  "IMMP_1",
  /*   60 */  "IMMBYTEMAPBASE",
  /*   61 */  "IMMP_M1",
  /*   62 */  "IMMP_M2",
  /*   63 */  "IMMD",
  /*   64 */  "IMMD0",
  /*   65 */  "IMMDPACKED",
  /*   66 */  "IMMF",
  /*   67 */  "IMMF0",
  /*   68 */  "IMMFPACKED",
  /*   69 */  "IMMN",
  /*   70 */  "IMMN0",
  /*   71 */  "IMMNKLASS",
  /*   72 */  "IREGI",
  /*   73 */  "IREGINOSP",
  /*   74 */  "IREGL",
  /*   75 */  "IREGLNOSP",
  /*   76 */  "IREGP",
  /*   77 */  "IREGPNOSP",
  /*   78 */  "IREGP_R0",
  /*   79 */  "IREGP_R1",
  /*   80 */  "IREGP_R2",
  /*   81 */  "IREGP_R3",
  /*   82 */  "IREGP_R4",
  /*   83 */  "IREGP_R5",
  /*   84 */  "IREGP_R10",
  /*   85 */  "IREGL_R0",
  /*   86 */  "IREGL_R2",
  /*   87 */  "IREGL_R3",
  /*   88 */  "IREGL_R11",
  /*   89 */  "IREGP_FP",
  /*   90 */  "IREGI_R0",
  /*   91 */  "IREGI_R2",
  /*   92 */  "IREGI_R3",
  /*   93 */  "IREGI_R4",
  /*   94 */  "IREGN",
  /*   95 */  "IREGN_R0",
  /*   96 */  "IREGN_R2",
  /*   97 */  "IREGN_R3",
  /*   98 */  "IREGNNOSP",
  /*   99 */  "IREGIHEAPBASE",
  /*  100 */  "VREGF",
  /*  101 */  "VREGD",
  /*  102 */  "VECD",
  /*  103 */  "VECX",
  /*  104 */  "VREGD_V0",
  /*  105 */  "VREGD_V1",
  /*  106 */  "VREGD_V2",
  /*  107 */  "VREGD_V3",
  /*  108 */  "VREGD_V4",
  /*  109 */  "VREGD_V5",
  /*  110 */  "VREGD_V6",
  /*  111 */  "VREGD_V7",
  /*  112 */  "VREGD_V8",
  /*  113 */  "VREGD_V9",
  /*  114 */  "VREGD_V10",
  /*  115 */  "VREGD_V11",
  /*  116 */  "VREGD_V12",
  /*  117 */  "VREGD_V13",
  /*  118 */  "VREGD_V14",
  /*  119 */  "VREGD_V15",
  /*  120 */  "VREGD_V16",
  /*  121 */  "VREGD_V17",
  /*  122 */  "VREGD_V18",
  /*  123 */  "VREGD_V19",
  /*  124 */  "VREGD_V20",
  /*  125 */  "VREGD_V21",
  /*  126 */  "VREGD_V22",
  /*  127 */  "VREGD_V23",
  /*  128 */  "VREGD_V24",
  /*  129 */  "VREGD_V25",
  /*  130 */  "VREGD_V26",
  /*  131 */  "VREGD_V27",
  /*  132 */  "VREGD_V28",
  /*  133 */  "VREGD_V29",
  /*  134 */  "VREGD_V30",
  /*  135 */  "VREGD_V31",
  /*  136 */  "RFLAGSREG",
  /*  137 */  "RFLAGSREGU",
  /*  138 */  "INLINE_CACHE_REGP",
  /*  139 */  "INTERPRETER_METHOD_OOP_REGP",
  /*  140 */  "THREAD_REGP",
  /*  141 */  "LR_REGP",
  /*  142 */  "INDIRECT",
  /*  143 */  "INDINDEXSCALEDI2L",
  /*  144 */  "INDINDEXSCALED",
  /*  145 */  "INDINDEXI2L",
  /*  146 */  "INDINDEX",
  /*  147 */  "INDOFFI",
  /*  148 */  "INDOFFI1",
  /*  149 */  "INDOFFI2",
  /*  150 */  "INDOFFI4",
  /*  151 */  "INDOFFI8",
  /*  152 */  "INDOFFI16",
  /*  153 */  "INDOFFL",
  /*  154 */  "INDOFFL1",
  /*  155 */  "INDOFFL2",
  /*  156 */  "INDOFFL4",
  /*  157 */  "INDOFFL8",
  /*  158 */  "INDOFFL16",
  /*  159 */  "INDIRECTN",
  /*  160 */  "INDINDEXSCALEDI2LN",
  /*  161 */  "INDINDEXSCALEDN",
  /*  162 */  "INDINDEXI2LN",
  /*  163 */  "INDINDEXN",
  /*  164 */  "INDOFFIN",
  /*  165 */  "INDOFFLN",
  /*  166 */  "THREAD_ANCHOR_PC",
  /*  167 */  "STACKSLOTP",
  /*  168 */  "STACKSLOTI",
  /*  169 */  "STACKSLOTF",
  /*  170 */  "STACKSLOTD",
  /*  171 */  "STACKSLOTL",
  /*  172 */  "CMPOP",
  /*  173 */  "CMPOPU",
  /*  174 */  "CMPOPEQNE",
  /*  175 */  "CMPOPLTGE",
  /*  176 */  "CMPOPUEQNELTGE",
  /*  177 */  "IREGL2I",
  // last operand
  /*  178 */  "VMEM4",
  /*  179 */  "VMEM8",
  /*  180 */  "VMEM16",
  /*  181 */  "MEMORY1",
  /*  182 */  "MEMORY2",
  /*  183 */  "MEMORY4",
  /*  184 */  "MEMORY8",
  /*  185 */  "MEMORY",
  /*  186 */  "IREGIORL2I",
  // last operand class
  /*  187 */  "_ConvI2L_iRegI_",
  /*  188 */  "_LShiftL__ConvI2L_iRegI__immIScale",
  /*  189 */  "_LShiftL_iRegL_immIScale",
  /*  190 */  "_DecodeN_iRegN_",
  /*  191 */  "_LoadB_memory1_",
  /*  192 */  "_LoadUB_memory1_",
  /*  193 */  "_LoadS_memory2_",
  /*  194 */  "_LoadUS_memory2_",
  /*  195 */  "_LoadI_memory4_",
  /*  196 */  "_ConvI2L__LoadI_memory4__",
  /*  197 */  "_LoadB_indirect_",
  /*  198 */  "_LoadUB_indirect_",
  /*  199 */  "_LoadUS_indirect_",
  /*  200 */  "_LoadS_indirect_",
  /*  201 */  "_LoadI_indirect_",
  /*  202 */  "_ConvI2L__LoadI_indirect__",
  /*  203 */  "_LoadL_memory8_",
  /*  204 */  "_CastP2X_iRegP_",
  /*  205 */  "_CastP2X__DecodeN_iRegN__",
  /*  206 */  "_Binary_iRegP_iRegP",
  /*  207 */  "_Binary_iRegLNoSp_iRegLNoSp",
  /*  208 */  "_Binary_iRegINoSp_iRegINoSp",
  /*  209 */  "_Binary_iRegNNoSp_iRegNNoSp",
  /*  210 */  "_Binary_iRegI_iRegI",
  /*  211 */  "_Binary_iRegL_iRegL",
  /*  212 */  "_Binary_iRegN_iRegN",
  /*  213 */  "_Binary_cmpOp_rFlagsReg",
  /*  214 */  "_Binary_iRegIorL2I_iRegIorL2I",
  /*  215 */  "_Binary_cmpOpU_rFlagsRegU",
  /*  216 */  "_Binary_immI0_iRegIorL2I",
  /*  217 */  "_Binary_iRegIorL2I_immI0",
  /*  218 */  "_Binary_immI_1_immI0",
  /*  219 */  "_Binary_iRegL_immL0",
  /*  220 */  "_Binary_immL0_iRegL",
  /*  221 */  "_Binary_iRegP_immP0",
  /*  222 */  "_Binary_immP0_iRegP",
  /*  223 */  "_Binary_iRegN_immN0",
  /*  224 */  "_Binary_immN0_iRegN",
  /*  225 */  "_Binary_vRegF_vRegF",
  /*  226 */  "_Binary_vRegD_vRegD",
  /*  227 */  "_ConvL2I_iRegL_",
  /*  228 */  "_ConvI2L_iRegIorL2I_",
  /*  229 */  "_LShiftL__ConvI2L_iRegIorL2I__immIScale",
  /*  230 */  "_MulI_iRegIorL2I_iRegIorL2I",
  /*  231 */  "_SubI_immI0_iRegIorL2I",
  /*  232 */  "_MulL_iRegL_iRegL",
  /*  233 */  "_SubL_immL0_iRegL",
  /*  234 */  "_MulL__ConvI2L_iRegIorL2I___ConvI2L_iRegIorL2I_",
  /*  235 */  "_SubL_immL0__ConvI2L_iRegIorL2I_",
  /*  236 */  "_XorI_iRegIorL2I_immI_M1",
  /*  237 */  "_XorL_iRegL_immL_M1",
  /*  238 */  "_XorI_iRegIorL2I_iRegIorL2I",
  /*  239 */  "_XorL_iRegL_iRegL",
  /*  240 */  "_URShiftI_iRegIorL2I_immI",
  /*  241 */  "_XorI__URShiftI_iRegIorL2I_immI_immI_M1",
  /*  242 */  "_URShiftL_iRegL_immI",
  /*  243 */  "_XorL__URShiftL_iRegL_immI_immL_M1",
  /*  244 */  "_RShiftI_iRegIorL2I_immI",
  /*  245 */  "_XorI__RShiftI_iRegIorL2I_immI_immI_M1",
  /*  246 */  "_RShiftL_iRegL_immI",
  /*  247 */  "_XorL__RShiftL_iRegL_immI_immL_M1",
  /*  248 */  "_LShiftI_iRegIorL2I_immI",
  /*  249 */  "_XorI__LShiftI_iRegIorL2I_immI_immI_M1",
  /*  250 */  "_LShiftL_iRegL_immI",
  /*  251 */  "_XorL__LShiftL_iRegL_immI_immL_M1",
  /*  252 */  "_XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  253 */  "_XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI",
  /*  254 */  "_XorL__URShiftL_iRegL_immI_iRegL",
  /*  255 */  "_XorL_iRegL__URShiftL_iRegL_immI",
  /*  256 */  "_XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  257 */  "_XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI",
  /*  258 */  "_XorL__RShiftL_iRegL_immI_iRegL",
  /*  259 */  "_XorL_iRegL__RShiftL_iRegL_immI",
  /*  260 */  "_XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  261 */  "_XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI",
  /*  262 */  "_XorL__LShiftL_iRegL_immI_iRegL",
  /*  263 */  "_XorL_iRegL__LShiftL_iRegL_immI",
  /*  264 */  "_AndI__URShiftI_iRegIorL2I_immI_immI_bitmask",
  /*  265 */  "_AndI_iRegIorL2I_immI_bitmask",
  /*  266 */  "_AndL_iRegL_immL_bitmask",
  /*  267 */  "_ConvI2L__AndI_iRegIorL2I_immI_bitmask_",
  /*  268 */  "_LShiftL_iRegL_iRegI",
  /*  269 */  "_SubI_immI_64_iRegI",
  /*  270 */  "_URShiftL_iRegL__SubI_immI_64_iRegI",
  /*  271 */  "_SubI_immI0_iRegI",
  /*  272 */  "_URShiftL_iRegL__SubI_immI0_iRegI",
  /*  273 */  "_LShiftI_iRegI_iRegI",
  /*  274 */  "_SubI_immI_32_iRegI",
  /*  275 */  "_URShiftI_iRegI__SubI_immI_32_iRegI",
  /*  276 */  "_URShiftI_iRegI__SubI_immI0_iRegI",
  /*  277 */  "_URShiftL_iRegL_iRegI",
  /*  278 */  "_LShiftL_iRegL__SubI_immI_64_iRegI",
  /*  279 */  "_LShiftL_iRegL__SubI_immI0_iRegI",
  /*  280 */  "_URShiftI_iRegI_iRegI",
  /*  281 */  "_LShiftI_iRegI__SubI_immI_32_iRegI",
  /*  282 */  "_LShiftI_iRegI__SubI_immI0_iRegI",
  /*  283 */  "_LShiftI_iRegIorL2I_immI_16",
  /*  284 */  "_RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16",
  /*  285 */  "_LShiftI_iRegIorL2I_immI_24",
  /*  286 */  "_RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24",
  /*  287 */  "_URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24",
  /*  288 */  "_LShiftL_iRegL_immI_48",
  /*  289 */  "_RShiftL__LShiftL_iRegL_immI_48_immI_48",
  /*  290 */  "_LShiftL_iRegL_immI_32",
  /*  291 */  "_RShiftL__LShiftL_iRegL_immI_32_immI_32",
  /*  292 */  "_LShiftL_iRegL_immI_56",
  /*  293 */  "_RShiftL__LShiftL_iRegL_immI_56_immI_56",
  /*  294 */  "_URShiftL__LShiftL_iRegL_immI_56_immI_56",
  /*  295 */  "_AndI_iRegIorL2I_immI_255",
  /*  296 */  "_AndI_iRegIorL2I_immI_65535",
  /*  297 */  "_AndL_iRegL_immL_255",
  /*  298 */  "_AndL_iRegL_immL_65535",
  /*  299 */  "_AndL_iRegL_immL_4294967295",
  /*  300 */  "_LShiftL__RShiftL__LShiftL_iRegL_immI_56_immI_56_immIExt",
  /*  301 */  "_LShiftL__RShiftL__LShiftL_iRegL_immI_48_immI_48_immIExt",
  /*  302 */  "_LShiftL__RShiftL__LShiftL_iRegL_immI_32_immI_32_immIExt",
  /*  303 */  "_LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_immIExt",
  /*  304 */  "_LShiftI__RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_immIExt",
  /*  305 */  "_LShiftL__ConvI2L_iRegIorL2I__immIExt",
  /*  306 */  "_LShiftL__AndL_iRegL_immL_255_immIExt",
  /*  307 */  "_LShiftL__AndL_iRegL_immL_65535_immIExt",
  /*  308 */  "_LShiftL__AndL_iRegL_immL_4294967295_immIExt",
  /*  309 */  "_LShiftI__AndI_iRegIorL2I_immI_255_immIExt",
  /*  310 */  "_LShiftI__AndI_iRegIorL2I_immI_65535_immIExt",
  /*  311 */  "_NegF_vRegF_",
  /*  312 */  "_Binary__NegF_vRegF__vRegF",
  /*  313 */  "_Binary_vRegF__NegF_vRegF_",
  /*  314 */  "_NegD_vRegD_",
  /*  315 */  "_Binary__NegD_vRegD__vRegD",
  /*  316 */  "_Binary_vRegD__NegD_vRegD_",
  /*  317 */  "_OverflowMulI_iRegIorL2I_iRegIorL2I",
  /*  318 */  "_OverflowMulL_iRegL_iRegL",
  /*  319 */  "_CmpI_iRegIorL2I_immI0",
  /*  320 */  "_CmpL_iRegL_immL0",
  /*  321 */  "_CmpP_iRegP_immP0",
  /*  322 */  "_CmpN_iRegN_immN0",
  /*  323 */  "_CmpP__DecodeN_iRegN__immP0",
  /*  324 */  "_CmpU_iRegIorL2I_immI0",
  /*  325 */  "_CmpUL_iRegL_immL0",
  /*  326 */  "_AndL_iRegL_immL",
  /*  327 */  "_CmpL__AndL_iRegL_immL_immL0",
  /*  328 */  "_AndI_iRegIorL2I_immI",
  /*  329 */  "_CmpI__AndI_iRegIorL2I_immI_immI0",
  /*  330 */  "_AndL_iRegL_iRegL",
  /*  331 */  "_AndI_iRegIorL2I_iRegIorL2I",
  /*  332 */  "_PartialSubtypeCheck_iRegP_R4_iRegP_R0",
  /*  333 */  "_Binary_iRegP_R1_iRegI_R2",
  /*  334 */  "_Binary_iRegP_R3_iRegI_R4",
  /*  335 */  "_Binary_iRegP_R1_iRegI_R4",
  /*  336 */  "_Binary_iRegP_R3_iRegI_R2",
  /*  337 */  "_Binary_iRegP_R3_immI_le_4",
  /*  338 */  "_Binary_iRegP_R3_immI_1",
  /*  339 */  "_Binary_iRegP_R1_iRegP_R3",
  /*  340 */  "_Binary_iRegP_R1_iRegI_R3",
  /*  341 */  "_MulVS_vecD_vecD",
  /*  342 */  "_MulVS_vecX_vecX",
  /*  343 */  "_MulVI_vecD_vecD",
  /*  344 */  "_MulVI_vecX_vecX",
  /*  345 */  "_Binary_vecD_vecD",
  /*  346 */  "_Binary_vecX_vecX",
  /*  347 */  "_NegVF_vecD_",
  /*  348 */  "_Binary__NegVF_vecD__vecD",
  /*  349 */  "_Binary_vecD__NegVF_vecD_",
  /*  350 */  "_NegVF_vecX_",
  /*  351 */  "_Binary__NegVF_vecX__vecX",
  /*  352 */  "_Binary_vecX__NegVF_vecX_",
  /*  353 */  "_NegVD_vecX_",
  /*  354 */  "_Binary__NegVD_vecX__vecX",
  /*  355 */  "_Binary_vecX__NegVD_vecX_",
  /*  356 */  "_LShiftCntV_immI_",
  /*  357 */  "_RShiftCntV_immI_",
  // last internally defined operand
  /*  358 */  "loadConI",
  /*  359 */  "loadConL",
  /*  360 */  "loadConP",
  /*  361 */  "loadConP0",
  /*  362 */  "loadConP1",
  /*  363 */  "loadByteMapBase",
  /*  364 */  "loadConN",
  /*  365 */  "loadConN0",
  /*  366 */  "loadConNKlass",
  /*  367 */  "loadConF_packed",
  /*  368 */  "loadConF",
  /*  369 */  "loadConD_packed",
  /*  370 */  "loadConD",
  /*  371 */  "countLeadingZerosI",
  /*  372 */  "countLeadingZerosL",
  /*  373 */  "countTrailingZerosI",
  /*  374 */  "countTrailingZerosL",
  /*  375 */  "castX2P",
  /*  376 */  "castP2X",
  /*  377 */  "checkCastPP",
  /*  378 */  "castPP",
  /*  379 */  "castII",
  /*  380 */  "storePConditional",
  /*  381 */  "storeLConditional",
  /*  382 */  "storeIConditional",
  /*  383 */  "negF_reg_reg",
  /*  384 */  "negD_reg_reg",
  /*  385 */  "absF_reg",
  /*  386 */  "absD_reg",
  /*  387 */  "MoveF2I_stack_reg",
  /*  388 */  "MoveI2F_stack_reg",
  /*  389 */  "MoveD2L_stack_reg",
  /*  390 */  "MoveL2D_stack_reg",
  /*  391 */  "MoveF2I_reg_reg",
  /*  392 */  "MoveI2F_reg_reg",
  /*  393 */  "MoveD2L_reg_reg",
  /*  394 */  "MoveL2D_reg_reg",
  /*  395 */  "overflowAddI_reg_reg",
  /*  396 */  "overflowAddI_reg_imm",
  /*  397 */  "overflowAddL_reg_reg",
  /*  398 */  "overflowAddL_reg_imm",
  /*  399 */  "overflowSubI_reg_reg",
  /*  400 */  "overflowSubI_reg_imm",
  /*  401 */  "overflowSubL_reg_reg",
  /*  402 */  "overflowSubL_reg_imm",
  /*  403 */  "overflowNegI_reg",
  /*  404 */  "overflowNegL_reg",
  /*  405 */  "overflowMulI_reg",
  /*  406 */  "overflowMulL_reg",
  /*  407 */  "compI_reg_reg",
  /*  408 */  "compI_reg_immI0",
  /*  409 */  "compI_reg_immIAddSub",
  /*  410 */  "compI_reg_immI",
  /*  411 */  "compU_reg_reg",
  /*  412 */  "compU_reg_immI0",
  /*  413 */  "compU_reg_immIAddSub",
  /*  414 */  "compU_reg_immI",
  /*  415 */  "compL_reg_reg",
  /*  416 */  "compL_reg_immL0",
  /*  417 */  "compL_reg_immLAddSub",
  /*  418 */  "compL_reg_immL",
  /*  419 */  "compUL_reg_reg",
  /*  420 */  "compUL_reg_immL0",
  /*  421 */  "compUL_reg_immLAddSub",
  /*  422 */  "compUL_reg_immL",
  /*  423 */  "compP_reg_reg",
  /*  424 */  "compN_reg_reg",
  /*  425 */  "testP_reg",
  /*  426 */  "testN_reg",
  /*  427 */  "compF_reg_reg",
  /*  428 */  "compF_reg_zero",
  /*  429 */  "compD_reg_reg",
  /*  430 */  "compD_reg_zero",
  /*  431 */  "cmpL_and",
  /*  432 */  "cmpI_and",
  /*  433 */  "cmpL_and_reg",
  /*  434 */  "cmpI_and_reg",
  /*  435 */  "cmpFastLock",
  /*  436 */  "cmpFastUnlock",
  /*  437 */  "partialSubtypeCheckVsZero",
  /*  438 */  "tlsLoadP",
  /*  439 */  "vsqrt2F",
  /*  440 */  "vsqrt4F",
  /*  441 */  "vsqrt2D",
  /*  442 */  "vabs8B",
  /*  443 */  "vabs16B",
  /*  444 */  "vabs4S",
  /*  445 */  "vabs8S",
  /*  446 */  "vabs2I",
  /*  447 */  "vabs4I",
  /*  448 */  "vabs2L",
  /*  449 */  "vabs2F",
  /*  450 */  "vabs4F",
  /*  451 */  "vabs2D",
  /*  452 */  "vneg2F",
  /*  453 */  "vneg4F",
  /*  454 */  "vneg2D",
  /*  455 */  "vshiftcnt8B",
  /*  456 */  "vshiftcnt8B_0",
  /*  457 */  "vshiftcnt16B",
  /*  458 */  "vshiftcnt16B_0",
  /*  459 */  "vpopcount4I",
  /*  460 */  "vpopcount2I",
  /*  461 */  "loadB",
  /*  462 */  "loadB2L",
  /*  463 */  "loadUB",
  /*  464 */  "loadUB2L",
  /*  465 */  "loadS",
  /*  466 */  "loadS2L",
  /*  467 */  "loadUS",
  /*  468 */  "loadUS2L",
  /*  469 */  "loadI",
  /*  470 */  "loadI2L",
  /*  471 */  "loadUI2L",
  /*  472 */  "loadL",
  /*  473 */  "loadRange",
  /*  474 */  "loadP",
  /*  475 */  "loadN",
  /*  476 */  "loadKlass",
  /*  477 */  "loadNKlass",
  /*  478 */  "loadF",
  /*  479 */  "loadD",
  /*  480 */  "storeimmCM0",
  /*  481 */  "storeimmCM0_ordered",
  /*  482 */  "storeB",
  /*  483 */  "storeimmB0",
  /*  484 */  "storeC",
  /*  485 */  "storeimmC0",
  /*  486 */  "storeI",
  /*  487 */  "storeimmI0",
  /*  488 */  "storeL",
  /*  489 */  "storeimmL0",
  /*  490 */  "storeP",
  /*  491 */  "storeimmP0",
  /*  492 */  "storeN",
  /*  493 */  "storeImmN0",
  /*  494 */  "storeF",
  /*  495 */  "storeD",
  /*  496 */  "storeNKlass",
  /*  497 */  "prefetchalloc",
  /*  498 */  "loadB_volatile",
  /*  499 */  "loadB2L_volatile",
  /*  500 */  "loadUB_volatile",
  /*  501 */  "loadUB2L_volatile",
  /*  502 */  "loadS_volatile",
  /*  503 */  "loadUS_volatile",
  /*  504 */  "loadUS2L_volatile",
  /*  505 */  "loadS2L_volatile",
  /*  506 */  "loadI_volatile",
  /*  507 */  "loadUI2L_volatile",
  /*  508 */  "loadL_volatile",
  /*  509 */  "loadP_volatile",
  /*  510 */  "loadN_volatile",
  /*  511 */  "loadF_volatile",
  /*  512 */  "loadD_volatile",
  /*  513 */  "storeB_volatile",
  /*  514 */  "storeC_volatile",
  /*  515 */  "storeI_volatile",
  /*  516 */  "storeL_volatile",
  /*  517 */  "storeP_volatile",
  /*  518 */  "storeN_volatile",
  /*  519 */  "storeF_volatile",
  /*  520 */  "storeD_volatile",
  /*  521 */  "cacheWB",
  /*  522 */  "cacheWBPreSync",
  /*  523 */  "cacheWBPostSync",
  /*  524 */  "bytes_reverse_int",
  /*  525 */  "bytes_reverse_long",
  /*  526 */  "bytes_reverse_unsigned_short",
  /*  527 */  "bytes_reverse_short",
  /*  528 */  "popCountI",
  /*  529 */  "popCountI_mem",
  /*  530 */  "popCountL",
  /*  531 */  "popCountL_mem",
  /*  532 */  "load_fence",
  /*  533 */  "unnecessary_membar_acquire",
  /*  534 */  "membar_acquire",
  /*  535 */  "membar_acquire_lock",
  /*  536 */  "store_fence",
  /*  537 */  "unnecessary_membar_release",
  /*  538 */  "membar_release",
  /*  539 */  "membar_storestore",
  /*  540 */  "membar_release_lock",
  /*  541 */  "unnecessary_membar_volatile",
  /*  542 */  "membar_volatile",
  /*  543 */  "convP2I",
  /*  544 */  "convN2I",
  /*  545 */  "encodeHeapOop",
  /*  546 */  "encodeHeapOop_not_null",
  /*  547 */  "decodeHeapOop",
  /*  548 */  "decodeHeapOop_not_null",
  /*  549 */  "encodeKlass_not_null",
  /*  550 */  "decodeKlass_not_null",
  /*  551 */  "loadPLocked",
  /*  552 */  "compareAndSwapB",
  /*  553 */  "compareAndSwapS",
  /*  554 */  "compareAndSwapI",
  /*  555 */  "compareAndSwapL",
  /*  556 */  "compareAndSwapP",
  /*  557 */  "compareAndSwapN",
  /*  558 */  "compareAndSwapBAcq",
  /*  559 */  "compareAndSwapSAcq",
  /*  560 */  "compareAndSwapIAcq",
  /*  561 */  "compareAndSwapLAcq",
  /*  562 */  "compareAndSwapPAcq",
  /*  563 */  "compareAndSwapNAcq",
  /*  564 */  "compareAndExchangeB",
  /*  565 */  "compareAndExchangeS",
  /*  566 */  "compareAndExchangeI",
  /*  567 */  "compareAndExchangeL",
  /*  568 */  "compareAndExchangeN",
  /*  569 */  "compareAndExchangeP",
  /*  570 */  "compareAndExchangeBAcq",
  /*  571 */  "compareAndExchangeSAcq",
  /*  572 */  "compareAndExchangeIAcq",
  /*  573 */  "compareAndExchangeLAcq",
  /*  574 */  "compareAndExchangeNAcq",
  /*  575 */  "compareAndExchangePAcq",
  /*  576 */  "weakCompareAndSwapB",
  /*  577 */  "weakCompareAndSwapS",
  /*  578 */  "weakCompareAndSwapI",
  /*  579 */  "weakCompareAndSwapL",
  /*  580 */  "weakCompareAndSwapN",
  /*  581 */  "weakCompareAndSwapP",
  /*  582 */  "weakCompareAndSwapBAcq",
  /*  583 */  "weakCompareAndSwapSAcq",
  /*  584 */  "weakCompareAndSwapIAcq",
  /*  585 */  "weakCompareAndSwapLAcq",
  /*  586 */  "weakCompareAndSwapNAcq",
  /*  587 */  "weakCompareAndSwapPAcq",
  /*  588 */  "get_and_setI",
  /*  589 */  "get_and_setL",
  /*  590 */  "get_and_setN",
  /*  591 */  "get_and_setP",
  /*  592 */  "get_and_setIAcq",
  /*  593 */  "get_and_setLAcq",
  /*  594 */  "get_and_setNAcq",
  /*  595 */  "get_and_setPAcq",
  /*  596 */  "get_and_addL",
  /*  597 */  "get_and_addL_no_res",
  /*  598 */  "get_and_addLi",
  /*  599 */  "get_and_addLi_no_res",
  /*  600 */  "get_and_addI",
  /*  601 */  "get_and_addI_no_res",
  /*  602 */  "get_and_addIi",
  /*  603 */  "get_and_addIi_no_res",
  /*  604 */  "get_and_addLAcq",
  /*  605 */  "get_and_addL_no_resAcq",
  /*  606 */  "get_and_addLiAcq",
  /*  607 */  "get_and_addLi_no_resAcq",
  /*  608 */  "get_and_addIAcq",
  /*  609 */  "get_and_addI_no_resAcq",
  /*  610 */  "get_and_addIiAcq",
  /*  611 */  "get_and_addIi_no_resAcq",
  /*  612 */  "cmpL3_reg_reg",
  /*  613 */  "cmpL3_reg_imm",
  /*  614 */  "cmovI_reg_reg",
  /*  615 */  "cmovUI_reg_reg",
  /*  616 */  "cmovI_zero_reg",
  /*  617 */  "cmovUI_zero_reg",
  /*  618 */  "cmovI_reg_zero",
  /*  619 */  "cmovUI_reg_zero",
  /*  620 */  "cmovI_reg_zero_one",
  /*  621 */  "cmovUI_reg_zero_one",
  /*  622 */  "cmovL_reg_reg",
  /*  623 */  "cmovUL_reg_reg",
  /*  624 */  "cmovL_reg_zero",
  /*  625 */  "cmovUL_reg_zero",
  /*  626 */  "cmovL_zero_reg",
  /*  627 */  "cmovUL_zero_reg",
  /*  628 */  "cmovP_reg_reg",
  /*  629 */  "cmovUP_reg_reg",
  /*  630 */  "cmovP_reg_zero",
  /*  631 */  "cmovUP_reg_zero",
  /*  632 */  "cmovP_zero_reg",
  /*  633 */  "cmovUP_zero_reg",
  /*  634 */  "cmovN_reg_reg",
  /*  635 */  "cmovUN_reg_reg",
  /*  636 */  "cmovN_reg_zero",
  /*  637 */  "cmovUN_reg_zero",
  /*  638 */  "cmovN_zero_reg",
  /*  639 */  "cmovUN_zero_reg",
  /*  640 */  "cmovF_reg",
  /*  641 */  "cmovUF_reg",
  /*  642 */  "cmovD_reg",
  /*  643 */  "cmovUD_reg",
  /*  644 */  "addI_reg_reg",
  /*  645 */  "addI_reg_imm",
  /*  646 */  "addI_reg_imm_i2l",
  /*  647 */  "addP_reg_reg",
  /*  648 */  "addP_reg_reg_ext",
  /*  649 */  "addP_reg_reg_lsl",
  /*  650 */  "addP_reg_reg_ext_shift",
  /*  651 */  "lshift_ext",
  /*  652 */  "addP_reg_imm",
  /*  653 */  "addL_reg_reg",
  /*  654 */  "addL_reg_imm",
  /*  655 */  "subI_reg_reg",
  /*  656 */  "subI_reg_imm",
  /*  657 */  "subL_reg_reg",
  /*  658 */  "subL_reg_imm",
  /*  659 */  "negI_reg",
  /*  660 */  "negL_reg",
  /*  661 */  "mulI",
  /*  662 */  "smulI",
  /*  663 */  "smulI_0",
  /*  664 */  "mulL",
  /*  665 */  "mulHiL_rReg",
  /*  666 */  "maddI",
  /*  667 */  "maddI_0",
  /*  668 */  "msubI",
  /*  669 */  "mnegI",
  /*  670 */  "mnegI_0",
  /*  671 */  "mnegI_1",
  /*  672 */  "mnegI_2",
  /*  673 */  "maddL",
  /*  674 */  "maddL_0",
  /*  675 */  "msubL",
  /*  676 */  "mnegL",
  /*  677 */  "mnegL_0",
  /*  678 */  "mnegL_1",
  /*  679 */  "mnegL_2",
  /*  680 */  "smaddL",
  /*  681 */  "smaddL_1",
  /*  682 */  "smaddL_0",
  /*  683 */  "smaddL_2",
  /*  684 */  "smsubL",
  /*  685 */  "smsubL_0",
  /*  686 */  "smnegL",
  /*  687 */  "smnegL_0",
  /*  688 */  "smnegL_1",
  /*  689 */  "smnegL_2",
  /*  690 */  "muladdS2I",
  /*  691 */  "divI",
  /*  692 */  "divL",
  /*  693 */  "modI",
  /*  694 */  "modL",
  /*  695 */  "lShiftI_reg_reg",
  /*  696 */  "lShiftI_reg_imm",
  /*  697 */  "urShiftI_reg_reg",
  /*  698 */  "urShiftI_reg_imm",
  /*  699 */  "rShiftI_reg_reg",
  /*  700 */  "rShiftI_reg_imm",
  /*  701 */  "lShiftL_reg_reg",
  /*  702 */  "lShiftL_reg_imm",
  /*  703 */  "urShiftL_reg_reg",
  /*  704 */  "urShiftL_reg_imm",
  /*  705 */  "urShiftP_reg_imm",
  /*  706 */  "rShiftL_reg_reg",
  /*  707 */  "rShiftL_reg_imm",
  /*  708 */  "regL_not_reg",
  /*  709 */  "regI_not_reg",
  /*  710 */  "AndI_reg_not_reg",
  /*  711 */  "AndI_reg_not_reg_0",
  /*  712 */  "AndL_reg_not_reg",
  /*  713 */  "AndL_reg_not_reg_0",
  /*  714 */  "OrI_reg_not_reg",
  /*  715 */  "OrI_reg_not_reg_0",
  /*  716 */  "OrL_reg_not_reg",
  /*  717 */  "OrL_reg_not_reg_0",
  /*  718 */  "XorI_reg_not_reg",
  /*  719 */  "XorI_reg_not_reg_0",
  /*  720 */  "XorL_reg_not_reg",
  /*  721 */  "XorL_reg_not_reg_0",
  /*  722 */  "AndI_reg_URShift_not_reg",
  /*  723 */  "AndI_reg_URShift_not_reg_0",
  /*  724 */  "AndL_reg_URShift_not_reg",
  /*  725 */  "AndL_reg_URShift_not_reg_0",
  /*  726 */  "AndI_reg_RShift_not_reg",
  /*  727 */  "AndI_reg_RShift_not_reg_0",
  /*  728 */  "AndL_reg_RShift_not_reg",
  /*  729 */  "AndL_reg_RShift_not_reg_0",
  /*  730 */  "AndI_reg_LShift_not_reg",
  /*  731 */  "AndI_reg_LShift_not_reg_0",
  /*  732 */  "AndL_reg_LShift_not_reg",
  /*  733 */  "AndL_reg_LShift_not_reg_0",
  /*  734 */  "XorI_reg_URShift_not_reg",
  /*  735 */  "XorI_reg_URShift_not_reg_1",
  /*  736 */  "XorI_reg_URShift_not_reg_0",
  /*  737 */  "XorI_reg_URShift_not_reg_2",
  /*  738 */  "XorL_reg_URShift_not_reg",
  /*  739 */  "XorL_reg_URShift_not_reg_1",
  /*  740 */  "XorL_reg_URShift_not_reg_0",
  /*  741 */  "XorL_reg_URShift_not_reg_2",
  /*  742 */  "XorI_reg_RShift_not_reg",
  /*  743 */  "XorI_reg_RShift_not_reg_1",
  /*  744 */  "XorI_reg_RShift_not_reg_0",
  /*  745 */  "XorI_reg_RShift_not_reg_2",
  /*  746 */  "XorL_reg_RShift_not_reg",
  /*  747 */  "XorL_reg_RShift_not_reg_1",
  /*  748 */  "XorL_reg_RShift_not_reg_0",
  /*  749 */  "XorL_reg_RShift_not_reg_2",
  /*  750 */  "XorI_reg_LShift_not_reg",
  /*  751 */  "XorI_reg_LShift_not_reg_1",
  /*  752 */  "XorI_reg_LShift_not_reg_0",
  /*  753 */  "XorI_reg_LShift_not_reg_2",
  /*  754 */  "XorL_reg_LShift_not_reg",
  /*  755 */  "XorL_reg_LShift_not_reg_1",
  /*  756 */  "XorL_reg_LShift_not_reg_0",
  /*  757 */  "XorL_reg_LShift_not_reg_2",
  /*  758 */  "OrI_reg_URShift_not_reg",
  /*  759 */  "OrI_reg_URShift_not_reg_0",
  /*  760 */  "OrL_reg_URShift_not_reg",
  /*  761 */  "OrL_reg_URShift_not_reg_0",
  /*  762 */  "OrI_reg_RShift_not_reg",
  /*  763 */  "OrI_reg_RShift_not_reg_0",
  /*  764 */  "OrL_reg_RShift_not_reg",
  /*  765 */  "OrL_reg_RShift_not_reg_0",
  /*  766 */  "OrI_reg_LShift_not_reg",
  /*  767 */  "OrI_reg_LShift_not_reg_0",
  /*  768 */  "OrL_reg_LShift_not_reg",
  /*  769 */  "OrL_reg_LShift_not_reg_0",
  /*  770 */  "AndI_reg_URShift_reg",
  /*  771 */  "AndI_reg_URShift_reg_0",
  /*  772 */  "AndL_reg_URShift_reg",
  /*  773 */  "AndL_reg_URShift_reg_0",
  /*  774 */  "AndI_reg_RShift_reg",
  /*  775 */  "AndI_reg_RShift_reg_0",
  /*  776 */  "AndL_reg_RShift_reg",
  /*  777 */  "AndL_reg_RShift_reg_0",
  /*  778 */  "AndI_reg_LShift_reg",
  /*  779 */  "AndI_reg_LShift_reg_0",
  /*  780 */  "AndL_reg_LShift_reg",
  /*  781 */  "AndL_reg_LShift_reg_0",
  /*  782 */  "XorI_reg_URShift_reg",
  /*  783 */  "XorI_reg_URShift_reg_0",
  /*  784 */  "XorL_reg_URShift_reg",
  /*  785 */  "XorL_reg_URShift_reg_0",
  /*  786 */  "XorI_reg_RShift_reg",
  /*  787 */  "XorI_reg_RShift_reg_0",
  /*  788 */  "XorL_reg_RShift_reg",
  /*  789 */  "XorL_reg_RShift_reg_0",
  /*  790 */  "XorI_reg_LShift_reg",
  /*  791 */  "XorI_reg_LShift_reg_0",
  /*  792 */  "XorL_reg_LShift_reg",
  /*  793 */  "XorL_reg_LShift_reg_0",
  /*  794 */  "OrI_reg_URShift_reg",
  /*  795 */  "OrI_reg_URShift_reg_0",
  /*  796 */  "OrL_reg_URShift_reg",
  /*  797 */  "OrL_reg_URShift_reg_0",
  /*  798 */  "OrI_reg_RShift_reg",
  /*  799 */  "OrI_reg_RShift_reg_0",
  /*  800 */  "OrL_reg_RShift_reg",
  /*  801 */  "OrL_reg_RShift_reg_0",
  /*  802 */  "OrI_reg_LShift_reg",
  /*  803 */  "OrI_reg_LShift_reg_0",
  /*  804 */  "OrL_reg_LShift_reg",
  /*  805 */  "OrL_reg_LShift_reg_0",
  /*  806 */  "AddI_reg_URShift_reg",
  /*  807 */  "AddI_reg_URShift_reg_0",
  /*  808 */  "AddL_reg_URShift_reg",
  /*  809 */  "AddL_reg_URShift_reg_0",
  /*  810 */  "AddI_reg_RShift_reg",
  /*  811 */  "AddI_reg_RShift_reg_0",
  /*  812 */  "AddL_reg_RShift_reg",
  /*  813 */  "AddL_reg_RShift_reg_0",
  /*  814 */  "AddI_reg_LShift_reg",
  /*  815 */  "AddI_reg_LShift_reg_0",
  /*  816 */  "AddL_reg_LShift_reg",
  /*  817 */  "AddL_reg_LShift_reg_0",
  /*  818 */  "SubI_reg_URShift_reg",
  /*  819 */  "SubL_reg_URShift_reg",
  /*  820 */  "SubI_reg_RShift_reg",
  /*  821 */  "SubL_reg_RShift_reg",
  /*  822 */  "SubI_reg_LShift_reg",
  /*  823 */  "SubL_reg_LShift_reg",
  /*  824 */  "sbfmL",
  /*  825 */  "sbfmwI",
  /*  826 */  "ubfmL",
  /*  827 */  "ubfmwI",
  /*  828 */  "ubfxwI",
  /*  829 */  "ubfxL",
  /*  830 */  "ubfxIConvI2L",
  /*  831 */  "ubfizwI",
  /*  832 */  "ubfizL",
  /*  833 */  "ubfizIConvI2L",
  /*  834 */  "extrOrL",
  /*  835 */  "extrOrL_0",
  /*  836 */  "extrOrI",
  /*  837 */  "extrOrI_0",
  /*  838 */  "extrAddL",
  /*  839 */  "extrAddL_0",
  /*  840 */  "extrAddI",
  /*  841 */  "extrAddI_0",
  /*  842 */  "rolL_rReg",
  /*  843 */  "rolI_rReg",
  /*  844 */  "rolL_rReg_Var_C_64",
  /*  845 */  "rolL_rReg_Var_C_64_0",
  /*  846 */  "rolL_rReg_Var_C0",
  /*  847 */  "rolL_rReg_Var_C0_0",
  /*  848 */  "rolI_rReg_Var_C_32",
  /*  849 */  "rolI_rReg_Var_C_32_0",
  /*  850 */  "rolI_rReg_Var_C0",
  /*  851 */  "rolI_rReg_Var_C0_0",
  /*  852 */  "rorL_rReg",
  /*  853 */  "rorI_rReg",
  /*  854 */  "rorL_rReg_Var_C_64",
  /*  855 */  "rorL_rReg_Var_C_64_0",
  /*  856 */  "rorL_rReg_Var_C0",
  /*  857 */  "rorL_rReg_Var_C0_0",
  /*  858 */  "rorI_rReg_Var_C_32",
  /*  859 */  "rorI_rReg_Var_C_32_0",
  /*  860 */  "rorI_rReg_Var_C0",
  /*  861 */  "rorI_rReg_Var_C0_0",
  /*  862 */  "AddExtI",
  /*  863 */  "AddExtI_0",
  /*  864 */  "SubExtI",
  /*  865 */  "AddExtI_sxth",
  /*  866 */  "AddExtI_sxth_0",
  /*  867 */  "AddExtI_sxtb",
  /*  868 */  "AddExtI_sxtb_0",
  /*  869 */  "AddExtI_uxtb",
  /*  870 */  "AddExtI_uxtb_0",
  /*  871 */  "AddExtL_sxth",
  /*  872 */  "AddExtL_sxth_0",
  /*  873 */  "AddExtL_sxtw",
  /*  874 */  "AddExtL_sxtw_0",
  /*  875 */  "AddExtL_sxtb",
  /*  876 */  "AddExtL_sxtb_0",
  /*  877 */  "AddExtL_uxtb",
  /*  878 */  "AddExtL_uxtb_0",
  /*  879 */  "AddExtI_uxtb_and",
  /*  880 */  "AddExtI_uxtb_and_0",
  /*  881 */  "AddExtI_uxth_and",
  /*  882 */  "AddExtI_uxth_and_0",
  /*  883 */  "AddExtL_uxtb_and",
  /*  884 */  "AddExtL_uxtb_and_0",
  /*  885 */  "AddExtL_uxth_and",
  /*  886 */  "AddExtL_uxth_and_0",
  /*  887 */  "AddExtL_uxtw_and",
  /*  888 */  "AddExtL_uxtw_and_0",
  /*  889 */  "SubExtI_uxtb_and",
  /*  890 */  "SubExtI_uxth_and",
  /*  891 */  "SubExtL_uxtb_and",
  /*  892 */  "SubExtL_uxth_and",
  /*  893 */  "SubExtL_uxtw_and",
  /*  894 */  "AddExtL_sxtb_shift",
  /*  895 */  "AddExtL_sxtb_shift_0",
  /*  896 */  "AddExtL_sxth_shift",
  /*  897 */  "AddExtL_sxth_shift_0",
  /*  898 */  "AddExtL_sxtw_shift",
  /*  899 */  "AddExtL_sxtw_shift_0",
  /*  900 */  "SubExtL_sxtb_shift",
  /*  901 */  "SubExtL_sxth_shift",
  /*  902 */  "SubExtL_sxtw_shift",
  /*  903 */  "AddExtI_sxtb_shift",
  /*  904 */  "AddExtI_sxtb_shift_0",
  /*  905 */  "AddExtI_sxth_shift",
  /*  906 */  "AddExtI_sxth_shift_0",
  /*  907 */  "SubExtI_sxtb_shift",
  /*  908 */  "SubExtI_sxth_shift",
  /*  909 */  "AddExtI_shift",
  /*  910 */  "AddExtI_shift_0",
  /*  911 */  "SubExtI_shift",
  /*  912 */  "AddExtL_uxtb_and_shift",
  /*  913 */  "AddExtL_uxtb_and_shift_0",
  /*  914 */  "AddExtL_uxth_and_shift",
  /*  915 */  "AddExtL_uxth_and_shift_0",
  /*  916 */  "AddExtL_uxtw_and_shift",
  /*  917 */  "AddExtL_uxtw_and_shift_0",
  /*  918 */  "SubExtL_uxtb_and_shift",
  /*  919 */  "SubExtL_uxth_and_shift",
  /*  920 */  "SubExtL_uxtw_and_shift",
  /*  921 */  "AddExtI_uxtb_and_shift",
  /*  922 */  "AddExtI_uxtb_and_shift_0",
  /*  923 */  "AddExtI_uxth_and_shift",
  /*  924 */  "AddExtI_uxth_and_shift_0",
  /*  925 */  "SubExtI_uxtb_and_shift",
  /*  926 */  "SubExtI_uxth_and_shift",
  /*  927 */  "addF_reg_reg",
  /*  928 */  "addD_reg_reg",
  /*  929 */  "subF_reg_reg",
  /*  930 */  "subD_reg_reg",
  /*  931 */  "mulF_reg_reg",
  /*  932 */  "mulD_reg_reg",
  /*  933 */  "maddF_reg_reg",
  /*  934 */  "maddD_reg_reg",
  /*  935 */  "msubF_reg_reg",
  /*  936 */  "msubF_reg_reg_0",
  /*  937 */  "msubD_reg_reg",
  /*  938 */  "msubD_reg_reg_0",
  /*  939 */  "mnaddF_reg_reg",
  /*  940 */  "mnaddF_reg_reg_0",
  /*  941 */  "mnaddD_reg_reg",
  /*  942 */  "mnaddD_reg_reg_0",
  /*  943 */  "mnsubF_reg_reg",
  /*  944 */  "mnsubD_reg_reg",
  /*  945 */  "maxF_reg_reg",
  /*  946 */  "minF_reg_reg",
  /*  947 */  "maxD_reg_reg",
  /*  948 */  "minD_reg_reg",
  /*  949 */  "divF_reg_reg",
  /*  950 */  "divD_reg_reg",
  /*  951 */  "absI_reg",
  /*  952 */  "absL_reg",
  /*  953 */  "sqrtD_reg",
  /*  954 */  "sqrtF_reg",
  /*  955 */  "roundD_reg",
  /*  956 */  "andI_reg_reg",
  /*  957 */  "andI_reg_imm",
  /*  958 */  "orI_reg_reg",
  /*  959 */  "orI_reg_imm",
  /*  960 */  "xorI_reg_reg",
  /*  961 */  "xorI_reg_imm",
  /*  962 */  "andL_reg_reg",
  /*  963 */  "andL_reg_imm",
  /*  964 */  "orL_reg_reg",
  /*  965 */  "orL_reg_imm",
  /*  966 */  "xorL_reg_reg",
  /*  967 */  "xorL_reg_imm",
  /*  968 */  "convI2L_reg_reg",
  /*  969 */  "convUI2L_reg_reg",
  /*  970 */  "convL2I_reg",
  /*  971 */  "convI2B",
  /*  972 */  "convP2B",
  /*  973 */  "convD2F_reg",
  /*  974 */  "convF2D_reg",
  /*  975 */  "convF2I_reg_reg",
  /*  976 */  "convF2L_reg_reg",
  /*  977 */  "convI2F_reg_reg",
  /*  978 */  "convL2F_reg_reg",
  /*  979 */  "convD2I_reg_reg",
  /*  980 */  "convD2L_reg_reg",
  /*  981 */  "convI2D_reg_reg",
  /*  982 */  "convL2D_reg_reg",
  /*  983 */  "MoveF2I_reg_stack",
  /*  984 */  "MoveI2F_reg_stack",
  /*  985 */  "MoveD2L_reg_stack",
  /*  986 */  "MoveL2D_reg_stack",
  /*  987 */  "clearArray_reg_reg",
  /*  988 */  "clearArray_imm_reg",
  /*  989 */  "overflowMulI_reg_branch",
  /*  990 */  "overflowMulL_reg_branch",
  /*  991 */  "compF3_reg_reg",
  /*  992 */  "compD3_reg_reg",
  /*  993 */  "compF3_reg_immF0",
  /*  994 */  "compD3_reg_immD0",
  /*  995 */  "cmpLTMask_reg_reg",
  /*  996 */  "cmpLTMask_reg_zero",
  /*  997 */  "cmovI_reg_reg_lt",
  /*  998 */  "minI_rReg",
  /*  999 */  "cmovI_reg_reg_gt",
  /* 1000 */  "maxI_rReg",
  /* 1001 */  "branch",
  /* 1002 */  "branchCon",
  /* 1003 */  "branchConU",
  /* 1004 */  "cmpI_imm0_branch",
  /* 1005 */  "cmpL_imm0_branch",
  /* 1006 */  "cmpP_imm0_branch",
  /* 1007 */  "cmpN_imm0_branch",
  /* 1008 */  "cmpP_narrowOop_imm0_branch",
  /* 1009 */  "cmpUI_imm0_branch",
  /* 1010 */  "cmpUL_imm0_branch",
  /* 1011 */  "cmpL_branch_sign",
  /* 1012 */  "cmpI_branch_sign",
  /* 1013 */  "cmpL_branch_bit",
  /* 1014 */  "cmpI_branch_bit",
  /* 1015 */  "far_cmpL_branch_sign",
  /* 1016 */  "far_cmpI_branch_sign",
  /* 1017 */  "far_cmpL_branch_bit",
  /* 1018 */  "far_cmpI_branch_bit",
  /* 1019 */  "branchLoopEnd",
  /* 1020 */  "branchLoopEndU",
  /* 1021 */  "safePoint",
  /* 1022 */  "CallStaticJavaDirect",
  /* 1023 */  "CallDynamicJavaDirect",
  /* 1024 */  "CallRuntimeDirect",
  /* 1025 */  "CallLeafDirect",
  /* 1026 */  "CallLeafNoFPDirect",
  /* 1027 */  "TailCalljmpInd",
  /* 1028 */  "TailjmpInd",
  /* 1029 */  "CreateException",
  /* 1030 */  "RethrowException",
  /* 1031 */  "Ret",
  /* 1032 */  "ShouldNotReachHere",
  /* 1033 */  "partialSubtypeCheck",
  /* 1034 */  "string_compareU",
  /* 1035 */  "string_compareL",
  /* 1036 */  "string_compareUL",
  /* 1037 */  "string_compareLU",
  /* 1038 */  "string_indexofUU",
  /* 1039 */  "string_indexofLL",
  /* 1040 */  "string_indexofUL",
  /* 1041 */  "string_indexof_conUU",
  /* 1042 */  "string_indexof_conLL",
  /* 1043 */  "string_indexof_conUL",
  /* 1044 */  "string_indexofU_char",
  /* 1045 */  "string_equalsL",
  /* 1046 */  "string_equalsU",
  /* 1047 */  "array_equalsB",
  /* 1048 */  "array_equalsC",
  /* 1049 */  "has_negatives",
  /* 1050 */  "string_compress",
  /* 1051 */  "string_inflate",
  /* 1052 */  "encode_iso_array",
  /* 1053 */  "loadV4",
  /* 1054 */  "loadV8",
  /* 1055 */  "loadV16",
  /* 1056 */  "storeV4",
  /* 1057 */  "storeV8",
  /* 1058 */  "storeV16",
  /* 1059 */  "replicate8B",
  /* 1060 */  "replicate16B",
  /* 1061 */  "replicate8B_imm",
  /* 1062 */  "replicate16B_imm",
  /* 1063 */  "replicate4S",
  /* 1064 */  "replicate8S",
  /* 1065 */  "replicate4S_imm",
  /* 1066 */  "replicate8S_imm",
  /* 1067 */  "replicate2I",
  /* 1068 */  "replicate4I",
  /* 1069 */  "replicate2I_imm",
  /* 1070 */  "replicate4I_imm",
  /* 1071 */  "replicate2L",
  /* 1072 */  "replicate2L_zero",
  /* 1073 */  "replicate2F",
  /* 1074 */  "replicate4F",
  /* 1075 */  "replicate2D",
  /* 1076 */  "reduce_add2I",
  /* 1077 */  "reduce_add4I",
  /* 1078 */  "reduce_mul2I",
  /* 1079 */  "reduce_mul4I",
  /* 1080 */  "reduce_add2F",
  /* 1081 */  "reduce_add4F",
  /* 1082 */  "reduce_mul2F",
  /* 1083 */  "reduce_mul4F",
  /* 1084 */  "reduce_add2D",
  /* 1085 */  "reduce_mul2D",
  /* 1086 */  "reduce_max2F",
  /* 1087 */  "reduce_max4F",
  /* 1088 */  "reduce_max2D",
  /* 1089 */  "reduce_min2F",
  /* 1090 */  "reduce_min4F",
  /* 1091 */  "reduce_min2D",
  /* 1092 */  "vadd8B",
  /* 1093 */  "vadd16B",
  /* 1094 */  "vadd4S",
  /* 1095 */  "vadd8S",
  /* 1096 */  "vadd2I",
  /* 1097 */  "vadd4I",
  /* 1098 */  "vadd2L",
  /* 1099 */  "vadd2F",
  /* 1100 */  "vadd4F",
  /* 1101 */  "vadd2D",
  /* 1102 */  "vsub8B",
  /* 1103 */  "vsub16B",
  /* 1104 */  "vsub4S",
  /* 1105 */  "vsub8S",
  /* 1106 */  "vsub2I",
  /* 1107 */  "vsub4I",
  /* 1108 */  "vsub2L",
  /* 1109 */  "vsub2F",
  /* 1110 */  "vsub4F",
  /* 1111 */  "vsub2D",
  /* 1112 */  "vmul8B",
  /* 1113 */  "vmul16B",
  /* 1114 */  "vmul4S",
  /* 1115 */  "vmul8S",
  /* 1116 */  "vmul2I",
  /* 1117 */  "vmul4I",
  /* 1118 */  "vmul2F",
  /* 1119 */  "vmul4F",
  /* 1120 */  "vmul2D",
  /* 1121 */  "vmla4S",
  /* 1122 */  "vmla4S_0",
  /* 1123 */  "vmla8S",
  /* 1124 */  "vmla8S_0",
  /* 1125 */  "vmla2I",
  /* 1126 */  "vmla2I_0",
  /* 1127 */  "vmla4I",
  /* 1128 */  "vmla4I_0",
  /* 1129 */  "vmla2F",
  /* 1130 */  "vmla4F",
  /* 1131 */  "vmla2D",
  /* 1132 */  "vmls4S",
  /* 1133 */  "vmls8S",
  /* 1134 */  "vmls2I",
  /* 1135 */  "vmls4I",
  /* 1136 */  "vmls2F",
  /* 1137 */  "vmls2F_0",
  /* 1138 */  "vmls4F",
  /* 1139 */  "vmls4F_0",
  /* 1140 */  "vmls2D",
  /* 1141 */  "vmls2D_0",
  /* 1142 */  "vmuladdS2I",
  /* 1143 */  "vdiv2F",
  /* 1144 */  "vdiv4F",
  /* 1145 */  "vdiv2D",
  /* 1146 */  "vand8B",
  /* 1147 */  "vand16B",
  /* 1148 */  "vor8B",
  /* 1149 */  "vor16B",
  /* 1150 */  "vxor8B",
  /* 1151 */  "vxor16B",
  /* 1152 */  "vsll8B",
  /* 1153 */  "vsll16B",
  /* 1154 */  "vsra8B",
  /* 1155 */  "vsra16B",
  /* 1156 */  "vsrl8B",
  /* 1157 */  "vsrl16B",
  /* 1158 */  "vsll8B_imm",
  /* 1159 */  "vsll16B_imm",
  /* 1160 */  "vsra8B_imm",
  /* 1161 */  "vsra16B_imm",
  /* 1162 */  "vsrl8B_imm",
  /* 1163 */  "vsrl16B_imm",
  /* 1164 */  "vsll4S",
  /* 1165 */  "vsll8S",
  /* 1166 */  "vsra4S",
  /* 1167 */  "vsra8S",
  /* 1168 */  "vsrl4S",
  /* 1169 */  "vsrl8S",
  /* 1170 */  "vsll4S_imm",
  /* 1171 */  "vsll8S_imm",
  /* 1172 */  "vsra4S_imm",
  /* 1173 */  "vsra8S_imm",
  /* 1174 */  "vsrl4S_imm",
  /* 1175 */  "vsrl8S_imm",
  /* 1176 */  "vsll2I",
  /* 1177 */  "vsll4I",
  /* 1178 */  "vsra2I",
  /* 1179 */  "vsra4I",
  /* 1180 */  "vsrl2I",
  /* 1181 */  "vsrl4I",
  /* 1182 */  "vsll2I_imm",
  /* 1183 */  "vsll4I_imm",
  /* 1184 */  "vsra2I_imm",
  /* 1185 */  "vsra4I_imm",
  /* 1186 */  "vsrl2I_imm",
  /* 1187 */  "vsrl4I_imm",
  /* 1188 */  "vsll2L",
  /* 1189 */  "vsra2L",
  /* 1190 */  "vsrl2L",
  /* 1191 */  "vsll2L_imm",
  /* 1192 */  "vsra2L_imm",
  /* 1193 */  "vsrl2L_imm",
  /* 1194 */  "vmax2F",
  /* 1195 */  "vmax4F",
  /* 1196 */  "vmax2D",
  /* 1197 */  "vmin2F",
  /* 1198 */  "vmin4F",
  /* 1199 */  "vmin2D",
  /* 1200 */  "vround2D_reg",
  /* 1201 */  "compareAndSwapP_shenandoah",
  /* 1202 */  "compareAndSwapN_shenandoah",
  /* 1203 */  "compareAndSwapPAcq_shenandoah",
  /* 1204 */  "compareAndSwapNAcq_shenandoah",
  /* 1205 */  "compareAndExchangeN_shenandoah",
  /* 1206 */  "compareAndExchangeP_shenandoah",
  /* 1207 */  "compareAndExchangeNAcq_shenandoah",
  /* 1208 */  "compareAndExchangePAcq_shenandoah",
  /* 1209 */  "weakCompareAndSwapN_shenandoah",
  /* 1210 */  "weakCompareAndSwapP_shenandoah",
  /* 1211 */  "weakCompareAndSwapNAcq_shenandoah",
  /* 1212 */  "weakCompareAndSwapPAcq_shenandoah",
  /* 1213 */  "zLoadP",
  /* 1214 */  "zLoadWeakP",
  /* 1215 */  "zLoadPVolatile",
  /* 1216 */  "zCompareAndSwapP",
  /* 1217 */  "zCompareAndSwapP_0",
  /* 1218 */  "zCompareAndSwapPAcq",
  /* 1219 */  "zCompareAndSwapPAcq_0",
  /* 1220 */  "zCompareAndExchangeP",
  /* 1221 */  "zCompareAndExchangePAcq",
  /* 1222 */  "zGetAndSetP",
  /* 1223 */  "zGetAndSetPAcq",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  true,
  /*   47 */  true,
  /*   48 */  true,
  /*   49 */  true,
  /*   50 */  true,
  /*   51 */  true,
  /*   52 */  true,
  /*   53 */  true,
  /*   54 */  true,
  /*   55 */  true,
  /*   56 */  true,
  /*   57 */  true,
  /*   58 */  true,
  /*   59 */  true,
  /*   60 */  true,
  /*   61 */  true,
  /*   62 */  true,
  /*   63 */  true,
  /*   64 */  true,
  /*   65 */  true,
  /*   66 */  true,
  /*   67 */  true,
  /*   68 */  true,
  /*   69 */  true,
  /*   70 */  true,
  /*   71 */  true,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  true,
  /*  173 */  true,
  /*  174 */  true,
  /*  175 */  true,
  /*  176 */  true,
  /*  177 */  false,
  // last operand
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  // last operand class
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  // last internally defined operand
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  /*  949 */  false,
  /*  950 */  false,
  /*  951 */  false,
  /*  952 */  false,
  /*  953 */  false,
  /*  954 */  false,
  /*  955 */  false,
  /*  956 */  false,
  /*  957 */  false,
  /*  958 */  false,
  /*  959 */  false,
  /*  960 */  false,
  /*  961 */  false,
  /*  962 */  false,
  /*  963 */  false,
  /*  964 */  false,
  /*  965 */  false,
  /*  966 */  false,
  /*  967 */  false,
  /*  968 */  false,
  /*  969 */  false,
  /*  970 */  false,
  /*  971 */  false,
  /*  972 */  false,
  /*  973 */  false,
  /*  974 */  false,
  /*  975 */  false,
  /*  976 */  false,
  /*  977 */  false,
  /*  978 */  false,
  /*  979 */  false,
  /*  980 */  false,
  /*  981 */  false,
  /*  982 */  false,
  /*  983 */  false,
  /*  984 */  false,
  /*  985 */  false,
  /*  986 */  false,
  /*  987 */  false,
  /*  988 */  false,
  /*  989 */  false,
  /*  990 */  false,
  /*  991 */  false,
  /*  992 */  false,
  /*  993 */  false,
  /*  994 */  false,
  /*  995 */  false,
  /*  996 */  false,
  /*  997 */  false,
  /*  998 */  false,
  /*  999 */  false,
  /* 1000 */  false,
  /* 1001 */  false,
  /* 1002 */  false,
  /* 1003 */  false,
  /* 1004 */  false,
  /* 1005 */  false,
  /* 1006 */  false,
  /* 1007 */  false,
  /* 1008 */  false,
  /* 1009 */  false,
  /* 1010 */  false,
  /* 1011 */  false,
  /* 1012 */  false,
  /* 1013 */  false,
  /* 1014 */  false,
  /* 1015 */  false,
  /* 1016 */  false,
  /* 1017 */  false,
  /* 1018 */  false,
  /* 1019 */  false,
  /* 1020 */  false,
  /* 1021 */  false,
  /* 1022 */  false,
  /* 1023 */  false,
  /* 1024 */  false,
  /* 1025 */  false,
  /* 1026 */  false,
  /* 1027 */  false,
  /* 1028 */  false,
  /* 1029 */  false,
  /* 1030 */  false,
  /* 1031 */  false,
  /* 1032 */  false,
  /* 1033 */  false,
  /* 1034 */  false,
  /* 1035 */  false,
  /* 1036 */  false,
  /* 1037 */  false,
  /* 1038 */  false,
  /* 1039 */  false,
  /* 1040 */  false,
  /* 1041 */  false,
  /* 1042 */  false,
  /* 1043 */  false,
  /* 1044 */  false,
  /* 1045 */  false,
  /* 1046 */  false,
  /* 1047 */  false,
  /* 1048 */  false,
  /* 1049 */  false,
  /* 1050 */  false,
  /* 1051 */  false,
  /* 1052 */  false,
  /* 1053 */  false,
  /* 1054 */  false,
  /* 1055 */  false,
  /* 1056 */  false,
  /* 1057 */  false,
  /* 1058 */  false,
  /* 1059 */  false,
  /* 1060 */  false,
  /* 1061 */  false,
  /* 1062 */  false,
  /* 1063 */  false,
  /* 1064 */  false,
  /* 1065 */  false,
  /* 1066 */  false,
  /* 1067 */  false,
  /* 1068 */  false,
  /* 1069 */  false,
  /* 1070 */  false,
  /* 1071 */  false,
  /* 1072 */  false,
  /* 1073 */  false,
  /* 1074 */  false,
  /* 1075 */  false,
  /* 1076 */  false,
  /* 1077 */  false,
  /* 1078 */  false,
  /* 1079 */  false,
  /* 1080 */  false,
  /* 1081 */  false,
  /* 1082 */  false,
  /* 1083 */  false,
  /* 1084 */  false,
  /* 1085 */  false,
  /* 1086 */  false,
  /* 1087 */  false,
  /* 1088 */  false,
  /* 1089 */  false,
  /* 1090 */  false,
  /* 1091 */  false,
  /* 1092 */  false,
  /* 1093 */  false,
  /* 1094 */  false,
  /* 1095 */  false,
  /* 1096 */  false,
  /* 1097 */  false,
  /* 1098 */  false,
  /* 1099 */  false,
  /* 1100 */  false,
  /* 1101 */  false,
  /* 1102 */  false,
  /* 1103 */  false,
  /* 1104 */  false,
  /* 1105 */  false,
  /* 1106 */  false,
  /* 1107 */  false,
  /* 1108 */  false,
  /* 1109 */  false,
  /* 1110 */  false,
  /* 1111 */  false,
  /* 1112 */  false,
  /* 1113 */  false,
  /* 1114 */  false,
  /* 1115 */  false,
  /* 1116 */  false,
  /* 1117 */  false,
  /* 1118 */  false,
  /* 1119 */  false,
  /* 1120 */  false,
  /* 1121 */  false,
  /* 1122 */  false,
  /* 1123 */  false,
  /* 1124 */  false,
  /* 1125 */  false,
  /* 1126 */  false,
  /* 1127 */  false,
  /* 1128 */  false,
  /* 1129 */  false,
  /* 1130 */  false,
  /* 1131 */  false,
  /* 1132 */  false,
  /* 1133 */  false,
  /* 1134 */  false,
  /* 1135 */  false,
  /* 1136 */  false,
  /* 1137 */  false,
  /* 1138 */  false,
  /* 1139 */  false,
  /* 1140 */  false,
  /* 1141 */  false,
  /* 1142 */  false,
  /* 1143 */  false,
  /* 1144 */  false,
  /* 1145 */  false,
  /* 1146 */  false,
  /* 1147 */  false,
  /* 1148 */  false,
  /* 1149 */  false,
  /* 1150 */  false,
  /* 1151 */  false,
  /* 1152 */  false,
  /* 1153 */  false,
  /* 1154 */  false,
  /* 1155 */  false,
  /* 1156 */  false,
  /* 1157 */  false,
  /* 1158 */  false,
  /* 1159 */  false,
  /* 1160 */  false,
  /* 1161 */  false,
  /* 1162 */  false,
  /* 1163 */  false,
  /* 1164 */  false,
  /* 1165 */  false,
  /* 1166 */  false,
  /* 1167 */  false,
  /* 1168 */  false,
  /* 1169 */  false,
  /* 1170 */  false,
  /* 1171 */  false,
  /* 1172 */  false,
  /* 1173 */  false,
  /* 1174 */  false,
  /* 1175 */  false,
  /* 1176 */  false,
  /* 1177 */  false,
  /* 1178 */  false,
  /* 1179 */  false,
  /* 1180 */  false,
  /* 1181 */  false,
  /* 1182 */  false,
  /* 1183 */  false,
  /* 1184 */  false,
  /* 1185 */  false,
  /* 1186 */  false,
  /* 1187 */  false,
  /* 1188 */  false,
  /* 1189 */  false,
  /* 1190 */  false,
  /* 1191 */  false,
  /* 1192 */  false,
  /* 1193 */  false,
  /* 1194 */  false,
  /* 1195 */  false,
  /* 1196 */  false,
  /* 1197 */  false,
  /* 1198 */  false,
  /* 1199 */  false,
  /* 1200 */  false,
  /* 1201 */  false,
  /* 1202 */  false,
  /* 1203 */  false,
  /* 1204 */  false,
  /* 1205 */  false,
  /* 1206 */  false,
  /* 1207 */  false,
  /* 1208 */  false,
  /* 1209 */  false,
  /* 1210 */  false,
  /* 1211 */  false,
  /* 1212 */  false,
  /* 1213 */  false,
  /* 1214 */  false,
  /* 1215 */  false,
  /* 1216 */  false,
  /* 1217 */  false,
  /* 1218 */  false,
  /* 1219 */  false,
  /* 1220 */  false,
  /* 1221 */  false,
  /* 1222 */  false,
  /* 1223 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // _last_machine_leaf: 14
  0, // AbsD: 15
  0, // AbsF: 16
  0, // AbsI: 17
  0, // AbsL: 18
  0, // AddD: 19
  0, // AddF: 20
  0, // AddI: 21
  0, // AddL: 22
  0, // AddP: 23
  0, // Allocate: 24
  0, // AllocateArray: 25
  0, // AndI: 26
  0, // AndL: 27
  0, // ArrayCopy: 28
  0, // AryEq: 29
  0, // AtanD: 30
  1, // Binary: 31
  1, // Bool: 32
  0, // BoxLock: 33
  0, // ReverseBytesI: 34
  0, // ReverseBytesL: 35
  0, // ReverseBytesUS: 36
  0, // ReverseBytesS: 37
  0, // CProj: 38
  0, // CacheWB: 39
  0, // CacheWBPreSync: 40
  0, // CacheWBPostSync: 41
  0, // CallDynamicJava: 42
  0, // CallJava: 43
  0, // CallLeaf: 44
  0, // CallLeafNoFP: 45
  0, // CallRuntime: 46
  0, // CallStaticJava: 47
  0, // CastII: 48
  0, // CastX2P: 49
  0, // CastP2X: 50
  0, // CastPP: 51
  0, // Catch: 52
  0, // CatchProj: 53
  0, // CheckCastPP: 54
  0, // ClearArray: 55
  0, // ConstraintCast: 56
  0, // CMoveD: 57
  0, // CMoveVD: 58
  0, // CMoveF: 59
  0, // CMoveVF: 60
  0, // CMoveI: 61
  0, // CMoveL: 62
  0, // CMoveP: 63
  0, // CMoveN: 64
  1, // CmpN: 65
  1, // CmpD: 66
  0, // CmpD3: 67
  1, // CmpF: 68
  0, // CmpF3: 69
  1, // CmpI: 70
  1, // CmpL: 71
  0, // CmpL3: 72
  0, // CmpLTMask: 73
  1, // CmpP: 74
  1, // CmpU: 75
  1, // CmpUL: 76
  0, // CompareAndSwapB: 77
  0, // CompareAndSwapS: 78
  0, // CompareAndSwapI: 79
  0, // CompareAndSwapL: 80
  0, // CompareAndSwapP: 81
  0, // CompareAndSwapN: 82
  0, // WeakCompareAndSwapB: 83
  0, // WeakCompareAndSwapS: 84
  0, // WeakCompareAndSwapI: 85
  0, // WeakCompareAndSwapL: 86
  0, // WeakCompareAndSwapP: 87
  0, // WeakCompareAndSwapN: 88
  0, // CompareAndExchangeB: 89
  0, // CompareAndExchangeS: 90
  0, // CompareAndExchangeI: 91
  0, // CompareAndExchangeL: 92
  0, // CompareAndExchangeP: 93
  0, // CompareAndExchangeN: 94
  0, // GetAndAddB: 95
  0, // GetAndAddS: 96
  0, // GetAndAddI: 97
  0, // GetAndAddL: 98
  0, // GetAndSetB: 99
  0, // GetAndSetS: 100
  0, // GetAndSetI: 101
  0, // GetAndSetL: 102
  0, // GetAndSetP: 103
  0, // GetAndSetN: 104
  0, // Con: 105
  0, // ConN: 106
  0, // ConNKlass: 107
  0, // ConD: 108
  0, // ConF: 109
  0, // ConI: 110
  0, // ConL: 111
  0, // ConP: 112
  0, // Conv2B: 113
  0, // ConvD2F: 114
  0, // ConvD2I: 115
  0, // ConvD2L: 116
  0, // ConvF2D: 117
  0, // ConvF2I: 118
  0, // ConvF2L: 119
  0, // ConvI2D: 120
  0, // ConvI2F: 121
  0, // ConvI2L: 122
  0, // ConvL2D: 123
  0, // ConvL2F: 124
  0, // ConvL2I: 125
  0, // CountedLoop: 126
  0, // CountedLoopEnd: 127
  0, // OuterStripMinedLoop: 128
  0, // OuterStripMinedLoopEnd: 129
  0, // CountLeadingZerosI: 130
  0, // CountLeadingZerosL: 131
  0, // CountTrailingZerosI: 132
  0, // CountTrailingZerosL: 133
  0, // CreateEx: 134
  0, // DecodeN: 135
  0, // DecodeNKlass: 136
  0, // DivD: 137
  0, // DivF: 138
  0, // DivI: 139
  0, // DivL: 140
  0, // DivMod: 141
  0, // DivModI: 142
  0, // DivModL: 143
  0, // EncodeISOArray: 144
  0, // EncodeP: 145
  0, // EncodePKlass: 146
  1, // FastLock: 147
  1, // FastUnlock: 148
  0, // FmaD: 149
  0, // FmaF: 150
  0, // Goto: 151
  0, // Halt: 152
  0, // HasNegatives: 153
  0, // If: 154
  0, // RangeCheck: 155
  0, // IfFalse: 156
  0, // IfTrue: 157
  0, // Initialize: 158
  0, // JProj: 159
  0, // Jump: 160
  0, // JumpProj: 161
  0, // LShiftI: 162
  0, // LShiftL: 163
  0, // LoadB: 164
  0, // LoadUB: 165
  0, // LoadUS: 166
  0, // LoadD: 167
  0, // LoadD_unaligned: 168
  0, // LoadF: 169
  0, // LoadI: 170
  0, // LoadKlass: 171
  0, // LoadNKlass: 172
  0, // LoadL: 173
  0, // LoadL_unaligned: 174
  0, // LoadPLocked: 175
  0, // LoadP: 176
  0, // LoadN: 177
  0, // LoadRange: 178
  0, // LoadS: 179
  0, // Lock: 180
  0, // Loop: 181
  0, // LoopLimit: 182
  0, // Mach: 183
  0, // MachProj: 184
  0, // MulAddS2I: 185
  0, // MaxD: 186
  0, // MaxF: 187
  0, // MaxI: 188
  0, // MemBarAcquire: 189
  0, // LoadFence: 190
  0, // SetVectMaskI: 191
  0, // MemBarAcquireLock: 192
  0, // MemBarCPUOrder: 193
  0, // MemBarRelease: 194
  0, // StoreFence: 195
  0, // MemBarReleaseLock: 196
  0, // MemBarVolatile: 197
  0, // MemBarStoreStore: 198
  0, // MergeMem: 199
  0, // MinD: 200
  0, // MinF: 201
  0, // MinI: 202
  0, // ModD: 203
  0, // ModF: 204
  0, // ModI: 205
  0, // ModL: 206
  0, // MoveI2F: 207
  0, // MoveF2I: 208
  0, // MoveL2D: 209
  0, // MoveD2L: 210
  0, // MulD: 211
  0, // MulF: 212
  0, // MulHiL: 213
  0, // MulI: 214
  0, // MulL: 215
  0, // Multi: 216
  0, // NegD: 217
  0, // NegF: 218
  0, // NeverBranch: 219
  0, // OnSpinWait: 220
  0, // Opaque1: 221
  0, // OpaqueLoopInit: 222
  0, // OpaqueLoopStride: 223
  0, // Opaque2: 224
  0, // Opaque3: 225
  0, // Opaque4: 226
  0, // ProfileBoolean: 227
  0, // OrI: 228
  0, // OrL: 229
  1, // OverflowAddI: 230
  1, // OverflowSubI: 231
  1, // OverflowMulI: 232
  1, // OverflowAddL: 233
  1, // OverflowSubL: 234
  1, // OverflowMulL: 235
  0, // PCTable: 236
  0, // Parm: 237
  0, // PartialSubtypeCheck: 238
  0, // SubTypeCheck: 239
  0, // Phi: 240
  0, // PopCountI: 241
  0, // PopCountL: 242
  0, // PopCountVI: 243
  0, // PrefetchAllocation: 244
  0, // Proj: 245
  0, // RShiftI: 246
  0, // RShiftL: 247
  0, // Region: 248
  0, // Rethrow: 249
  0, // Return: 250
  0, // Root: 251
  0, // RoundDouble: 252
  0, // RoundDoubleMode: 253
  0, // RoundDoubleModeV: 254
  0, // RoundFloat: 255
  0, // SafePoint: 256
  0, // SafePointScalarObject: 257
  0, // ShenandoahCompareAndExchangeP: 258
  0, // ShenandoahCompareAndExchangeN: 259
  0, // ShenandoahCompareAndSwapN: 260
  0, // ShenandoahCompareAndSwapP: 261
  0, // ShenandoahWeakCompareAndSwapN: 262
  0, // ShenandoahWeakCompareAndSwapP: 263
  0, // ShenandoahEnqueueBarrier: 264
  0, // ShenandoahLoadReferenceBarrier: 265
  0, // SCMemProj: 266
  0, // SqrtD: 267
  0, // SqrtF: 268
  0, // Start: 269
  0, // StartOSR: 270
  0, // StoreB: 271
  0, // StoreC: 272
  0, // StoreCM: 273
  0, // StorePConditional: 274
  0, // StoreIConditional: 275
  0, // StoreLConditional: 276
  0, // StoreD: 277
  0, // StoreF: 278
  0, // StoreI: 279
  0, // StoreL: 280
  0, // StoreP: 281
  0, // StoreN: 282
  0, // StoreNKlass: 283
  0, // StrComp: 284
  0, // StrCompressedCopy: 285
  0, // StrEquals: 286
  0, // StrIndexOf: 287
  0, // StrIndexOfChar: 288
  0, // StrInflatedCopy: 289
  0, // SubD: 290
  0, // SubF: 291
  0, // SubI: 292
  0, // SubL: 293
  0, // TailCall: 294
  0, // TailJump: 295
  0, // MacroLogicV: 296
  0, // ThreadLocal: 297
  0, // Unlock: 298
  0, // URShiftI: 299
  0, // URShiftL: 300
  0, // XorI: 301
  0, // XorL: 302
  0, // Vector: 303
  0, // AddVB: 304
  0, // AddVS: 305
  0, // AddVI: 306
  0, // AddReductionVI: 307
  0, // AddVL: 308
  0, // AddReductionVL: 309
  0, // AddVF: 310
  0, // AddReductionVF: 311
  0, // AddVD: 312
  0, // AddReductionVD: 313
  0, // SubVB: 314
  0, // SubVS: 315
  0, // SubVI: 316
  0, // SubVL: 317
  0, // SubVF: 318
  0, // SubVD: 319
  0, // MulVB: 320
  0, // MulVS: 321
  0, // MulVI: 322
  0, // MulReductionVI: 323
  0, // MulVL: 324
  0, // MulReductionVL: 325
  0, // MulVF: 326
  0, // MulReductionVF: 327
  0, // MulVD: 328
  0, // MulReductionVD: 329
  0, // MulAddVS2VI: 330
  0, // FmaVD: 331
  0, // FmaVF: 332
  0, // DivVF: 333
  0, // DivVD: 334
  0, // AbsVB: 335
  0, // AbsVS: 336
  0, // AbsVI: 337
  0, // AbsVL: 338
  0, // AbsVF: 339
  0, // AbsVD: 340
  0, // NegVF: 341
  0, // NegVD: 342
  0, // SqrtVD: 343
  0, // SqrtVF: 344
  0, // LShiftCntV: 345
  0, // RShiftCntV: 346
  0, // LShiftVB: 347
  0, // LShiftVS: 348
  0, // LShiftVI: 349
  0, // LShiftVL: 350
  0, // RShiftVB: 351
  0, // RShiftVS: 352
  0, // RShiftVI: 353
  0, // RShiftVL: 354
  0, // URShiftVB: 355
  0, // URShiftVS: 356
  0, // URShiftVI: 357
  0, // URShiftVL: 358
  0, // AndV: 359
  0, // AndReductionV: 360
  0, // OrV: 361
  0, // OrReductionV: 362
  0, // XorV: 363
  0, // XorReductionV: 364
  0, // MinV: 365
  0, // MaxV: 366
  0, // MinReductionV: 367
  0, // MaxReductionV: 368
  0, // LoadVector: 369
  0, // StoreVector: 370
  0, // Pack: 371
  0, // PackB: 372
  0, // PackS: 373
  0, // PackI: 374
  0, // PackL: 375
  0, // PackF: 376
  0, // PackD: 377
  0, // Pack2L: 378
  0, // Pack2D: 379
  0, // ReplicateB: 380
  0, // ReplicateS: 381
  0, // ReplicateI: 382
  0, // ReplicateL: 383
  0, // ReplicateF: 384
  0, // ReplicateD: 385
  0, // Extract: 386
  0, // ExtractB: 387
  0, // ExtractUB: 388
  0, // ExtractC: 389
  0, // ExtractS: 390
  0, // ExtractI: 391
  0, // ExtractL: 392
  0, // ExtractF: 393
  0, // ExtractD: 394
  0, // Digit: 395
  0, // LowerCase: 396
  0, // UpperCase: 397
  0 // no trailing comma // Whitespace: 398
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R12",
  "R12_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "R16",
  "R16_H",
  "R17",
  "R17_H",
  "R18",
  "R18_H",
  "R0",
  "R0_H",
  "R1",
  "R1_H",
  "R2",
  "R2_H",
  "R3",
  "R3_H",
  "R4",
  "R4_H",
  "R5",
  "R5_H",
  "R6",
  "R6_H",
  "R7",
  "R7_H",
  "R19",
  "R19_H",
  "R20",
  "R20_H",
  "R21",
  "R21_H",
  "R22",
  "R22_H",
  "R23",
  "R23_H",
  "R24",
  "R24_H",
  "R25",
  "R25_H",
  "R26",
  "R26_H",
  "R27",
  "R27_H",
  "R28",
  "R28_H",
  "R29",
  "R29_H",
  "R30",
  "R30_H",
  "R31",
  "R31_H",
  "V16",
  "V16_H",
  "V16_J",
  "V16_K",
  "V17",
  "V17_H",
  "V17_J",
  "V17_K",
  "V18",
  "V18_H",
  "V18_J",
  "V18_K",
  "V19",
  "V19_H",
  "V19_J",
  "V19_K",
  "V20",
  "V20_H",
  "V20_J",
  "V20_K",
  "V21",
  "V21_H",
  "V21_J",
  "V21_K",
  "V22",
  "V22_H",
  "V22_J",
  "V22_K",
  "V23",
  "V23_H",
  "V23_J",
  "V23_K",
  "V24",
  "V24_H",
  "V24_J",
  "V24_K",
  "V25",
  "V25_H",
  "V25_J",
  "V25_K",
  "V26",
  "V26_H",
  "V26_J",
  "V26_K",
  "V27",
  "V27_H",
  "V27_J",
  "V27_K",
  "V28",
  "V28_H",
  "V28_J",
  "V28_K",
  "V29",
  "V29_H",
  "V29_J",
  "V29_K",
  "V30",
  "V30_H",
  "V30_J",
  "V30_K",
  "V31",
  "V31_H",
  "V31_J",
  "V31_K",
  "V0",
  "V0_H",
  "V0_J",
  "V0_K",
  "V1",
  "V1_H",
  "V1_J",
  "V1_K",
  "V2",
  "V2_H",
  "V2_J",
  "V2_K",
  "V3",
  "V3_H",
  "V3_J",
  "V3_K",
  "V4",
  "V4_H",
  "V4_J",
  "V4_K",
  "V5",
  "V5_H",
  "V5_J",
  "V5_K",
  "V6",
  "V6_H",
  "V6_J",
  "V6_K",
  "V7",
  "V7_H",
  "V7_J",
  "V7_K",
  "V8",
  "V8_H",
  "V8_J",
  "V8_K",
  "V9",
  "V9_H",
  "V9_J",
  "V9_K",
  "V10",
  "V10_H",
  "V10_J",
  "V10_K",
  "V11",
  "V11_H",
  "V11_J",
  "V11_K",
  "V12",
  "V12_H",
  "V12_J",
  "V12_K",
  "V13",
  "V13_H",
  "V13_J",
  "V13_K",
  "V14",
  "V14_H",
  "V14_J",
  "V14_K",
  "V15",
  "V15_H",
  "V15_J",
  "V15_K",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	r10->as_VMReg()        ,
	r10->as_VMReg()->next(),
	r11->as_VMReg()        ,
	r11->as_VMReg()->next(),
	r12->as_VMReg()        ,
	r12->as_VMReg()->next(),
	r13->as_VMReg()        ,
	r13->as_VMReg()->next(),
	r14->as_VMReg()        ,
	r14->as_VMReg()->next(),
	r15->as_VMReg()        ,
	r15->as_VMReg()->next(),
	r16->as_VMReg()        ,
	r16->as_VMReg()->next(),
	r17->as_VMReg()        ,
	r17->as_VMReg()->next(),
	r18->as_VMReg()        ,
	r18->as_VMReg()->next(),
	r0->as_VMReg()         ,
	r0->as_VMReg()->next() ,
	r1->as_VMReg()         ,
	r1->as_VMReg()->next() ,
	r2->as_VMReg()         ,
	r2->as_VMReg()->next() ,
	r3->as_VMReg()         ,
	r3->as_VMReg()->next() ,
	r4->as_VMReg()         ,
	r4->as_VMReg()->next() ,
	r5->as_VMReg()         ,
	r5->as_VMReg()->next() ,
	r6->as_VMReg()         ,
	r6->as_VMReg()->next() ,
	r7->as_VMReg()         ,
	r7->as_VMReg()->next() ,
	r19->as_VMReg()        ,
	r19->as_VMReg()->next(),
	r20->as_VMReg()        ,
	r20->as_VMReg()->next(),
	r21->as_VMReg()        ,
	r21->as_VMReg()->next(),
	r22->as_VMReg()        ,
	r22->as_VMReg()->next(),
	r23->as_VMReg()        ,
	r23->as_VMReg()->next(),
	r24->as_VMReg()        ,
	r24->as_VMReg()->next(),
	r25->as_VMReg()        ,
	r25->as_VMReg()->next(),
	r26->as_VMReg()        ,
	r26->as_VMReg()->next(),
	r27->as_VMReg()        ,
	r27->as_VMReg()->next(),
	r28->as_VMReg()        ,
	r28->as_VMReg()->next(),
	r29->as_VMReg()        ,
	r29->as_VMReg()->next(),
	r30->as_VMReg()        ,
	r30->as_VMReg()->next(),
	r31_sp->as_VMReg()     ,
	r31_sp->as_VMReg()->next(),
	v16->as_VMReg()         ,
	v16->as_VMReg()->next() ,
	v16->as_VMReg()->next(2),
	v16->as_VMReg()->next(3),
	v17->as_VMReg()         ,
	v17->as_VMReg()->next() ,
	v17->as_VMReg()->next(2),
	v17->as_VMReg()->next(3),
	v18->as_VMReg()         ,
	v18->as_VMReg()->next() ,
	v18->as_VMReg()->next(2),
	v18->as_VMReg()->next(3),
	v19->as_VMReg()         ,
	v19->as_VMReg()->next() ,
	v19->as_VMReg()->next(2),
	v19->as_VMReg()->next(3),
	v20->as_VMReg()         ,
	v20->as_VMReg()->next() ,
	v20->as_VMReg()->next(2),
	v20->as_VMReg()->next(3),
	v21->as_VMReg()         ,
	v21->as_VMReg()->next() ,
	v21->as_VMReg()->next(2),
	v21->as_VMReg()->next(3),
	v22->as_VMReg()         ,
	v22->as_VMReg()->next() ,
	v22->as_VMReg()->next(2),
	v22->as_VMReg()->next(3),
	v23->as_VMReg()         ,
	v23->as_VMReg()->next() ,
	v23->as_VMReg()->next(2),
	v23->as_VMReg()->next(3),
	v24->as_VMReg()         ,
	v24->as_VMReg()->next() ,
	v24->as_VMReg()->next(2),
	v24->as_VMReg()->next(3),
	v25->as_VMReg()         ,
	v25->as_VMReg()->next() ,
	v25->as_VMReg()->next(2),
	v25->as_VMReg()->next(3),
	v26->as_VMReg()         ,
	v26->as_VMReg()->next() ,
	v26->as_VMReg()->next(2),
	v26->as_VMReg()->next(3),
	v27->as_VMReg()         ,
	v27->as_VMReg()->next() ,
	v27->as_VMReg()->next(2),
	v27->as_VMReg()->next(3),
	v28->as_VMReg()         ,
	v28->as_VMReg()->next() ,
	v28->as_VMReg()->next(2),
	v28->as_VMReg()->next(3),
	v29->as_VMReg()         ,
	v29->as_VMReg()->next() ,
	v29->as_VMReg()->next(2),
	v29->as_VMReg()->next(3),
	v30->as_VMReg()         ,
	v30->as_VMReg()->next() ,
	v30->as_VMReg()->next(2),
	v30->as_VMReg()->next(3),
	v31->as_VMReg()         ,
	v31->as_VMReg()->next() ,
	v31->as_VMReg()->next(2),
	v31->as_VMReg()->next(3),
	v0->as_VMReg()          ,
	v0->as_VMReg()->next()  ,
	v0->as_VMReg()->next(2) ,
	v0->as_VMReg()->next(3) ,
	v1->as_VMReg()          ,
	v1->as_VMReg()->next()  ,
	v1->as_VMReg()->next(2) ,
	v1->as_VMReg()->next(3) ,
	v2->as_VMReg()          ,
	v2->as_VMReg()->next()  ,
	v2->as_VMReg()->next(2) ,
	v2->as_VMReg()->next(3) ,
	v3->as_VMReg()          ,
	v3->as_VMReg()->next()  ,
	v3->as_VMReg()->next(2) ,
	v3->as_VMReg()->next(3) ,
	v4->as_VMReg()          ,
	v4->as_VMReg()->next()  ,
	v4->as_VMReg()->next(2) ,
	v4->as_VMReg()->next(3) ,
	v5->as_VMReg()          ,
	v5->as_VMReg()->next()  ,
	v5->as_VMReg()->next(2) ,
	v5->as_VMReg()->next(3) ,
	v6->as_VMReg()          ,
	v6->as_VMReg()->next()  ,
	v6->as_VMReg()->next(2) ,
	v6->as_VMReg()->next(3) ,
	v7->as_VMReg()          ,
	v7->as_VMReg()->next()  ,
	v7->as_VMReg()->next(2) ,
	v7->as_VMReg()->next(3) ,
	v8->as_VMReg()          ,
	v8->as_VMReg()->next()  ,
	v8->as_VMReg()->next(2) ,
	v8->as_VMReg()->next(3) ,
	v9->as_VMReg()          ,
	v9->as_VMReg()->next()  ,
	v9->as_VMReg()->next(2) ,
	v9->as_VMReg()->next(3) ,
	v10->as_VMReg()         ,
	v10->as_VMReg()->next() ,
	v10->as_VMReg()->next(2),
	v10->as_VMReg()->next(3),
	v11->as_VMReg()         ,
	v11->as_VMReg()->next() ,
	v11->as_VMReg()->next(2),
	v11->as_VMReg()->next(3),
	v12->as_VMReg()         ,
	v12->as_VMReg()->next() ,
	v12->as_VMReg()->next(2),
	v12->as_VMReg()->next(3),
	v13->as_VMReg()         ,
	v13->as_VMReg()->next() ,
	v13->as_VMReg()->next(2),
	v13->as_VMReg()->next(3),
	v14->as_VMReg()         ,
	v14->as_VMReg()->next() ,
	v14->as_VMReg()->next(2),
	v14->as_VMReg()->next(3),
	v15->as_VMReg()         ,
	v15->as_VMReg()->next() ,
	v15->as_VMReg()->next(2),
	v15->as_VMReg()->next(3),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x10',  // R16
  (unsigned char)'\x10',  // R16_H
  (unsigned char)'\x11',  // R17
  (unsigned char)'\x11',  // R17_H
  (unsigned char)'\x12',  // R18
  (unsigned char)'\x12',  // R18_H
  (unsigned char)'\x0',  // R0
  (unsigned char)'\x0',  // R0_H
  (unsigned char)'\x1',  // R1
  (unsigned char)'\x1',  // R1_H
  (unsigned char)'\x2',  // R2
  (unsigned char)'\x2',  // R2_H
  (unsigned char)'\x3',  // R3
  (unsigned char)'\x3',  // R3_H
  (unsigned char)'\x4',  // R4
  (unsigned char)'\x4',  // R4_H
  (unsigned char)'\x5',  // R5
  (unsigned char)'\x5',  // R5_H
  (unsigned char)'\x6',  // R6
  (unsigned char)'\x6',  // R6_H
  (unsigned char)'\x7',  // R7
  (unsigned char)'\x7',  // R7_H
  (unsigned char)'\x13',  // R19
  (unsigned char)'\x13',  // R19_H
  (unsigned char)'\x14',  // R20
  (unsigned char)'\x14',  // R20_H
  (unsigned char)'\x15',  // R21
  (unsigned char)'\x15',  // R21_H
  (unsigned char)'\x16',  // R22
  (unsigned char)'\x16',  // R22_H
  (unsigned char)'\x17',  // R23
  (unsigned char)'\x17',  // R23_H
  (unsigned char)'\x18',  // R24
  (unsigned char)'\x18',  // R24_H
  (unsigned char)'\x19',  // R25
  (unsigned char)'\x19',  // R25_H
  (unsigned char)'\x1A',  // R26
  (unsigned char)'\x1A',  // R26_H
  (unsigned char)'\x1B',  // R27
  (unsigned char)'\x1B',  // R27_H
  (unsigned char)'\x1C',  // R28
  (unsigned char)'\x1C',  // R28_H
  (unsigned char)'\x1D',  // R29
  (unsigned char)'\x1D',  // R29_H
  (unsigned char)'\x1E',  // R30
  (unsigned char)'\x1E',  // R30_H
  (unsigned char)'\x1F',  // R31
  (unsigned char)'\x1F',  // R31_H
  (unsigned char)'\x10',  // V16
  (unsigned char)'\x10',  // V16_H
  (unsigned char)'\x10',  // V16_J
  (unsigned char)'\x10',  // V16_K
  (unsigned char)'\x11',  // V17
  (unsigned char)'\x11',  // V17_H
  (unsigned char)'\x11',  // V17_J
  (unsigned char)'\x11',  // V17_K
  (unsigned char)'\x12',  // V18
  (unsigned char)'\x12',  // V18_H
  (unsigned char)'\x12',  // V18_J
  (unsigned char)'\x12',  // V18_K
  (unsigned char)'\x13',  // V19
  (unsigned char)'\x13',  // V19_H
  (unsigned char)'\x13',  // V19_J
  (unsigned char)'\x13',  // V19_K
  (unsigned char)'\x14',  // V20
  (unsigned char)'\x14',  // V20_H
  (unsigned char)'\x14',  // V20_J
  (unsigned char)'\x14',  // V20_K
  (unsigned char)'\x15',  // V21
  (unsigned char)'\x15',  // V21_H
  (unsigned char)'\x15',  // V21_J
  (unsigned char)'\x15',  // V21_K
  (unsigned char)'\x16',  // V22
  (unsigned char)'\x16',  // V22_H
  (unsigned char)'\x16',  // V22_J
  (unsigned char)'\x16',  // V22_K
  (unsigned char)'\x17',  // V23
  (unsigned char)'\x17',  // V23_H
  (unsigned char)'\x17',  // V23_J
  (unsigned char)'\x17',  // V23_K
  (unsigned char)'\x18',  // V24
  (unsigned char)'\x18',  // V24_H
  (unsigned char)'\x18',  // V24_J
  (unsigned char)'\x18',  // V24_K
  (unsigned char)'\x19',  // V25
  (unsigned char)'\x19',  // V25_H
  (unsigned char)'\x19',  // V25_J
  (unsigned char)'\x19',  // V25_K
  (unsigned char)'\x1A',  // V26
  (unsigned char)'\x1A',  // V26_H
  (unsigned char)'\x1A',  // V26_J
  (unsigned char)'\x1A',  // V26_K
  (unsigned char)'\x1B',  // V27
  (unsigned char)'\x1B',  // V27_H
  (unsigned char)'\x1B',  // V27_J
  (unsigned char)'\x1B',  // V27_K
  (unsigned char)'\x1C',  // V28
  (unsigned char)'\x1C',  // V28_H
  (unsigned char)'\x1C',  // V28_J
  (unsigned char)'\x1C',  // V28_K
  (unsigned char)'\x1D',  // V29
  (unsigned char)'\x1D',  // V29_H
  (unsigned char)'\x1D',  // V29_J
  (unsigned char)'\x1D',  // V29_K
  (unsigned char)'\x1E',  // V30
  (unsigned char)'\x1E',  // V30_H
  (unsigned char)'\x1E',  // V30_J
  (unsigned char)'\x1E',  // V30_K
  (unsigned char)'\x1F',  // V31
  (unsigned char)'\x1F',  // V31_H
  (unsigned char)'\x1F',  // V31_J
  (unsigned char)'\x1F',  // V31_K
  (unsigned char)'\x0',  // V0
  (unsigned char)'\x0',  // V0_H
  (unsigned char)'\x0',  // V0_J
  (unsigned char)'\x0',  // V0_K
  (unsigned char)'\x1',  // V1
  (unsigned char)'\x1',  // V1_H
  (unsigned char)'\x1',  // V1_J
  (unsigned char)'\x1',  // V1_K
  (unsigned char)'\x2',  // V2
  (unsigned char)'\x2',  // V2_H
  (unsigned char)'\x2',  // V2_J
  (unsigned char)'\x2',  // V2_K
  (unsigned char)'\x3',  // V3
  (unsigned char)'\x3',  // V3_H
  (unsigned char)'\x3',  // V3_J
  (unsigned char)'\x3',  // V3_K
  (unsigned char)'\x4',  // V4
  (unsigned char)'\x4',  // V4_H
  (unsigned char)'\x4',  // V4_J
  (unsigned char)'\x4',  // V4_K
  (unsigned char)'\x5',  // V5
  (unsigned char)'\x5',  // V5_H
  (unsigned char)'\x5',  // V5_J
  (unsigned char)'\x5',  // V5_K
  (unsigned char)'\x6',  // V6
  (unsigned char)'\x6',  // V6_H
  (unsigned char)'\x6',  // V6_J
  (unsigned char)'\x6',  // V6_K
  (unsigned char)'\x7',  // V7
  (unsigned char)'\x7',  // V7_H
  (unsigned char)'\x7',  // V7_J
  (unsigned char)'\x7',  // V7_K
  (unsigned char)'\x8',  // V8
  (unsigned char)'\x8',  // V8_H
  (unsigned char)'\x8',  // V8_J
  (unsigned char)'\x8',  // V8_K
  (unsigned char)'\x9',  // V9
  (unsigned char)'\x9',  // V9_H
  (unsigned char)'\x9',  // V9_J
  (unsigned char)'\x9',  // V9_K
  (unsigned char)'\xA',  // V10
  (unsigned char)'\xA',  // V10_H
  (unsigned char)'\xA',  // V10_J
  (unsigned char)'\xA',  // V10_K
  (unsigned char)'\xB',  // V11
  (unsigned char)'\xB',  // V11_H
  (unsigned char)'\xB',  // V11_J
  (unsigned char)'\xB',  // V11_K
  (unsigned char)'\xC',  // V12
  (unsigned char)'\xC',  // V12_H
  (unsigned char)'\xC',  // V12_J
  (unsigned char)'\xC',  // V12_K
  (unsigned char)'\xD',  // V13
  (unsigned char)'\xD',  // V13_H
  (unsigned char)'\xD',  // V13_J
  (unsigned char)'\xD',  // V13_K
  (unsigned char)'\xE',  // V14
  (unsigned char)'\xE',  // V14_H
  (unsigned char)'\xE',  // V14_J
  (unsigned char)'\xE',  // V14_K
  (unsigned char)'\xF',  // V15
  (unsigned char)'\xF',  // V15_H
  (unsigned char)'\xF',  // V15_J
  (unsigned char)'\xF',  // V15_K
  (unsigned char)'\x20' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegINoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *iRegLNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG_mask();
}

const RegMask *iRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *iRegPNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_PTR_REG_mask();
}

const RegMask *iRegP_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R0_REG_mask();
}

const RegMask *iRegP_R1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R1_REG_mask();
}

const RegMask *iRegP_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R2_REG_mask();
}

const RegMask *iRegP_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R3_REG_mask();
}

const RegMask *iRegP_R4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R4_REG_mask();
}

const RegMask *iRegP_R5Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R5_REG_mask();
}

const RegMask *iRegP_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *iRegL_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R0_REG_mask();
}

const RegMask *iRegL_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R2_REG_mask();
}

const RegMask *iRegL_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R3_REG_mask();
}

const RegMask *iRegL_R11Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R11_REG_mask();
}

const RegMask *iRegP_FPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_REG_mask();
}

const RegMask *iRegI_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R0_REG_mask();
}

const RegMask *iRegI_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R2_REG_mask();
}

const RegMask *iRegI_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R3_REG_mask();
}

const RegMask *iRegI_R4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R4_REG_mask();
}

const RegMask *iRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegN_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R0_REG_mask();
}

const RegMask *iRegN_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R2_REG_mask();
}

const RegMask *iRegN_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R3_REG_mask();
}

const RegMask *iRegNNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegIHeapbaseOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &HEAPBASE_REG_mask();
}

const RegMask *vRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask();
}

const RegMask *vRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_mask();
}

const RegMask *vRegD_V0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V0_REG_mask();
}

const RegMask *vRegD_V1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V1_REG_mask();
}

const RegMask *vRegD_V2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V2_REG_mask();
}

const RegMask *vRegD_V3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V3_REG_mask();
}

const RegMask *vRegD_V4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V4_REG_mask();
}

const RegMask *vRegD_V5Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V5_REG_mask();
}

const RegMask *vRegD_V6Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V6_REG_mask();
}

const RegMask *vRegD_V7Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V7_REG_mask();
}

const RegMask *vRegD_V8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V8_REG_mask();
}

const RegMask *vRegD_V9Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V9_REG_mask();
}

const RegMask *vRegD_V10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V10_REG_mask();
}

const RegMask *vRegD_V11Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V11_REG_mask();
}

const RegMask *vRegD_V12Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V12_REG_mask();
}

const RegMask *vRegD_V13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V13_REG_mask();
}

const RegMask *vRegD_V14Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V14_REG_mask();
}

const RegMask *vRegD_V15Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V15_REG_mask();
}

const RegMask *vRegD_V16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V16_REG_mask();
}

const RegMask *vRegD_V17Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V17_REG_mask();
}

const RegMask *vRegD_V18Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V18_REG_mask();
}

const RegMask *vRegD_V19Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V19_REG_mask();
}

const RegMask *vRegD_V20Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V20_REG_mask();
}

const RegMask *vRegD_V21Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V21_REG_mask();
}

const RegMask *vRegD_V22Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V22_REG_mask();
}

const RegMask *vRegD_V23Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V23_REG_mask();
}

const RegMask *vRegD_V24Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V24_REG_mask();
}

const RegMask *vRegD_V25Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V25_REG_mask();
}

const RegMask *vRegD_V26Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V26_REG_mask();
}

const RegMask *vRegD_V27Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V27_REG_mask();
}

const RegMask *vRegD_V28Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V28_REG_mask();
}

const RegMask *vRegD_V29Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V29_REG_mask();
}

const RegMask *vRegD_V30Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V30_REG_mask();
}

const RegMask *vRegD_V31Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V31_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *rFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *inline_cache_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

const RegMask *interpreter_method_oop_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

const RegMask *thread_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &THREAD_REG_mask();
}

const RegMask *lr_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LR_REG_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indIndexScaledI2LOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG32_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexI2LOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG32_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indOffIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indirectNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledI2LNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexI2LNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indOffINOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffLNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *thread_anchor_pcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *iRegL2IOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

//------------------Define members for classes derived from MachNode----------
// Build short branch version of this instruction
MachNode *far_cmpL_branch_signNode::short_branch_version() {
  cmpL_branch_signNode *node = new cmpL_branch_signNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_branch_signNode::short_branch_version() {
  cmpI_branch_signNode *node = new cmpI_branch_signNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_branch_bitNode::short_branch_version() {
  cmpL_branch_bitNode *node = new cmpL_branch_bitNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_branch_bitNode::short_branch_version() {
  cmpI_branch_bitNode *node = new cmpI_branch_bitNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
}

void overflowMulI_reg_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void overflowMulI_reg_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void overflowMulL_reg_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void overflowMulL_reg_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUI_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUI_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP0Node::reloc() const {
  return 1;
}

int loadConP1Node::reloc() const {
  return 1;
}

int loadByteMapBaseNode::reloc() const {
  return 1;
}

int loadConF_packedNode::reloc() const {
  return 2;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConD_packedNode::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int storeimmCM0Node::reloc() const {
  return 2;
}

int storeimmCM0_orderedNode::reloc() const {
  return 2;
}

int storeBNode::reloc() const {
  return 3;
}

int storeimmB0Node::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 3;
}

int storeimmC0Node::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 3;
}

int storeimmI0Node::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeimmL0Node::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeimmP0Node::reloc() const {
  return 3;
}

int storeNNode::reloc() const {
  return 2;
}

int storeImmN0Node::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeNKlassNode::reloc() const {
  return 2;
}

int prefetchallocNode::reloc() const {
  return 1;
}

int storeB_volatileNode::reloc() const {
  return 1;
}

int storeC_volatileNode::reloc() const {
  return 1;
}

int storeI_volatileNode::reloc() const {
  return 1;
}

int bytes_reverse_intNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_shortNode::reloc() const {
  return 1;
}

int bytes_reverse_shortNode::reloc() const {
  return 1;
}

int countLeadingZerosINode::reloc() const {
  return 1;
}

int countTrailingZerosINode::reloc() const {
  return 1;
}

int popCountINode::reloc() const {
  return 1;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int get_and_addINode::reloc() const {
  return 1;
}

int get_and_addI_no_resNode::reloc() const {
  return 1;
}

int get_and_addIAcqNode::reloc() const {
  return 1;
}

int get_and_addI_no_resAcqNode::reloc() const {
  return 1;
}

int cmovI_reg_regNode::reloc() const {
  return 2;
}

int cmovUI_reg_regNode::reloc() const {
  return 2;
}

int cmovI_zero_regNode::reloc() const {
  return 1;
}

int cmovUI_zero_regNode::reloc() const {
  return 1;
}

int cmovI_reg_zeroNode::reloc() const {
  return 1;
}

int cmovUI_reg_zeroNode::reloc() const {
  return 1;
}

int cmovP_reg_zeroNode::reloc() const {
  return 1;
}

int cmovUP_reg_zeroNode::reloc() const {
  return 1;
}

int cmovP_zero_regNode::reloc() const {
  return 1;
}

int cmovUP_zero_regNode::reloc() const {
  return 1;
}

int addI_reg_regNode::reloc() const {
  return 2;
}

int addI_reg_immNode::reloc() const {
  return 1;
}

int addP_reg_reg_extNode::reloc() const {
  return 1;
}

int addP_reg_reg_ext_shiftNode::reloc() const {
  return 1;
}

int lshift_extNode::reloc() const {
  return 1;
}

int subI_reg_regNode::reloc() const {
  return 2;
}

int subI_reg_immNode::reloc() const {
  return 1;
}

int negI_regNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 2;
}

int smulINode::reloc() const {
  return 2;
}

int smulI_0Node::reloc() const {
  return 2;
}

int maddINode::reloc() const {
  return 3;
}

int maddI_0Node::reloc() const {
  return 3;
}

int msubINode::reloc() const {
  return 3;
}

int mnegINode::reloc() const {
  return 2;
}

int mnegI_0Node::reloc() const {
  return 2;
}

int mnegI_1Node::reloc() const {
  return 2;
}

int mnegI_2Node::reloc() const {
  return 2;
}

int smaddLNode::reloc() const {
  return 2;
}

int smaddL_1Node::reloc() const {
  return 2;
}

int smaddL_0Node::reloc() const {
  return 2;
}

int smaddL_2Node::reloc() const {
  return 2;
}

int smsubLNode::reloc() const {
  return 2;
}

int smsubL_0Node::reloc() const {
  return 2;
}

int smnegLNode::reloc() const {
  return 2;
}

int smnegL_0Node::reloc() const {
  return 2;
}

int smnegL_1Node::reloc() const {
  return 2;
}

int smnegL_2Node::reloc() const {
  return 2;
}

int muladdS2INode::reloc() const {
  return 4;
}

int divINode::reloc() const {
  return 2;
}

int modINode::reloc() const {
  return 2;
}

int lShiftI_reg_regNode::reloc() const {
  return 2;
}

int lShiftI_reg_immNode::reloc() const {
  return 1;
}

int urShiftI_reg_regNode::reloc() const {
  return 2;
}

int urShiftI_reg_immNode::reloc() const {
  return 1;
}

int rShiftI_reg_regNode::reloc() const {
  return 2;
}

int rShiftI_reg_immNode::reloc() const {
  return 1;
}

int lShiftL_reg_regNode::reloc() const {
  return 1;
}

int urShiftL_reg_regNode::reloc() const {
  return 1;
}

int rShiftL_reg_regNode::reloc() const {
  return 1;
}

int regI_not_regNode::reloc() const {
  return 1;
}

int AndI_reg_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_2Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_2Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_2Node::reloc() const {
  return 2;
}

int OrI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_URShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_RShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_LShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_URShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_RShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_LShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_URShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_RShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_LShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int SubI_reg_URShift_regNode::reloc() const {
  return 2;
}

int SubI_reg_RShift_regNode::reloc() const {
  return 2;
}

int SubI_reg_LShift_regNode::reloc() const {
  return 2;
}

int sbfmwINode::reloc() const {
  return 1;
}

int ubfmwINode::reloc() const {
  return 1;
}

int ubfxwINode::reloc() const {
  return 1;
}

int ubfxIConvI2LNode::reloc() const {
  return 1;
}

int ubfizwINode::reloc() const {
  return 1;
}

int ubfizIConvI2LNode::reloc() const {
  return 1;
}

int extrOrINode::reloc() const {
  return 2;
}

int extrOrI_0Node::reloc() const {
  return 2;
}

int extrAddINode::reloc() const {
  return 2;
}

int extrAddI_0Node::reloc() const {
  return 2;
}

int AddExtINode::reloc() const {
  return 1;
}

int AddExtI_0Node::reloc() const {
  return 1;
}

int SubExtINode::reloc() const {
  return 1;
}

int AddExtI_sxthNode::reloc() const {
  return 2;
}

int AddExtI_sxth_0Node::reloc() const {
  return 2;
}

int AddExtI_sxtbNode::reloc() const {
  return 2;
}

int AddExtI_sxtb_0Node::reloc() const {
  return 2;
}

int AddExtI_uxtbNode::reloc() const {
  return 2;
}

int AddExtI_uxtb_0Node::reloc() const {
  return 2;
}

int AddExtI_uxtb_andNode::reloc() const {
  return 2;
}

int AddExtI_uxtb_and_0Node::reloc() const {
  return 2;
}

int AddExtI_uxth_andNode::reloc() const {
  return 2;
}

int AddExtI_uxth_and_0Node::reloc() const {
  return 2;
}

int SubExtI_uxtb_andNode::reloc() const {
  return 2;
}

int SubExtI_uxth_andNode::reloc() const {
  return 2;
}

int AddExtI_sxtb_shiftNode::reloc() const {
  return 2;
}

int AddExtI_sxtb_shift_0Node::reloc() const {
  return 2;
}

int AddExtI_sxth_shiftNode::reloc() const {
  return 2;
}

int AddExtI_sxth_shift_0Node::reloc() const {
  return 2;
}

int SubExtI_sxtb_shiftNode::reloc() const {
  return 2;
}

int SubExtI_sxth_shiftNode::reloc() const {
  return 2;
}

int AddExtI_shiftNode::reloc() const {
  return 1;
}

int AddExtI_shift_0Node::reloc() const {
  return 1;
}

int SubExtI_shiftNode::reloc() const {
  return 1;
}

int AddExtI_uxtb_and_shiftNode::reloc() const {
  return 2;
}

int AddExtI_uxtb_and_shift_0Node::reloc() const {
  return 2;
}

int AddExtI_uxth_and_shiftNode::reloc() const {
  return 2;
}

int AddExtI_uxth_and_shift_0Node::reloc() const {
  return 2;
}

int SubExtI_uxtb_and_shiftNode::reloc() const {
  return 2;
}

int SubExtI_uxth_and_shiftNode::reloc() const {
  return 2;
}

int mnsubF_reg_regNode::reloc() const {
  return 1;
}

int mnsubD_reg_regNode::reloc() const {
  return 1;
}

int absI_regNode::reloc() const {
  return 1;
}

int andI_reg_regNode::reloc() const {
  return 2;
}

int andI_reg_immNode::reloc() const {
  return 1;
}

int orI_reg_regNode::reloc() const {
  return 2;
}

int orI_reg_immNode::reloc() const {
  return 1;
}

int xorI_reg_regNode::reloc() const {
  return 2;
}

int xorI_reg_immNode::reloc() const {
  return 1;
}

int convI2L_reg_regNode::reloc() const {
  return 1;
}

int convUI2L_reg_regNode::reloc() const {
  return 1;
}

int convI2BNode::reloc() const {
  return 1;
}

int convI2F_reg_regNode::reloc() const {
  return 1;
}

int convI2D_reg_regNode::reloc() const {
  return 1;
}

int overflowAddI_reg_regNode::reloc() const {
  return 2;
}

int overflowAddI_reg_immNode::reloc() const {
  return 1;
}

int overflowSubI_reg_regNode::reloc() const {
  return 2;
}

int overflowSubI_reg_immNode::reloc() const {
  return 1;
}

int overflowNegI_regNode::reloc() const {
  return 1;
}

int overflowMulI_regNode::reloc() const {
  return 2;
}

int overflowMulI_reg_branchNode::reloc() const {
  return 2;
}

int testP_regNode::reloc() const {
  return 1;
}

int compF_reg_zeroNode::reloc() const {
  return 1;
}

int compD_reg_zeroNode::reloc() const {
  return 1;
}

int compF3_reg_immF0Node::reloc() const {
  return 1;
}

int compD3_reg_immD0Node::reloc() const {
  return 1;
}

int cmpLTMask_reg_regNode::reloc() const {
  return 2;
}

int cmpLTMask_reg_zeroNode::reloc() const {
  return 1;
}

int cmpI_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int cmpUI_imm0_branchNode::reloc() const {
  return 1;
}

int cmpI_branch_signNode::reloc() const {
  return 1;
}

int cmpI_branch_bitNode::reloc() const {
  return 1;
}

int far_cmpI_branch_signNode::reloc() const {
  return 1;
}

int far_cmpI_branch_bitNode::reloc() const {
  return 1;
}

int cmpI_andNode::reloc() const {
  return 1;
}

int cmpI_and_regNode::reloc() const {
  return 2;
}

int safePointNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int TailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int partialSubtypeCheckVsZeroNode::reloc() const {
  return 1;
}

int loadV4Node::reloc() const {
  return 1;
}

int loadV8Node::reloc() const {
  return 1;
}

int loadV16Node::reloc() const {
  return 1;
}

int storeV4Node::reloc() const {
  return 2;
}

int storeV8Node::reloc() const {
  return 2;
}

int storeV16Node::reloc() const {
  return 2;
}

int replicate8BNode::reloc() const {
  return 1;
}

int replicate16BNode::reloc() const {
  return 1;
}

int replicate4SNode::reloc() const {
  return 1;
}

int replicate8SNode::reloc() const {
  return 1;
}

int replicate2INode::reloc() const {
  return 1;
}

int replicate4INode::reloc() const {
  return 1;
}

int reduce_add2INode::reloc() const {
  return 1;
}

int reduce_add4INode::reloc() const {
  return 1;
}

int reduce_mul2INode::reloc() const {
  return 1;
}

int reduce_mul4INode::reloc() const {
  return 1;
}

int vshiftcnt8BNode::reloc() const {
  return 1;
}

int vshiftcnt8B_0Node::reloc() const {
  return 1;
}

int vshiftcnt16BNode::reloc() const {
  return 1;
}

int vshiftcnt16B_0Node::reloc() const {
  return 1;
}

int zLoadPNode::reloc() const {
  return 1;
}

int zLoadWeakPNode::reloc() const {
  return 1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2560 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrsbw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11289 "ad_aarch64.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2568 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrsb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11306 "ad_aarch64.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2584 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11323 "ad_aarch64.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2584 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11340 "ad_aarch64.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2592 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrshw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11357 "ad_aarch64.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2600 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrsh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11374 "ad_aarch64.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2616 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11391 "ad_aarch64.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2616 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11408 "ad_aarch64.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2632 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11425 "ad_aarch64.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2640 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrsw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11442 "ad_aarch64.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2632 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11460 "ad_aarch64.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2648 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 11477 "ad_aarch64.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2632 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11494 "ad_aarch64.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2648 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 11511 "ad_aarch64.cpp"
  }
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2632 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11528 "ad_aarch64.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2648 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 11545 "ad_aarch64.cpp"
  }
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2632 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11562 "ad_aarch64.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2656 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrs, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11579 "ad_aarch64.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2664 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrd, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 11596 "ad_aarch64.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3119 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    u_int32_t con = (u_int32_t)opnd_array(1)->constant();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    if (con == 0) {
      __ movw(dst_reg, zr);
    } else {
      __ movw(dst_reg, con);
    }
  
#line 11618 "ad_aarch64.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 3130 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    u_int64_t con = (u_int64_t)opnd_array(1)->constantL();
    if (con == 0) {
      __ mov(dst_reg, zr);
    } else {
      __ mov(dst_reg, con);
    }
  
#line 11640 "ad_aarch64.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3141 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL || con == (address)1) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      if (rtype == relocInfo::oop_type) {
        __ movoop(dst_reg, (jobject)con, /*immediate*/true);
      } else if (rtype == relocInfo::metadata_type) {
        __ mov_metadata(dst_reg, (Metadata*)con);
      } else {
        assert(rtype == relocInfo::none, "unexpected reloc type");
        if (con < (address)(uintptr_t)os::vm_page_size()) {
          __ mov(dst_reg, con);
        } else {
          unsigned long offset;
          __ adrp(dst_reg, con, offset);
          __ add(dst_reg, dst_reg, offset);
        }
      }
    }
  
#line 11676 "ad_aarch64.cpp"
  }
}

void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3166 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, zr);
  
#line 11693 "ad_aarch64.cpp"
  }
}

void loadConP1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3172 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, (u_int64_t)1);
  
#line 11710 "ad_aarch64.cpp"
  }
}

void loadByteMapBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3178 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ load_byte_map_base(opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 11726 "ad_aarch64.cpp"
  }
}

void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3183 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::oop_type, "unexpected reloc type");
      __ set_narrow_oop(dst_reg, (jobject)con);
    }
  
#line 11750 "ad_aarch64.cpp"
  }
}

void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3196 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, zr);
  
#line 11767 "ad_aarch64.cpp"
  }
}

void loadConNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 3202 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::metadata_type, "unexpected reloc type");
      __ set_narrow_klass(dst_reg, (Klass *)con);
    }
  
#line 11791 "ad_aarch64.cpp"
  }
}

void loadConF_packedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7231 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), (double)opnd_array(1)->constantF());
  
#line 11807 "ad_aarch64.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7249 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldrs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11823 "ad_aarch64.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 7250 "src/hotspot/cpu/aarch64/aarch64.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 11832 "ad_aarch64.cpp"
  }
}
void loadConD_packedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7262 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constantD());
  
#line 11847 "ad_aarch64.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7279 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldrd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11863 "ad_aarch64.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 7280 "src/hotspot/cpu/aarch64/aarch64.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 11872 "ad_aarch64.cpp"
  }
}
void storeimmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2680 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11889 "ad_aarch64.cpp"
  }
}

void storeimmCM0_orderedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2786 "src/hotspot/cpu/aarch64/aarch64.ad"

      C2_MacroAssembler _masm(&cbuf);
      __ membar(Assembler::StoreStore);
      loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11908 "ad_aarch64.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2672 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strb, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11926 "ad_aarch64.cpp"
  }
}

void storeimmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2680 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 1);
  
#line 11944 "ad_aarch64.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2688 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strh, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11962 "ad_aarch64.cpp"
  }
}

void storeimmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2696 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strh, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 2);
  
#line 11980 "ad_aarch64.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2704 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 11998 "ad_aarch64.cpp"
  }
}

void storeimmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2712 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strw, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 12016 "ad_aarch64.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2720 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      C2_MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 12042 "ad_aarch64.cpp"
  }
}

void storeimmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2736 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::str, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 12060 "ad_aarch64.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2720 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      C2_MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 12086 "ad_aarch64.cpp"
  }
}

void storeimmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2736 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::str, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 12104 "ad_aarch64.cpp"
  }
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2704 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 12122 "ad_aarch64.cpp"
  }
}

void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2712 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strw, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 12140 "ad_aarch64.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2744 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strs, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 12158 "ad_aarch64.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2752 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strd, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
  
#line 12176 "ad_aarch64.cpp"
  }
}

void storeNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2704 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
  
#line 12194 "ad_aarch64.cpp"
  }
}

void prefetchallocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 3098 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
      __ prfm(Address(base, disp), PSTL1KEEP);
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ prfm(Address(base, index_reg, Address::lsl(scale)), PSTL1KEEP);
      } else {
        __ lea(rscratch1, Address(base, disp));
	__ prfm(Address(rscratch1, index_reg, Address::lsl(scale)), PSTL1KEEP);
      }
    }
  
#line 12224 "ad_aarch64.cpp"
  }
}

void loadB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2857 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarb);
    __ sxtb(dst_reg, dst_reg);
  
#line 12242 "ad_aarch64.cpp"
  }
}

void loadB2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2857 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarb);
    __ sxtb(dst_reg, dst_reg);
  
#line 12260 "ad_aarch64.cpp"
  }
}

void loadUB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2869 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarb);
  
#line 12276 "ad_aarch64.cpp"
  }
}

void loadUB2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2869 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarb);
  
#line 12292 "ad_aarch64.cpp"
  }
}

void loadS_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2874 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarh);
    __ sxthw(dst_reg, dst_reg);
  
#line 12310 "ad_aarch64.cpp"
  }
}

void loadUS_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2888 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarh);
  
#line 12326 "ad_aarch64.cpp"
  }
}

void loadUS2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2893 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarh);
  
#line 12342 "ad_aarch64.cpp"
  }
}

void loadS2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2881 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarh);
    __ sxth(dst_reg, dst_reg);
  
#line 12360 "ad_aarch64.cpp"
  }
}

void loadI_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2903 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarw);
  
#line 12376 "ad_aarch64.cpp"
  }
}

void loadUI2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2903 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarw);
  
#line 12393 "ad_aarch64.cpp"
  }
}

void loadL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2908 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldar);
  
#line 12409 "ad_aarch64.cpp"
  }
}

void loadP_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2908 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldar);
  
#line 12425 "ad_aarch64.cpp"
  }
}

void loadN_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2903 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarw);
  
#line 12441 "ad_aarch64.cpp"
  }
}

void loadF_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2913 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(rscratch1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldarw);
    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), rscratch1);
  
#line 12458 "ad_aarch64.cpp"
  }
}

void loadD_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2919 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(rscratch1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
             rscratch1, ldar);
    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), rscratch1);
  
#line 12475 "ad_aarch64.cpp"
  }
}

void storeB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2834 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlrb);
  
#line 12492 "ad_aarch64.cpp"
  }
}

void storeC_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2839 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlrh);
  
#line 12509 "ad_aarch64.cpp"
  }
}

void storeI_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2844 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlrw);
  
#line 12526 "ad_aarch64.cpp"
  }
}

void storeL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2925 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      C2_MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    MOV_VOLATILE(src_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlr);
  
#line 12552 "ad_aarch64.cpp"
  }
}

void storeP_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2925 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      C2_MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    MOV_VOLATILE(src_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlr);
  
#line 12578 "ad_aarch64.cpp"
  }
}

void storeN_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2844 "src/hotspot/cpu/aarch64/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlrw);
  
#line 12595 "ad_aarch64.cpp"
  }
}

void storeF_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2939 "src/hotspot/cpu/aarch64/aarch64.ad"

    {
      C2_MacroAssembler _masm(&cbuf);
      FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
      __ fmovs(rscratch2, src_reg);
    }
    MOV_VOLATILE(rscratch2, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlrw);
  
#line 12617 "ad_aarch64.cpp"
  }
}

void storeD_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2949 "src/hotspot/cpu/aarch64/aarch64.ad"

    {
      C2_MacroAssembler _masm(&cbuf);
      FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
      __ fmovd(rscratch2, src_reg);
    }
    MOV_VOLATILE(rscratch2, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
                 rscratch1, stlr);
  
#line 12639 "ad_aarch64.cpp"
  }
}

void cacheWBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7860 "src/hotspot/cpu/aarch64/aarch64.ad"

    assert(opnd_array(1)->index_position() < 0, "should be");
    assert(opnd_array(1)->disp(ra_,this,idx1)== 0, "should be");
    __ cache_wb(Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), 0));
  
#line 12657 "ad_aarch64.cpp"
  }
}

void cacheWBPreSyncNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7875 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cache_wbsync(true);
  
#line 12673 "ad_aarch64.cpp"
  }
}

void cacheWBPostSyncNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7888 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cache_wbsync(false);
  
#line 12689 "ad_aarch64.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7903 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ revw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12705 "ad_aarch64.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7916 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rev(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12721 "ad_aarch64.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7929 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rev16w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12737 "ad_aarch64.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7943 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rev16w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ sbfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), 0U, 15U);
  
#line 12754 "ad_aarch64.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7959 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12770 "ad_aarch64.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7971 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12786 "ad_aarch64.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7984 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rbitw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 12803 "ad_aarch64.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7998 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rbit(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 12820 "ad_aarch64.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8020 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ movw(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */); // ensure top 32 bits 0
    __ mov(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12841 "ad_aarch64.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8041 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister tmp_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrs, tmp_reg, opnd_array(1)->opcode(),
              as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 4);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12863 "ad_aarch64.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8064 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12883 "ad_aarch64.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8084 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister tmp_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldrd, tmp_reg, opnd_array(1)->opcode(),
              as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), 8);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12905 "ad_aarch64.cpp"
  }
}

void load_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8105 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ membar(Assembler::LoadLoad|Assembler::LoadStore);
  
#line 12921 "ad_aarch64.cpp"
  }
}

void unnecessary_membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8118 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_acquire (elided)");
  
#line 12937 "ad_aarch64.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8132 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_acquire");
    __ membar(Assembler::LoadLoad|Assembler::LoadStore);
  
#line 12954 "ad_aarch64.cpp"
  }
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8147 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_acquire_lock (elided)");
  
#line 12970 "ad_aarch64.cpp"
  }
}

void store_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8160 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ membar(Assembler::LoadStore|Assembler::StoreStore);
  
#line 12986 "ad_aarch64.cpp"
  }
}

void unnecessary_membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8173 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_release (elided)");
  
#line 13002 "ad_aarch64.cpp"
  }
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8186 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_release");
    __ membar(Assembler::LoadStore|Assembler::StoreStore);
  
#line 13019 "ad_aarch64.cpp"
  }
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8199 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ membar(Assembler::StoreStore);
  
#line 13035 "ad_aarch64.cpp"
  }
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8211 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_release_lock (elided)");
  
#line 13051 "ad_aarch64.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8225 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_volatile (elided)");
  
#line 13067 "ad_aarch64.cpp"
  }
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8239 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ block_comment("membar_volatile");
    __ membar(Assembler::StoreLoad);
  
#line 13084 "ad_aarch64.cpp"
  }
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/aarch64/aarch64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mov(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 13102 "ad_aarch64.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8271 "src/hotspot/cpu/aarch64/aarch64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mov(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 13120 "ad_aarch64.cpp"
  }
}

void convP2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8286 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ movw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13136 "ad_aarch64.cpp"
  }
}

void convN2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8302 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ movw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13152 "ad_aarch64.cpp"
  }
}

void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8317 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ encode_heap_oop(d, s);
  
#line 13170 "ad_aarch64.cpp"
  }
}

void encodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8330 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ encode_heap_oop_not_null(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13186 "ad_aarch64.cpp"
  }
}

void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8342 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop(d, s);
  
#line 13204 "ad_aarch64.cpp"
  }
}

void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8356 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop_not_null(d, s);
  
#line 13222 "ad_aarch64.cpp"
  }
}

void encodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8374 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ encode_klass_not_null(dst_reg, src_reg);
  
#line 13240 "ad_aarch64.cpp"
  }
}

void decodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8389 "src/hotspot/cpu/aarch64/aarch64.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    if (dst_reg != src_reg) {
      __ decode_klass_not_null(dst_reg, src_reg);
    } else {
      __ decode_klass_not_null(dst_reg);
    }
  
#line 13262 "ad_aarch64.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2961 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
       if (disp != 0) {
        __ lea(rscratch1, Address(base, disp));
        __ ldaxr(dst_reg, rscratch1);
      } else {
        // TODO
        // should we ever get anything other than this case?
        __ ldaxr(dst_reg, base);
      }
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ lea(rscratch1, Address(base, index_reg, Address::lsl(scale)));
        __ ldaxr(dst_reg, rscratch1);
      } else {
        __ lea(rscratch1, Address(base, disp));
        __ lea(rscratch1, Address(rscratch1, index_reg, Address::lsl(scale)));
        __ ldaxr(dst_reg, rscratch1);
      }
    }
  
#line 13341 "ad_aarch64.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2990 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register src_reg = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* newval */);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
       if (disp != 0) {
        __ lea(rscratch2, Address(base, disp));
        __ stlxr(rscratch1, src_reg, rscratch2);
      } else {
        // TODO
        // should we ever get anything other than this case?
        __ stlxr(rscratch1, src_reg, base);
      }
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ lea(rscratch2, Address(base, index_reg, Address::lsl(scale)));
        __ stlxr(rscratch1, src_reg, rscratch2);
      } else {
        __ lea(rscratch2, Address(base, disp));
        __ lea(rscratch2, Address(rscratch2, index_reg, Address::lsl(scale)));
        __ stlxr(rscratch1, src_reg, rscratch2);
      }
    }
    __ cmpw(rscratch1, zr);
  
#line 13384 "ad_aarch64.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3057 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13405 "ad_aarch64.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3065 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13426 "ad_aarch64.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3044 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13447 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13457 "ad_aarch64.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3036 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13478 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13488 "ad_aarch64.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3028 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13509 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13519 "ad_aarch64.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3020 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13540 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13550 "ad_aarch64.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3020 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13571 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13581 "ad_aarch64.cpp"
  }
}

void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3028 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13602 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13612 "ad_aarch64.cpp"
  }
}

void compareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3081 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13633 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13643 "ad_aarch64.cpp"
  }
}

void compareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3073 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13664 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13674 "ad_aarch64.cpp"
  }
}

void compareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3065 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13695 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13705 "ad_aarch64.cpp"
  }
}

void compareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3057 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13726 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13736 "ad_aarch64.cpp"
  }
}

void compareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3057 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13757 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13767 "ad_aarch64.cpp"
  }
}

void compareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 3065 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, noreg);
  
#line 13788 "ad_aarch64.cpp"
  }
  {

#line 3090 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13798 "ad_aarch64.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8794 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    __ sxtbw(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13820 "ad_aarch64.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8810 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    __ sxthw(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13842 "ad_aarch64.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8826 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13863 "ad_aarch64.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8841 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13884 "ad_aarch64.cpp"
  }
}

void compareAndExchangeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8856 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13905 "ad_aarch64.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8872 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13926 "ad_aarch64.cpp"
  }
}

void compareAndExchangeBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8888 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    __ sxtbw(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13948 "ad_aarch64.cpp"
  }
}

void compareAndExchangeSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8905 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    __ sxthw(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13970 "ad_aarch64.cpp"
  }
}

void compareAndExchangeIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8923 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 13991 "ad_aarch64.cpp"
  }
}

void compareAndExchangeLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8939 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 14012 "ad_aarch64.cpp"
  }
}

void compareAndExchangeNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8956 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 14033 "ad_aarch64.cpp"
  }
}

void compareAndExchangePAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8972 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ false, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 14054 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8988 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14075 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9005 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14096 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9022 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14117 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9039 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14138 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9056 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14159 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9074 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14180 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9092 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::byte, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14201 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9110 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::halfword, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14222 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9128 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14243 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9146 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14264 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9164 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14285 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9182 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true,
               /*weak*/ true, noreg);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* res */, Assembler::EQ);
  
#line 14306 "ad_aarch64.cpp"
  }
}

void get_and_setINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9198 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14323 "ad_aarch64.cpp"
  }
}

void get_and_setLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9208 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14340 "ad_aarch64.cpp"
  }
}

void get_and_setNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9218 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14357 "ad_aarch64.cpp"
  }
}

void get_and_setPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9229 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14374 "ad_aarch64.cpp"
  }
}

void get_and_setIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9240 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgalw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14391 "ad_aarch64.cpp"
  }
}

void get_and_setLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9251 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14408 "ad_aarch64.cpp"
  }
}

void get_and_setNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9262 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgalw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14425 "ad_aarch64.cpp"
  }
}

void get_and_setPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9273 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14442 "ad_aarch64.cpp"
  }
}

void get_and_addLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9284 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14459 "ad_aarch64.cpp"
  }
}

void get_and_addL_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9295 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_add(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14476 "ad_aarch64.cpp"
  }
}

void get_and_addLiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9305 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14493 "ad_aarch64.cpp"
  }
}

void get_and_addLi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9316 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_add(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14510 "ad_aarch64.cpp"
  }
}

void get_and_addINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9326 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14527 "ad_aarch64.cpp"
  }
}

void get_and_addI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9337 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14544 "ad_aarch64.cpp"
  }
}

void get_and_addIiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9347 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14561 "ad_aarch64.cpp"
  }
}

void get_and_addIi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9358 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14578 "ad_aarch64.cpp"
  }
}

void get_and_addLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9369 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14595 "ad_aarch64.cpp"
  }
}

void get_and_addL_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9380 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addal(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14612 "ad_aarch64.cpp"
  }
}

void get_and_addLiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9391 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14629 "ad_aarch64.cpp"
  }
}

void get_and_addLi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9402 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addal(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14646 "ad_aarch64.cpp"
  }
}

void get_and_addIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9413 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14663 "ad_aarch64.cpp"
  }
}

void get_and_addI_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9424 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addalw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14680 "ad_aarch64.cpp"
  }
}

void get_and_addIiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9435 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14697 "ad_aarch64.cpp"
  }
}

void get_and_addIi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9446 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ atomic_addalw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 14714 "ad_aarch64.cpp"
  }
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9466 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmp(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::NE);
    __ cnegw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::LT);
  
#line 14733 "ad_aarch64.cpp"
  }
}

void cmpL3_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9486 "src/hotspot/cpu/aarch64/aarch64.ad"

    int32_t con = (int32_t)opnd_array(2)->constantL();
     if (con < 0) {
      __ adds(zr, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, -con);
    } else {
      __ subs(zr, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, con);
    }
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::NE);
    __ cnegw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::LT);
  
#line 14757 "ad_aarch64.cpp"
  }
}

void cmovI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9519 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14779 "ad_aarch64.cpp"
  }
}

void cmovUI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9535 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14801 "ad_aarch64.cpp"
  }
}

void cmovI_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9560 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14823 "ad_aarch64.cpp"
  }
}

void cmovUI_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9576 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14845 "ad_aarch64.cpp"
  }
}

void cmovI_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9592 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14867 "ad_aarch64.cpp"
  }
}

void cmovUI_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9608 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14889 "ad_aarch64.cpp"
  }
}

void cmovI_reg_zero_oneNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9629 "src/hotspot/cpu/aarch64/aarch64.ad"

    // equivalently
    // cset(as_Register($dst$$reg),
    //      negate_condition((Assembler::Condition)$cmp$$cmpcode));
    __ csincw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14914 "ad_aarch64.cpp"
  }
}

void cmovUI_reg_zero_oneNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9648 "src/hotspot/cpu/aarch64/aarch64.ad"

    // equivalently
    // cset(as_Register($dst$$reg),
    //      negate_condition((Assembler::Condition)$cmp$$cmpcode));
    __ csincw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14939 "ad_aarch64.cpp"
  }
}

void cmovL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9667 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14961 "ad_aarch64.cpp"
  }
}

void cmovUL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9683 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14983 "ad_aarch64.cpp"
  }
}

void cmovL_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9701 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15005 "ad_aarch64.cpp"
  }
}

void cmovUL_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9717 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15027 "ad_aarch64.cpp"
  }
}

void cmovL_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9733 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15049 "ad_aarch64.cpp"
  }
}

void cmovUL_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9749 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15071 "ad_aarch64.cpp"
  }
}

void cmovP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9765 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15093 "ad_aarch64.cpp"
  }
}

void cmovUP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9781 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15115 "ad_aarch64.cpp"
  }
}

void cmovP_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9799 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15137 "ad_aarch64.cpp"
  }
}

void cmovUP_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9815 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15159 "ad_aarch64.cpp"
  }
}

void cmovP_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9831 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15181 "ad_aarch64.cpp"
  }
}

void cmovUP_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9847 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15203 "ad_aarch64.cpp"
  }
}

void cmovN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9863 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15225 "ad_aarch64.cpp"
  }
}

void cmovUN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9879 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15247 "ad_aarch64.cpp"
  }
}

void cmovN_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9897 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15269 "ad_aarch64.cpp"
  }
}

void cmovUN_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9913 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15291 "ad_aarch64.cpp"
  }
}

void cmovN_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9929 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15313 "ad_aarch64.cpp"
  }
}

void cmovUN_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9945 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 15335 "ad_aarch64.cpp"
  }
}

void cmovF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9962 "src/hotspot/cpu/aarch64/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcsels(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 15358 "ad_aarch64.cpp"
  }
}

void cmovUF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9980 "src/hotspot/cpu/aarch64/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcsels(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 15381 "ad_aarch64.cpp"
  }
}

void cmovD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9998 "src/hotspot/cpu/aarch64/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcseld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 15404 "ad_aarch64.cpp"
  }
}

void cmovUD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10016 "src/hotspot/cpu/aarch64/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcseld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 15427 "ad_aarch64.cpp"
  }
}

void addI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10045 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15446 "ad_aarch64.cpp"
  }
}

void addI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3217 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 15472 "ad_aarch64.cpp"
  }
}

void addI_reg_imm_i2lNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3217 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 15498 "ad_aarch64.cpp"
  }
}

void addP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10089 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15517 "ad_aarch64.cpp"
  }
}

void addP_reg_reg_extNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10104 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
  
#line 15536 "ad_aarch64.cpp"
  }
}

void addP_reg_reg_lslNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// scale
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10119 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lea(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           Address(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   Address::lsl(opnd_array(3)->constant())));
  
#line 15556 "ad_aarch64.cpp"
  }
}

void addP_reg_reg_ext_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// scale
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10134 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lea(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           Address(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   Address::sxtw(opnd_array(3)->constant())));
  
#line 15576 "ad_aarch64.cpp"
  }
}

void lshift_extNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scale
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10149 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sbfiz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
          as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
          opnd_array(2)->constant()& 63, MIN(32, (-opnd_array(2)->constant()) & 63));
  
#line 15595 "ad_aarch64.cpp"
  }
}

void addP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3231 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 15621 "ad_aarch64.cpp"
  }
}

void addL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10183 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15640 "ad_aarch64.cpp"
  }
}

void addL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3231 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 15666 "ad_aarch64.cpp"
  }
}

void subI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10214 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15685 "ad_aarch64.cpp"
  }
}

void subI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3217 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x1 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 15711 "ad_aarch64.cpp"
  }
}

void subL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10246 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15730 "ad_aarch64.cpp"
  }
}

void subL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3231 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x1 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 15756 "ad_aarch64.cpp"
  }
}

void negI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10278 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 15774 "ad_aarch64.cpp"
  }
}

void negL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10294 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ neg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 15792 "ad_aarch64.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10310 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15811 "ad_aarch64.cpp"
  }
}

void smulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10325 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smull(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15830 "ad_aarch64.cpp"
  }
}

void smulI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10325 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smull(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 15849 "ad_aarch64.cpp"
  }
}

void mulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10342 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mul(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15868 "ad_aarch64.cpp"
  }
}

void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10358 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smulh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15887 "ad_aarch64.cpp"
  }
}

void maddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10375 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ maddw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 15908 "ad_aarch64.cpp"
  }
}

void maddI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10375 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ maddw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 15929 "ad_aarch64.cpp"
  }
}

void msubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10391 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ msubw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 15950 "ad_aarch64.cpp"
  }
}

void mnegINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10410 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mnegw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 15970 "ad_aarch64.cpp"
  }
}

void mnegI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10410 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mnegw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 15990 "ad_aarch64.cpp"
  }
}

void mnegI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10410 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mnegw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 16010 "ad_aarch64.cpp"
  }
}

void mnegI_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10410 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mnegw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16030 "ad_aarch64.cpp"
  }
}

void maddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10427 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ madd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16051 "ad_aarch64.cpp"
  }
}

void maddL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10427 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ madd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16072 "ad_aarch64.cpp"
  }
}

void msubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10443 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ msub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16093 "ad_aarch64.cpp"
  }
}

void mnegLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10462 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mneg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 16113 "ad_aarch64.cpp"
  }
}

void mnegL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10462 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mneg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 16133 "ad_aarch64.cpp"
  }
}

void mnegL_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10462 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mneg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 16153 "ad_aarch64.cpp"
  }
}

void mnegL_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10462 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mneg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16173 "ad_aarch64.cpp"
  }
}

void smaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10479 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smaddl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16194 "ad_aarch64.cpp"
  }
}

void smaddL_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10479 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smaddl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16215 "ad_aarch64.cpp"
  }
}

void smaddL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10479 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smaddl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16236 "ad_aarch64.cpp"
  }
}

void smaddL_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10479 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smaddl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16257 "ad_aarch64.cpp"
  }
}

void smsubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10495 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smsubl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16278 "ad_aarch64.cpp"
  }
}

void smsubL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10495 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smsubl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 16299 "ad_aarch64.cpp"
  }
}

void smnegLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10512 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smnegl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 16319 "ad_aarch64.cpp"
  }
}

void smnegL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10512 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smnegl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 16339 "ad_aarch64.cpp"
  }
}

void smnegL_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10512 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smnegl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 16359 "ad_aarch64.cpp"
  }
}

void smnegL_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10512 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smnegl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16379 "ad_aarch64.cpp"
  }
}

void muladdS2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10530 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulw(rscratch1, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ maddw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src4 */), rscratch1); 
#line 16398 "ad_aarch64.cpp"
  }
}

void divINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3245 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, false, rscratch1);
  
#line 16418 "ad_aarch64.cpp"
  }
}

void divLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3253 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, false, rscratch1);
  
#line 16438 "ad_aarch64.cpp"
  }
}

void modINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3261 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, true, rscratch1);
  
#line 16458 "ad_aarch64.cpp"
  }
}

void modLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 3269 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, true, rscratch1);
  
#line 16478 "ad_aarch64.cpp"
  }
}

void lShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10596 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lslvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16497 "ad_aarch64.cpp"
  }
}

void lShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10612 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lslw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 16516 "ad_aarch64.cpp"
  }
}

void urShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10628 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsrvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16535 "ad_aarch64.cpp"
  }
}

void urShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10644 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 16554 "ad_aarch64.cpp"
  }
}

void rShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10660 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ asrvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16573 "ad_aarch64.cpp"
  }
}

void rShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10676 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ asrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 16592 "ad_aarch64.cpp"
  }
}

void lShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10697 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lslv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16611 "ad_aarch64.cpp"
  }
}

void lShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10713 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x3f);
  
#line 16630 "ad_aarch64.cpp"
  }
}

void urShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10729 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsrv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16649 "ad_aarch64.cpp"
  }
}

void urShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10745 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 16668 "ad_aarch64.cpp"
  }
}

void urShiftP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10761 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ lsr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 16687 "ad_aarch64.cpp"
  }
}

void rShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10777 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ asrv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16706 "ad_aarch64.cpp"
  }
}

void rShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10793 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ asr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 16725 "ad_aarch64.cpp"
  }
}

void regL_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10811 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              zr,
              Assembler::LSL, 0);
  
#line 16745 "ad_aarch64.cpp"
  }
}

void regI_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10827 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              zr,
              Assembler::LSL, 0);
  
#line 16765 "ad_aarch64.cpp"
  }
}

void AndI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10844 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16786 "ad_aarch64.cpp"
  }
}

void AndI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10844 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 16807 "ad_aarch64.cpp"
  }
}

void AndL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10861 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16828 "ad_aarch64.cpp"
  }
}

void AndL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10861 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 16849 "ad_aarch64.cpp"
  }
}

void OrI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10878 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16870 "ad_aarch64.cpp"
  }
}

void OrI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10878 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 16891 "ad_aarch64.cpp"
  }
}

void OrL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10895 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16912 "ad_aarch64.cpp"
  }
}

void OrL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10895 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 16933 "ad_aarch64.cpp"
  }
}

void XorI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// m1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10912 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16954 "ad_aarch64.cpp"
  }
}

void XorI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10912 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 16975 "ad_aarch64.cpp"
  }
}

void XorL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// m1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10929 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 16996 "ad_aarch64.cpp"
  }
}

void XorL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10929 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 17017 "ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10946 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17040 "ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10946 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17063 "ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10964 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17086 "ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10964 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17109 "ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10982 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17132 "ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10982 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17155 "ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11000 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17178 "ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11000 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17201 "ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11018 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17224 "ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11018 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 17247 "ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11036 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17270 "ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11036 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 17293 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11054 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17316 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11054 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17339 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11054 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSR,
              opnd_array(4)->constant()& 0x1f);
  
#line 17362 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11054 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17385 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11072 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17408 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11072 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17431 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11072 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSR,
              opnd_array(4)->constant()& 0x3f);
  
#line 17454 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11072 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17477 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11090 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17500 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11090 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17523 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11090 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::ASR,
              opnd_array(4)->constant()& 0x1f);
  
#line 17546 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11090 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17569 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11108 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17592 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11108 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17615 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11108 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::ASR,
              opnd_array(4)->constant()& 0x3f);
  
#line 17638 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11108 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17661 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17684 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 17707 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSL,
              opnd_array(4)->constant()& 0x1f);
  
#line 17730 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17753 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11144 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17776 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11144 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 17799 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11144 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSL,
              opnd_array(4)->constant()& 0x3f);
  
#line 17822 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11144 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17845 "ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11162 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17868 "ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11162 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17891 "ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11180 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17914 "ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11180 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17937 "ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11198 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17960 "ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11198 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17983 "ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11216 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18006 "ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11216 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18029 "ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11234 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 18052 "ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11234 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 18075 "ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11252 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 18098 "ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11252 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 18121 "ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11271 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18143 "ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11271 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18165 "ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11290 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18187 "ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11290 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18209 "ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11309 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18231 "ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11309 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18253 "ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11328 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18275 "ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11328 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18297 "ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11347 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 18319 "ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11347 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 18341 "ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11366 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 18363 "ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11366 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 18385 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11385 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18407 "ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11385 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18429 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11404 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18451 "ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11404 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18473 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11423 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18495 "ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11423 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18517 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11442 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18539 "ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11442 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18561 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11461 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 18583 "ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11461 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 18605 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11480 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 18627 "ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11480 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 18649 "ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11499 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18671 "ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11499 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18693 "ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11518 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18715 "ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11518 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18737 "ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11537 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18759 "ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11537 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18781 "ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11556 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18803 "ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11556 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 18825 "ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11575 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 18847 "ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11575 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 18869 "ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11594 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 18891 "ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11594 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 18913 "ad_aarch64.cpp"
  }
}

void AddI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11613 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 18935 "ad_aarch64.cpp"
  }
}

void AddI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11613 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 18957 "ad_aarch64.cpp"
  }
}

void AddL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11632 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 18979 "ad_aarch64.cpp"
  }
}

void AddL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11632 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 19001 "ad_aarch64.cpp"
  }
}

void AddI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11651 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 19023 "ad_aarch64.cpp"
  }
}

void AddI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11651 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 19045 "ad_aarch64.cpp"
  }
}

void AddL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11670 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 19067 "ad_aarch64.cpp"
  }
}

void AddL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11670 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 19089 "ad_aarch64.cpp"
  }
}

void AddI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11689 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 19111 "ad_aarch64.cpp"
  }
}

void AddI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11689 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 19133 "ad_aarch64.cpp"
  }
}

void AddL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11708 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 19155 "ad_aarch64.cpp"
  }
}

void AddL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11708 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 19177 "ad_aarch64.cpp"
  }
}

void SubI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11727 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 19199 "ad_aarch64.cpp"
  }
}

void SubL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11746 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 19221 "ad_aarch64.cpp"
  }
}

void SubI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11765 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 19243 "ad_aarch64.cpp"
  }
}

void SubL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11784 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 19265 "ad_aarch64.cpp"
  }
}

void SubI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11803 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 19287 "ad_aarch64.cpp"
  }
}

void SubL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11822 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 19309 "ad_aarch64.cpp"
  }
}

void sbfmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11842 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(2)->constant()& 63;
    int rshift = opnd_array(3)->constant()& 63;
    int s = 63 - lshift;
    int r = (rshift - lshift) & 63;
    __ sbfm(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 19333 "ad_aarch64.cpp"
  }
}

void sbfmwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11862 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(2)->constant()& 31;
    int rshift = opnd_array(3)->constant()& 31;
    int s = 31 - lshift;
    int r = (rshift - lshift) & 31;
    __ sbfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 19357 "ad_aarch64.cpp"
  }
}

void ubfmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11882 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(2)->constant()& 63;
    int rshift = opnd_array(3)->constant()& 63;
    int s = 63 - lshift;
    int r = (rshift - lshift) & 63;
    __ ubfm(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 19381 "ad_aarch64.cpp"
  }
}

void ubfmwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11902 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(2)->constant()& 31;
    int rshift = opnd_array(3)->constant()& 31;
    int s = 31 - lshift;
    int r = (rshift - lshift) & 31;
    __ ubfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 19405 "ad_aarch64.cpp"
  }
}

void ubfxwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11924 "src/hotspot/cpu/aarch64/aarch64.ad"

    int rshift = opnd_array(2)->constant()& 31;
    long mask = opnd_array(3)->constant();
    int width = exact_log2(mask+1);
    __ ubfxw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 19427 "ad_aarch64.cpp"
  }
}

void ubfxLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11941 "src/hotspot/cpu/aarch64/aarch64.ad"

    int rshift = opnd_array(2)->constant()& 63;
    long mask = opnd_array(3)->constantL();
    int width = exact_log2_long(mask+1);
    __ ubfx(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 19449 "ad_aarch64.cpp"
  }
}

void ubfxIConvI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11961 "src/hotspot/cpu/aarch64/aarch64.ad"

    int rshift = opnd_array(2)->constant()& 31;
    long mask = opnd_array(3)->constant();
    int width = exact_log2(mask+1);
    __ ubfx(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 19471 "ad_aarch64.cpp"
  }
}

void ubfizwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11980 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(3)->constant()& 31;
    long mask = opnd_array(2)->constant();
    int width = exact_log2(mask+1);
    __ ubfizw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
          as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), lshift, width);
  
#line 19493 "ad_aarch64.cpp"
  }
}

void ubfizLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11998 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(3)->constant()& 63;
    long mask = opnd_array(2)->constantL();
    int width = exact_log2_long(mask+1);
    __ ubfiz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
          as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), lshift, width);
  
#line 19515 "ad_aarch64.cpp"
  }
}

void ubfizIConvI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12016 "src/hotspot/cpu/aarch64/aarch64.ad"

    int lshift = opnd_array(3)->constant()& 63;
    long mask = opnd_array(2)->constant();
    int width = exact_log2(mask+1);
    __ ubfiz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), lshift, width);
  
#line 19537 "ad_aarch64.cpp"
  }
}

void extrOrLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12036 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 63);
  
#line 19557 "ad_aarch64.cpp"
  }
}

void extrOrL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12036 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 63);
  
#line 19577 "ad_aarch64.cpp"
  }
}

void extrOrINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12051 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 31);
  
#line 19597 "ad_aarch64.cpp"
  }
}

void extrOrI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12051 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 31);
  
#line 19617 "ad_aarch64.cpp"
  }
}

void extrAddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12066 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 63);
  
#line 19637 "ad_aarch64.cpp"
  }
}

void extrAddL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12066 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 63);
  
#line 19657 "ad_aarch64.cpp"
  }
}

void extrAddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12081 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 31);
  
#line 19677 "ad_aarch64.cpp"
  }
}

void extrAddI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12081 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 31);
  
#line 19697 "ad_aarch64.cpp"
  }
}

void rolL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12097 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(rscratch1, zr, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ rorv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            rscratch1);
    
#line 19716 "ad_aarch64.cpp"
  }
}

void rolI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12113 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subw(rscratch1, zr, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ rorvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            rscratch1);
    
#line 19735 "ad_aarch64.cpp"
  }
}

void rorL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12165 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rorv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    
#line 19753 "ad_aarch64.cpp"
  }
}

void rorI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12180 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ rorvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    
#line 19771 "ad_aarch64.cpp"
  }
}

void AddExtINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12231 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 19789 "ad_aarch64.cpp"
  }
}

void AddExtI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12231 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw);
   
#line 19807 "ad_aarch64.cpp"
  }
}

void SubExtINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12244 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 19825 "ad_aarch64.cpp"
  }
}

void AddExtI_sxthNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12258 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth);
   
#line 19845 "ad_aarch64.cpp"
  }
}

void AddExtI_sxth_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12258 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth);
   
#line 19865 "ad_aarch64.cpp"
  }
}

void AddExtI_sxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12271 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb);
   
#line 19885 "ad_aarch64.cpp"
  }
}

void AddExtI_sxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12271 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb);
   
#line 19905 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12284 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 19925 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12284 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 19945 "ad_aarch64.cpp"
  }
}

void AddExtL_sxthNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12297 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth);
   
#line 19965 "ad_aarch64.cpp"
  }
}

void AddExtL_sxth_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12297 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth);
   
#line 19985 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12310 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 20005 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12310 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw);
   
#line 20025 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12323 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb);
   
#line 20045 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12323 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb);
   
#line 20065 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12336 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 20085 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12336 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 20105 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12350 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 20124 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12350 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 20143 "ad_aarch64.cpp"
  }
}

void AddExtI_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12363 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 20162 "ad_aarch64.cpp"
  }
}

void AddExtI_uxth_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12363 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth);
   
#line 20181 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12376 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 20200 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12376 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 20219 "ad_aarch64.cpp"
  }
}

void AddExtL_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12389 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 20238 "ad_aarch64.cpp"
  }
}

void AddExtL_uxth_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12389 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth);
   
#line 20257 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12402 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw);
   
#line 20276 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12402 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtw);
   
#line 20295 "ad_aarch64.cpp"
  }
}

void SubExtI_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12415 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 20314 "ad_aarch64.cpp"
  }
}

void SubExtI_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12428 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 20333 "ad_aarch64.cpp"
  }
}

void SubExtL_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12441 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 20352 "ad_aarch64.cpp"
  }
}

void SubExtL_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12454 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 20371 "ad_aarch64.cpp"
  }
}

void SubExtL_uxtw_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12467 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw);
   
#line 20390 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtb_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12481 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb, (opnd_array(5)->constant()));
   
#line 20411 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtb_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12481 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb, (opnd_array(4)->constant()));
   
#line 20432 "ad_aarch64.cpp"
  }
}

void AddExtL_sxth_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12494 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth, (opnd_array(5)->constant()));
   
#line 20453 "ad_aarch64.cpp"
  }
}

void AddExtL_sxth_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12494 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth, (opnd_array(4)->constant()));
   
#line 20474 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtw_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12507 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw, (opnd_array(5)->constant()));
   
#line 20495 "ad_aarch64.cpp"
  }
}

void AddExtL_sxtw_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12507 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw, (opnd_array(4)->constant()));
   
#line 20516 "ad_aarch64.cpp"
  }
}

void SubExtL_sxtb_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12520 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb, (opnd_array(5)->constant()));
   
#line 20537 "ad_aarch64.cpp"
  }
}

void SubExtL_sxth_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12533 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth, (opnd_array(5)->constant()));
   
#line 20558 "ad_aarch64.cpp"
  }
}

void SubExtL_sxtw_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12546 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw, (opnd_array(5)->constant()));
   
#line 20579 "ad_aarch64.cpp"
  }
}

void AddExtI_sxtb_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12559 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb, (opnd_array(5)->constant()));
   
#line 20600 "ad_aarch64.cpp"
  }
}

void AddExtI_sxtb_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12559 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb, (opnd_array(4)->constant()));
   
#line 20621 "ad_aarch64.cpp"
  }
}

void AddExtI_sxth_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12572 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth, (opnd_array(5)->constant()));
   
#line 20642 "ad_aarch64.cpp"
  }
}

void AddExtI_sxth_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12572 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth, (opnd_array(4)->constant()));
   
#line 20663 "ad_aarch64.cpp"
  }
}

void SubExtI_sxtb_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12585 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb, (opnd_array(5)->constant()));
   
#line 20684 "ad_aarch64.cpp"
  }
}

void SubExtI_sxth_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// lshift2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12598 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth, (opnd_array(5)->constant()));
   
#line 20705 "ad_aarch64.cpp"
  }
}

void AddExtI_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12612 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw, (opnd_array(3)->constant()));
   
#line 20724 "ad_aarch64.cpp"
  }
}

void AddExtI_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12612 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw, (opnd_array(2)->constant()));
   
#line 20743 "ad_aarch64.cpp"
  }
}

void SubExtI_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12625 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw, (opnd_array(3)->constant()));
   
#line 20762 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12639 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb, (opnd_array(4)->constant()));
   
#line 20782 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_and_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12639 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb, (opnd_array(3)->constant()));
   
#line 20802 "ad_aarch64.cpp"
  }
}

void AddExtL_uxth_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12652 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth, (opnd_array(4)->constant()));
   
#line 20822 "ad_aarch64.cpp"
  }
}

void AddExtL_uxth_and_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12652 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth, (opnd_array(3)->constant()));
   
#line 20842 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12665 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw, (opnd_array(4)->constant()));
   
#line 20862 "ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_and_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12665 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtw, (opnd_array(3)->constant()));
   
#line 20882 "ad_aarch64.cpp"
  }
}

void SubExtL_uxtb_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12678 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb, (opnd_array(4)->constant()));
   
#line 20902 "ad_aarch64.cpp"
  }
}

void SubExtL_uxth_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12691 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth, (opnd_array(4)->constant()));
   
#line 20922 "ad_aarch64.cpp"
  }
}

void SubExtL_uxtw_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12704 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw, (opnd_array(4)->constant()));
   
#line 20942 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12717 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb, (opnd_array(4)->constant()));
   
#line 20962 "ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_and_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12717 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb, (opnd_array(3)->constant()));
   
#line 20982 "ad_aarch64.cpp"
  }
}

void AddExtI_uxth_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12730 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth, (opnd_array(4)->constant()));
   
#line 21002 "ad_aarch64.cpp"
  }
}

void AddExtI_uxth_and_shift_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12730 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth, (opnd_array(3)->constant()));
   
#line 21022 "ad_aarch64.cpp"
  }
}

void SubExtI_uxtb_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12743 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb, (opnd_array(4)->constant()));
   
#line 21042 "ad_aarch64.cpp"
  }
}

void SubExtI_uxth_and_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12756 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth, (opnd_array(4)->constant()));
   
#line 21062 "ad_aarch64.cpp"
  }
}

void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12773 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadds(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21081 "ad_aarch64.cpp"
  }
}

void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12788 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ faddd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21100 "ad_aarch64.cpp"
  }
}

void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12803 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsubs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21119 "ad_aarch64.cpp"
  }
}

void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12818 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsubd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21138 "ad_aarch64.cpp"
  }
}

void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12833 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmuls(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21157 "ad_aarch64.cpp"
  }
}

void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12848 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmuld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21176 "ad_aarch64.cpp"
  }
}

void maddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12864 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmadds(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21197 "ad_aarch64.cpp"
  }
}

void maddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12881 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaddd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21218 "ad_aarch64.cpp"
  }
}

void msubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12899 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmsubs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21239 "ad_aarch64.cpp"
  }
}

void msubF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12899 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmsubs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21260 "ad_aarch64.cpp"
  }
}

void msubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12917 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmsubd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21281 "ad_aarch64.cpp"
  }
}

void msubD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12917 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmsubd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21302 "ad_aarch64.cpp"
  }
}

void mnaddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12935 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnmadds(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21323 "ad_aarch64.cpp"
  }
}

void mnaddF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12935 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnmadds(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21344 "ad_aarch64.cpp"
  }
}

void mnaddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12953 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnmaddd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21365 "ad_aarch64.cpp"
  }
}

void mnaddD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12953 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnmaddd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21386 "ad_aarch64.cpp"
  }
}

void mnsubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12970 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnmsubs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21407 "ad_aarch64.cpp"
  }
}

void mnsubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12987 "src/hotspot/cpu/aarch64/aarch64.ad"

  // n.b. insn name should be fnmsubd
    __ fnmsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 21429 "ad_aarch64.cpp"
  }
}

void maxF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13004 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaxs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21448 "ad_aarch64.cpp"
  }
}

void minF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13018 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmins(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21467 "ad_aarch64.cpp"
  }
}

void maxD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13032 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaxd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21486 "ad_aarch64.cpp"
  }
}

void minD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13046 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmind(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21505 "ad_aarch64.cpp"
  }
}

void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13062 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fdivs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21524 "ad_aarch64.cpp"
  }
}

void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13077 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fdivd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21543 "ad_aarch64.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13092 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnegs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21560 "ad_aarch64.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13106 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fnegd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21577 "ad_aarch64.cpp"
  }
}

void absI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13124 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cnegw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::LT);
  
#line 21594 "ad_aarch64.cpp"
  }
}

void absL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13141 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmp(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cneg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::LT);
  
#line 21611 "ad_aarch64.cpp"
  }
}

void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13153 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fabss(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21628 "ad_aarch64.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13166 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fabsd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21645 "ad_aarch64.cpp"
  }
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13179 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsqrtd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21662 "ad_aarch64.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13192 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsqrts(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21679 "ad_aarch64.cpp"
  }
}

void roundD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rmode
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13204 "src/hotspot/cpu/aarch64/aarch64.ad"

    switch (opnd_array(2)->constant()) {
      case RoundDoubleModeNode::rmode_rint:
        __ frintnd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                   as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_floor:
        __ frintmd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                   as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_ceil:
        __ frintpd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                   as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
    }
  
#line 21709 "ad_aarch64.cpp"
  }
}

void andI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13237 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21728 "ad_aarch64.cpp"
  }
}

void andI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13252 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constant()));
  
#line 21747 "ad_aarch64.cpp"
  }
}

void orI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13269 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21766 "ad_aarch64.cpp"
  }
}

void orI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13284 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constant()));
  
#line 21785 "ad_aarch64.cpp"
  }
}

void xorI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13301 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21804 "ad_aarch64.cpp"
  }
}

void xorI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13316 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constant()));
  
#line 21823 "ad_aarch64.cpp"
  }
}

void andL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13334 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21842 "ad_aarch64.cpp"
  }
}

void andL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13349 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constantL()));
  
#line 21861 "ad_aarch64.cpp"
  }
}

void orL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13366 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21880 "ad_aarch64.cpp"
  }
}

void orL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13381 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (unsigned long)(opnd_array(2)->constantL()));
  
#line 21899 "ad_aarch64.cpp"
  }
}

void xorL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13398 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 21918 "ad_aarch64.cpp"
  }
}

void xorL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13413 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (unsigned long)(opnd_array(2)->constantL()));
  
#line 21937 "ad_aarch64.cpp"
  }
}

void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13428 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sbfm(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 0, 31);
  
#line 21953 "ad_aarch64.cpp"
  }
}

void convUI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13441 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ubfm(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 0, 31);
  
#line 21970 "ad_aarch64.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13454 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ movw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 21986 "ad_aarch64.cpp"
  }
}

void convI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13471 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cset(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::NE);
  
#line 22003 "ad_aarch64.cpp"
  }
}

void convP2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13489 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmp(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cset(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::NE);
  
#line 22020 "ad_aarch64.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13503 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvtd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22036 "ad_aarch64.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13516 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvts(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22052 "ad_aarch64.cpp"
  }
}

void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13529 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvtzsw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22068 "ad_aarch64.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13542 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvtzs(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22084 "ad_aarch64.cpp"
  }
}

void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13555 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ scvtfws(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22100 "ad_aarch64.cpp"
  }
}

void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13568 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ scvtfs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22116 "ad_aarch64.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13581 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvtzdw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22132 "ad_aarch64.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13594 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcvtzd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22148 "ad_aarch64.cpp"
  }
}

void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13607 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ scvtfwd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22164 "ad_aarch64.cpp"
  }
}

void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13620 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ scvtfd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22180 "ad_aarch64.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13639 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldrw(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 22196 "ad_aarch64.cpp"
  }
}

void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13657 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldrs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 22212 "ad_aarch64.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13675 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldr(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 22228 "ad_aarch64.cpp"
  }
}

void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13693 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ldrd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 22244 "ad_aarch64.cpp"
  }
}

void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13711 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ strs(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 22260 "ad_aarch64.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13729 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ strw(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 22276 "ad_aarch64.cpp"
  }
}

void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13747 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ strd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 22292 "ad_aarch64.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13765 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ str(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 22308 "ad_aarch64.cpp"
  }
}

void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13783 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovs(opnd_array(0)->as_Register(ra_,this)/* dst */, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22324 "ad_aarch64.cpp"
  }
}

void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13801 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22340 "ad_aarch64.cpp"
  }
}

void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13819 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovd(opnd_array(0)->as_Register(ra_,this)/* dst */, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22356 "ad_aarch64.cpp"
  }
}

void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13837 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22372 "ad_aarch64.cpp"
  }
}

void clearArray_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13856 "src/hotspot/cpu/aarch64/aarch64.ad"

    address tpc = __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 22393 "ad_aarch64.cpp"
  }
}

void clearArray_imm_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13877 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, (u_int64_t)opnd_array(1)->constantL());
  
#line 22410 "ad_aarch64.cpp"
  }
}

void overflowAddI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13893 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmnw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22427 "ad_aarch64.cpp"
  }
}

void overflowAddI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13906 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmnw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 22444 "ad_aarch64.cpp"
  }
}

void overflowAddL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13919 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmn(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22461 "ad_aarch64.cpp"
  }
}

void overflowAddL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13932 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmn(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 22478 "ad_aarch64.cpp"
  }
}

void overflowSubI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13945 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22495 "ad_aarch64.cpp"
  }
}

void overflowSubI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13958 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 22512 "ad_aarch64.cpp"
  }
}

void overflowSubL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13971 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmp(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 22529 "ad_aarch64.cpp"
  }
}

void overflowSubL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13984 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subs(zr, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 22546 "ad_aarch64.cpp"
  }
}

void overflowNegI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13997 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(zr, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */);
  
#line 22563 "ad_aarch64.cpp"
  }
}

void overflowNegL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14010 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmp(zr, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */);
  
#line 22580 "ad_aarch64.cpp"
  }
}

void overflowMulI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14027 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smull(rscratch1, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
    __ subs(zr, rscratch1, rscratch1, ext::sxtw);      // NE => overflow
    __ movw(rscratch1, 0x80000000);                    // Develop 0 (EQ),
    __ cselw(rscratch1, rscratch1, zr, Assembler::NE); // or 0x80000000 (NE)
    __ cmpw(rscratch1, 1);                             // 0x80000000 - 1 => VS
  
#line 22601 "ad_aarch64.cpp"
  }
}

void overflowMulI_reg_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14049 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ smull(rscratch1, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */);
    __ subs(zr, rscratch1, rscratch1, ext::sxtw);      // NE => overflow
    __ br(cond == Assembler::VS ? Assembler::NE : Assembler::EQ, *L);
  
#line 22624 "ad_aarch64.cpp"
  }
}

void overflowMulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14071 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mul(rscratch1, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);   // Result bits 0..63
    __ smulh(rscratch2, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */); // Result bits 64..127
    __ cmp(rscratch2, rscratch1, Assembler::ASR, 63);    // Top is pure sign ext
    __ movw(rscratch1, 0x80000000);                    // Develop 0 (EQ),
    __ cselw(rscratch1, rscratch1, zr, Assembler::NE); // or 0x80000000 (NE)
    __ cmpw(rscratch1, 1);                             // 0x80000000 - 1 => VS
  
#line 22646 "ad_aarch64.cpp"
  }
}

void overflowMulL_reg_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14095 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ mul(rscratch1, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */);   // Result bits 0..63
    __ smulh(rscratch2, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */); // Result bits 64..127
    __ cmp(rscratch2, rscratch1, Assembler::ASR, 63);    // Top is pure sign ext
    __ br(cond == Assembler::VS ? Assembler::NE : Assembler::EQ, *L);
  
#line 22670 "ad_aarch64.cpp"
  }
}

void compI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3279 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 22689 "ad_aarch64.cpp"
  }
}

void compI_reg_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3286 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 22712 "ad_aarch64.cpp"
  }
}

void compI_reg_immIAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3286 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 22735 "ad_aarch64.cpp"
  }
}

void compI_reg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3297 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int32_t val = (u_int32_t)opnd_array(2)->constant();
    __ movw(rscratch1, val);
    __ cmpw(reg1, rscratch1);
  
#line 22755 "ad_aarch64.cpp"
  }
}

void compU_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3279 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 22774 "ad_aarch64.cpp"
  }
}

void compU_reg_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3286 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 22797 "ad_aarch64.cpp"
  }
}

void compU_reg_immIAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3286 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 22820 "ad_aarch64.cpp"
  }
}

void compU_reg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3297 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int32_t val = (u_int32_t)opnd_array(2)->constant();
    __ movw(rscratch1, val);
    __ cmpw(reg1, rscratch1);
  
#line 22840 "ad_aarch64.cpp"
  }
}

void compL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3305 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 22859 "ad_aarch64.cpp"
  }
}

void compL_reg_immL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3312 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 22886 "ad_aarch64.cpp"
  }
}

void compL_reg_immLAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3312 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 22913 "ad_aarch64.cpp"
  }
}

void compL_reg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3327 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int64_t val = (u_int64_t)opnd_array(2)->constantL();
    __ mov(rscratch1, val);
    __ cmp(reg1, rscratch1);
  
#line 22933 "ad_aarch64.cpp"
  }
}

void compUL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3305 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 22952 "ad_aarch64.cpp"
  }
}

void compUL_reg_immL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3312 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 22979 "ad_aarch64.cpp"
  }
}

void compUL_reg_immLAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3312 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 23006 "ad_aarch64.cpp"
  }
}

void compUL_reg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3327 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int64_t val = (u_int64_t)opnd_array(2)->constantL();
    __ mov(rscratch1, val);
    __ cmp(reg1, rscratch1);
  
#line 23026 "ad_aarch64.cpp"
  }
}

void compP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3335 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 23045 "ad_aarch64.cpp"
  }
}

void compN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 3342 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 23064 "ad_aarch64.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3349 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    __ cmp(reg, zr);
  
#line 23082 "ad_aarch64.cpp"
  }
}

void testN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3355 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    __ cmpw(reg, zr);
  
#line 23100 "ad_aarch64.cpp"
  }
}

void compF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14406 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcmps(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23117 "ad_aarch64.cpp"
  }
}

void compF_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14420 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcmps(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 0.0);
  
#line 23134 "ad_aarch64.cpp"
  }
}

void compD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14435 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcmpd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23151 "ad_aarch64.cpp"
  }
}

void compD_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14449 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fcmpd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 0.0);
  
#line 23168 "ad_aarch64.cpp"
  }
}

void compF3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14467 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister s2 = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmps(s1, s2);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 23194 "ad_aarch64.cpp"
  }
}

void compD3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14495 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister s2 = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmpd(s1, s2);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 23220 "ad_aarch64.cpp"
  }
}

void compF3_reg_immF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14522 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmps(s1, 0.0);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 23245 "ad_aarch64.cpp"
  }
}

void compD3_reg_immD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14549 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmpd(s1, 0.0);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 23270 "ad_aarch64.cpp"
  }
}

void cmpLTMask_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14576 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* p */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* q */));
    __ csetw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::LT);
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), zr, as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 23289 "ad_aarch64.cpp"
  }
}

void cmpLTMask_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14594 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ asrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 31);
  
#line 23306 "ad_aarch64.cpp"
  }
}

void cmovI_reg_reg_ltNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14611 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             Assembler::LT);
  
#line 23327 "ad_aarch64.cpp"
  }
}

void cmovI_reg_reg_gtNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14642 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             Assembler::GT);
  
#line 23348 "ad_aarch64.cpp"
  }
}

void branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3361 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(1)->label();
    __ b(*L);
  
#line 23365 "ad_aarch64.cpp"
  }
}

void branchConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3367 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 23384 "ad_aarch64.cpp"
  }
}

void branchConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3373 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 23403 "ad_aarch64.cpp"
  }
}

void cmpI_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14733 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23427 "ad_aarch64.cpp"
  }
}

void cmpL_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14750 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23451 "ad_aarch64.cpp"
  }
}

void cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14767 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23475 "ad_aarch64.cpp"
  }
}

void cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14784 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23499 "ad_aarch64.cpp"
  }
}

void cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14801 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* oop */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* oop */, *L);
  
#line 23523 "ad_aarch64.cpp"
  }
}

void cmpUI_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14818 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ || cond == Assembler::LS)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23547 "ad_aarch64.cpp"
  }
}

void cmpUL_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14835 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ || cond == Assembler::LS)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 23571 "ad_aarch64.cpp"
  }
}

void cmpL_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14855 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 63, *L);
  
#line 23593 "ad_aarch64.cpp"
  }
}

void cmpI_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14871 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 31, *L);
  
#line 23615 "ad_aarch64.cpp"
  }
}

void cmpL_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14888 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2_long(opnd_array(3)->constantL());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L);
  
#line 23638 "ad_aarch64.cpp"
  }
}

void cmpI_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14905 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2((juint)opnd_array(3)->constant());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L);
  
#line 23661 "ad_aarch64.cpp"
  }
}

void far_cmpL_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14922 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 63, *L, /*far*/true);
  
#line 23683 "ad_aarch64.cpp"
  }
}

void far_cmpI_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14937 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 31, *L, /*far*/true);
  
#line 23705 "ad_aarch64.cpp"
  }
}

void far_cmpL_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14953 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2_long(opnd_array(3)->constantL());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L, /*far*/true);
  
#line 23728 "ad_aarch64.cpp"
  }
}

void far_cmpI_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14969 "src/hotspot/cpu/aarch64/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2((juint)opnd_array(3)->constant());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L, /*far*/true);
  
#line 23751 "ad_aarch64.cpp"
  }
}

void cmpL_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 14987 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ tst(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 23769 "ad_aarch64.cpp"
  }
}

void cmpI_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15000 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ tstw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 23787 "ad_aarch64.cpp"
  }
}

void cmpL_and_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15011 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ tst(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23805 "ad_aarch64.cpp"
  }
}

void cmpI_and_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15022 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ tstw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23823 "ad_aarch64.cpp"
  }
}

void branchLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3367 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 23842 "ad_aarch64.cpp"
  }
}

void branchLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 3373 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 23861 "ad_aarch64.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 3498 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;
    Label cas_failed;

    assert_different_registers(oop, box, tmp, disp_hdr);

    // Load markWord from object into displaced_header.
    __ ldr(disp_hdr, Address(oop, oopDesc::mark_offset_in_bytes()));

    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_enter(box, oop, disp_hdr, tmp, true, cont);
    }

    // Check for existing monitor
    __ tbnz(disp_hdr, exact_log2(markWord::monitor_value), object_has_monitor);

    // Set tmp to be (markWord of object | UNLOCK_VALUE).
    __ orr(tmp, disp_hdr, markWord::unlocked_value);

    // Initialize the box. (Must happen before we update the object mark!)
    __ str(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // Compare object markWord with an unlocked value (tmp) and if
    // equal exchange the stack address of our box with object markWord.
    // On failure disp_hdr contains the possibly locked markWord.
    __ cmpxchg(oop, tmp, box, Assembler::xword, /*acquire*/ true,
               /*release*/ true, /*weak*/ false, disp_hdr);
    __ br(Assembler::EQ, cont);

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // If the compare-and-exchange succeeded, then we found an unlocked
    // object, will have now locked it will continue at label cont

    __ bind(cas_failed);
    // We did not see an unlocked object so try the fast recursive case.

    // Check if the owner is self by comparing the value in the
    // markWord of object (disp_hdr) with the stack pointer.
    __ mov(rscratch1, sp);
    __ sub(disp_hdr, disp_hdr, rscratch1);
    __ mov(tmp, (address) (~(os::vm_page_size()-1) | markWord::lock_mask_in_place));
    // If condition is true we are cont and hence we can store 0 as the
    // displaced header in the box, which indicates that it is a recursive lock.
    __ ands(tmp/*==0?*/, disp_hdr, tmp);   // Sets flags for result
    __ str(tmp/*==0, perhaps*/, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    __ b(cont);

    // Handle existing monitor.
    __ bind(object_has_monitor);

    // The object's monitor m is unlocked iff m->owner == NULL,
    // otherwise m->owner may contain a thread or a stack address.
    //
    // Try to CAS m->owner from NULL to current thread.
    __ add(tmp, disp_hdr, (ObjectMonitor::owner_offset_in_bytes()-markWord::monitor_value));
    __ cmpxchg(tmp, zr, rthread, Assembler::xword, /*acquire*/ true,
               /*release*/ true, /*weak*/ false, noreg); // Sets flags for result

    // Store a non-null value into the box to avoid looking like a re-entrant
    // lock. The fast-path monitor unlock code checks for
    // markWord::monitor_value so use markWord::unused_mark which has the
    // relevant bit set, and also matches ObjectSynchronizer::enter.
    __ mov(tmp, (address)markWord::unused_mark().value());
    __ str(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    __ bind(cont);
    // flag == EQ indicates success
    // flag == NE indicates failure
  
#line 23953 "ad_aarch64.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 3576 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;

    assert_different_registers(oop, box, tmp, disp_hdr);

    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_exit(oop, tmp, cont);
    }

    // Find the lock address and load the displaced header from the stack.
    __ ldr(disp_hdr, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // If the displaced header is 0, we have a recursive unlock.
    __ cmp(disp_hdr, zr);
    __ br(Assembler::EQ, cont);

    // Handle existing monitor.
    __ ldr(tmp, Address(oop, oopDesc::mark_offset_in_bytes()));
    __ tbnz(disp_hdr, exact_log2(markWord::monitor_value), object_has_monitor);

    // Check if it is still a light weight lock, this is is true if we
    // see the stack address of the basicLock in the markWord of the
    // object.

    __ cmpxchg(oop, box, disp_hdr, Assembler::xword, /*acquire*/ false,
               /*release*/ true, /*weak*/ false, tmp);
    __ b(cont);

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // Handle existing monitor.
    __ bind(object_has_monitor);
    STATIC_ASSERT(markWord::monitor_value <= INT_MAX);
    __ add(tmp, tmp, -(int)markWord::monitor_value); // monitor
    __ ldr(rscratch1, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
    __ ldr(disp_hdr, Address(tmp, ObjectMonitor::recursions_offset_in_bytes()));
    __ eor(rscratch1, rscratch1, rthread); // Will be 0 if we are the owner.
    __ orr(rscratch1, rscratch1, disp_hdr); // Will be 0 if there are 0 recursions
    __ cmp(rscratch1, zr); // Sets flags for result
    __ br(Assembler::NE, cont);

    __ ldr(rscratch1, Address(tmp, ObjectMonitor::EntryList_offset_in_bytes()));
    __ ldr(disp_hdr, Address(tmp, ObjectMonitor::cxq_offset_in_bytes()));
    __ orr(rscratch1, rscratch1, disp_hdr); // Will be 0 if both are 0.
    __ cmp(rscratch1, zr); // Sets flags for result
    __ cbnz(rscratch1, cont);
    // need a release store here
    __ lea(tmp, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
    __ stlr(zr, tmp); // set unowned

    __ bind(cont);
    // flag == EQ indicates success
    // flag == NE indicates failure
  
#line 24028 "ad_aarch64.cpp"
  }
}

void safePointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15117 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ read_polling_page(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* poll */), relocInfo::poll_type);
  
#line 24044 "ad_aarch64.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3397 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);

    address addr = (address)opnd_array(1)->method();
    address call;
    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      call = __ trampoline_call(Address(addr, relocInfo::runtime_call_type), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      call = __ trampoline_call(Address(addr, rspec), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
      // Emit stub for static call
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    }
  
#line 24085 "ad_aarch64.cpp"
  }
  {

#line 3437 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 24097 "ad_aarch64.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3427 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    int method_index = resolved_method_index(cbuf);
    address call = __ ic_call((address)opnd_array(1)->method(), method_index);
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 24118 "ad_aarch64.cpp"
  }
  {

#line 3437 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 24130 "ad_aarch64.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3445 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blr(rscratch1);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 24168 "ad_aarch64.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3445 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blr(rscratch1);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 24206 "ad_aarch64.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 3445 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blr(rscratch1);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 24244 "ad_aarch64.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 3482 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    __ br(target_reg);
  
#line 24262 "ad_aarch64.cpp"
  }
}

void TailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 3488 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    // exception oop should be in r0
    // ret addr has been popped into lr
    // callee expects it in r3
    __ mov(r3, lr);
    __ br(target_reg);
  
#line 24284 "ad_aarch64.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 3472 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ far_jump(RuntimeAddress(OptoRuntime::rethrow_stub()));
  
#line 24313 "ad_aarch64.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 3477 "src/hotspot/cpu/aarch64/aarch64.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ ret(lr);
  
#line 24329 "ad_aarch64.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15297 "src/hotspot/cpu/aarch64/aarch64.ad"

    if (is_reachable()) {
      __ stop(_halt_reason);
    }
  
#line 24347 "ad_aarch64.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 3380 "src/hotspot/cpu/aarch64/aarch64.ad"

     Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
     Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
     Register temp_reg = as_Register(R2_enc);
     Register result_reg = as_Register(opnd_array(0)->reg(ra_,this)/* result */);

     Label miss;
     C2_MacroAssembler _masm(&cbuf);
     __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
     if ((0x1 /*primary()*/)) {
       __ mov(result_reg, zr);
     }
     __ bind(miss);
  
#line 24376 "ad_aarch64.cpp"
  }
}

void partialSubtypeCheckVsZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 3380 "src/hotspot/cpu/aarch64/aarch64.ad"

     Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
     Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
     Register temp_reg = as_Register(R2_enc);
     Register result_reg = as_Register(R5_enc);

     Label miss;
     C2_MacroAssembler _masm(&cbuf);
     __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
     if ((0x0 /*primary()*/)) {
       __ mov(result_reg, zr);
     }
     __ bind(miss);
  
#line 24406 "ad_aarch64.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15352 "src/hotspot/cpu/aarch64/aarch64.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc), as_Register(R11_enc),
                      fnoreg, fnoreg, fnoreg, StrIntrinsicNode::UU);
  
#line 24429 "ad_aarch64.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15370 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc), as_Register(R11_enc),
                      fnoreg, fnoreg, fnoreg, StrIntrinsicNode::LL);
  
#line 24451 "ad_aarch64.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15389 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc), as_Register(R11_enc),
                      as_FloatRegister(V0_enc), as_FloatRegister(V1_enc),
                      as_FloatRegister(V2_enc), StrIntrinsicNode::UL);
  
#line 24474 "ad_aarch64.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15409 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc), as_Register(R11_enc),
                      as_FloatRegister(V0_enc), as_FloatRegister(V1_enc),
                      as_FloatRegister(V2_enc),StrIntrinsicNode::LU);
  
#line 24497 "ad_aarch64.cpp"
  }
}

void string_indexofUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp5
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15429 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */,
                      opnd_array(9)->as_Register(ra_,this,idx9)/* tmp5 */, opnd_array(10)->as_Register(ra_,this,idx10)/* tmp6 */,
                      -1, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::UU);
  
#line 24527 "ad_aarch64.cpp"
  }
}

void string_indexofLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp5
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15450 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */,
                      opnd_array(9)->as_Register(ra_,this,idx9)/* tmp5 */, opnd_array(10)->as_Register(ra_,this,idx10)/* tmp6 */,
                      -1, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::LL);
  
#line 24557 "ad_aarch64.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp5
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15471 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */,
                      opnd_array(9)->as_Register(ra_,this,idx9)/* tmp5 */, opnd_array(10)->as_Register(ra_,this,idx10)/* tmp6 */,
                      -1, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::UL);
  
#line 24587 "ad_aarch64.cpp"
  }
}

void string_indexof_conUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15492 "src/hotspot/cpu/aarch64/aarch64.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, zr, zr,
                      icnt2, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::UU);
  
#line 24615 "ad_aarch64.cpp"
  }
}

void string_indexof_conLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15513 "src/hotspot/cpu/aarch64/aarch64.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, zr, zr,
                      icnt2, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::LL);
  
#line 24643 "ad_aarch64.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15534 "src/hotspot/cpu/aarch64/aarch64.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, zr, zr,
                      icnt2, opnd_array(0)->as_Register(ra_,this)/* result */, StrIntrinsicNode::UL);
  
#line 24671 "ad_aarch64.cpp"
  }
}

void string_indexofU_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15555 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                           opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */,
                           opnd_array(6)->as_Register(ra_,this,idx6)/* tmp3 */);
  
#line 24694 "ad_aarch64.cpp"
  }
}

void string_equalsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15571 "src/hotspot/cpu/aarch64/aarch64.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 1);
  
#line 24714 "ad_aarch64.cpp"
  }
}

void string_equalsUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15587 "src/hotspot/cpu/aarch64/aarch64.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 2);
  
#line 24734 "ad_aarch64.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15604 "src/hotspot/cpu/aarch64/aarch64.ad"

    address tpc = __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                                   opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */,
                                   opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R10_enc), 1);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 24760 "ad_aarch64.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15625 "src/hotspot/cpu/aarch64/aarch64.ad"

    address tpc = __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                                   opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */,
                                   opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R10_enc), 2);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 24786 "ad_aarch64.cpp"
  }
}

void has_negativesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15642 "src/hotspot/cpu/aarch64/aarch64.ad"

    address tpc = __ has_negatives(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */, opnd_array(0)->as_Register(ra_,this)/* result */);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 24807 "ad_aarch64.cpp"
  }
}

void string_compressNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15662 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ char_array_compress(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                           opnd_array(4)->as_FloatRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_FloatRegister(ra_,this,idx5)/* tmp2 */,
                           opnd_array(6)->as_FloatRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_FloatRegister(ra_,this,idx7)/* tmp4 */,
                           opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 24832 "ad_aarch64.cpp"
  }
}

void string_inflateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15679 "src/hotspot/cpu/aarch64/aarch64.ad"

    address tpc = __ byte_array_inflate(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                                        opnd_array(4)->as_FloatRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_FloatRegister(ra_,this,idx5)/* tmp2 */,
                                        opnd_array(6)->as_FloatRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp4 */);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 24860 "ad_aarch64.cpp"
  }
}

void encode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15702 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
         opnd_array(0)->as_Register(ra_,this)/* result */, as_FloatRegister(V0_enc),  as_FloatRegister(V1_enc),
         as_FloatRegister(V2_enc),  as_FloatRegister(V3_enc));
  
#line 24880 "ad_aarch64.cpp"
  }
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void loadV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2796 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::S,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24910 "ad_aarch64.cpp"
  }
}

void loadV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2802 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::D,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24927 "ad_aarch64.cpp"
  }
}

void loadV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2808 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::Q,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24944 "ad_aarch64.cpp"
  }
}

void storeV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2814 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::S,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24962 "ad_aarch64.cpp"
  }
}

void storeV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2820 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::D,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24980 "ad_aarch64.cpp"
  }
}

void storeV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2826 "src/hotspot/cpu/aarch64/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::Q,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 24998 "ad_aarch64.cpp"
  }
}

void replicate8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15804 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25014 "ad_aarch64.cpp"
  }
}

void replicate16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15816 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25030 "ad_aarch64.cpp"
  }
}

void replicate8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15829 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, opnd_array(1)->constant()& 0xff);
  
#line 25046 "ad_aarch64.cpp"
  }
}

void replicate16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15841 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, opnd_array(1)->constant()& 0xff);
  
#line 25062 "ad_aarch64.cpp"
  }
}

void replicate4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15854 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25078 "ad_aarch64.cpp"
  }
}

void replicate8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15866 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25094 "ad_aarch64.cpp"
  }
}

void replicate4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15879 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H, opnd_array(1)->constant()& 0xffff);
  
#line 25110 "ad_aarch64.cpp"
  }
}

void replicate8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15891 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H, opnd_array(1)->constant()& 0xffff);
  
#line 25126 "ad_aarch64.cpp"
  }
}

void replicate2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15903 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25142 "ad_aarch64.cpp"
  }
}

void replicate4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15915 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25158 "ad_aarch64.cpp"
  }
}

void replicate2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15927 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, opnd_array(1)->constant());
  
#line 25174 "ad_aarch64.cpp"
  }
}

void replicate4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15939 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, opnd_array(1)->constant());
  
#line 25190 "ad_aarch64.cpp"
  }
}

void replicate2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15951 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25206 "ad_aarch64.cpp"
  }
}

void replicate2L_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15963 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
           as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
           as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 25224 "ad_aarch64.cpp"
  }
}

void replicate2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15977 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25241 "ad_aarch64.cpp"
  }
}

void replicate4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 15990 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25258 "ad_aarch64.cpp"
  }
}

void replicate2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16003 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 25275 "ad_aarch64.cpp"
  }
}

void reduce_add2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// isrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16022 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ umov(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), __ S, 0);
    __ umov(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), __ S, 1);
    __ addw(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* isrc */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ addw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
  
#line 25297 "ad_aarch64.cpp"
  }
}

void reduce_add4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// isrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// itmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16040 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* vtmp */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ umov(opnd_array(4)->as_Register(ra_,this,idx4)/* itmp */, as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* vtmp */), __ S, 0);
    __ addw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* itmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* isrc */);
  
#line 25319 "ad_aarch64.cpp"
  }
}

void reduce_mul2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// isrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16059 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ umov(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), __ S, 0);
    __ mul(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* isrc */);
    __ umov(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), __ S, 1);
    __ mul(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */);
  
#line 25341 "ad_aarch64.cpp"
  }
}

void reduce_mul4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// isrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// itmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16080 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* vtmp */), __ D,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ mulv(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* vtmp */), __ T2S,
            as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* vtmp */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ umov(opnd_array(5)->as_Register(ra_,this,idx5)/* itmp */, as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* vtmp */), __ S, 0);
    __ mul(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(5)->as_Register(ra_,this,idx5)/* itmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* isrc */);
    __ umov(opnd_array(5)->as_Register(ra_,this,idx5)/* itmp */, as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* vtmp */), __ S, 1);
    __ mul(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(5)->as_Register(ra_,this,idx5)/* itmp */, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */);
  
#line 25368 "ad_aarch64.cpp"
  }
}

void reduce_add2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16102 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25392 "ad_aarch64.cpp"
  }
}

void reduce_add4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 2);
    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 3);
    __ fadds(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25424 "ad_aarch64.cpp"
  }
}

void reduce_mul2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16154 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25448 "ad_aarch64.cpp"
  }
}

void reduce_mul4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16178 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 2);
    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 3);
    __ fmuls(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25480 "ad_aarch64.cpp"
  }
}

void reduce_add2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16206 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ faddd(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ D,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ faddd(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25504 "ad_aarch64.cpp"
  }
}

void reduce_mul2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16226 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmuld(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ D,
           as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmuld(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25528 "ad_aarch64.cpp"
  }
}

void reduce_max2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16245 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaxs(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmaxs(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25549 "ad_aarch64.cpp"
  }
}

void reduce_max4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16260 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaxv(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T4S, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ fmaxs(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */));
  
#line 25568 "ad_aarch64.cpp"
  }
}

void reduce_max2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16275 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmaxd(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ D, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmaxd(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25589 "ad_aarch64.cpp"
  }
}

void reduce_min2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16291 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmins(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ S, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmins(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25610 "ad_aarch64.cpp"
  }
}

void reduce_min4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// fsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16306 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fminv(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T4S, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ fmins(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* fsrc */));
  
#line 25629 "ad_aarch64.cpp"
  }
}

void reduce_min2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dsrc
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// vsrc
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16321 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmind(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dsrc */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */));
    __ ins(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ D, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* vsrc */), 0, 1);
    __ fmind(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */));
  
#line 25650 "ad_aarch64.cpp"
  }
}

void vadd8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16340 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25669 "ad_aarch64.cpp"
  }
}

void vadd16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16354 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25688 "ad_aarch64.cpp"
  }
}

void vadd4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16369 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25707 "ad_aarch64.cpp"
  }
}

void vadd8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16383 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25726 "ad_aarch64.cpp"
  }
}

void vadd2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16397 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25745 "ad_aarch64.cpp"
  }
}

void vadd4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16411 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25764 "ad_aarch64.cpp"
  }
}

void vadd2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16425 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25783 "ad_aarch64.cpp"
  }
}

void vadd2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16439 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25802 "ad_aarch64.cpp"
  }
}

void vadd4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16453 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25821 "ad_aarch64.cpp"
  }
}

void vadd2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16466 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25840 "ad_aarch64.cpp"
  }
}

void vsub8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16483 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25859 "ad_aarch64.cpp"
  }
}

void vsub16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16497 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25878 "ad_aarch64.cpp"
  }
}

void vsub4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16512 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25897 "ad_aarch64.cpp"
  }
}

void vsub8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16526 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25916 "ad_aarch64.cpp"
  }
}

void vsub2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16540 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25935 "ad_aarch64.cpp"
  }
}

void vsub4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16554 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25954 "ad_aarch64.cpp"
  }
}

void vsub2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16568 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25973 "ad_aarch64.cpp"
  }
}

void vsub2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16582 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 25992 "ad_aarch64.cpp"
  }
}

void vsub4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16596 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26011 "ad_aarch64.cpp"
  }
}

void vsub2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16610 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26030 "ad_aarch64.cpp"
  }
}

void vmul8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16627 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26049 "ad_aarch64.cpp"
  }
}

void vmul16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16641 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26068 "ad_aarch64.cpp"
  }
}

void vmul4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16656 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26087 "ad_aarch64.cpp"
  }
}

void vmul8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16670 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26106 "ad_aarch64.cpp"
  }
}

void vmul2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16684 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26125 "ad_aarch64.cpp"
  }
}

void vmul4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16698 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26144 "ad_aarch64.cpp"
  }
}

void vmul2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16712 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26163 "ad_aarch64.cpp"
  }
}

void vmul4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16726 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26182 "ad_aarch64.cpp"
  }
}

void vmul2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16740 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26201 "ad_aarch64.cpp"
  }
}

void vmla4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16757 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26221 "ad_aarch64.cpp"
  }
}

void vmla4S_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16757 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26241 "ad_aarch64.cpp"
  }
}

void vmla8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16771 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26261 "ad_aarch64.cpp"
  }
}

void vmla8S_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16771 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26281 "ad_aarch64.cpp"
  }
}

void vmla2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16785 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26301 "ad_aarch64.cpp"
  }
}

void vmla2I_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16785 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26321 "ad_aarch64.cpp"
  }
}

void vmla4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16799 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26341 "ad_aarch64.cpp"
  }
}

void vmla4I_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16799 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26361 "ad_aarch64.cpp"
  }
}

void vmla2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16813 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmla(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26381 "ad_aarch64.cpp"
  }
}

void vmla4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16827 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmla(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26401 "ad_aarch64.cpp"
  }
}

void vmla2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16841 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmla(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26421 "ad_aarch64.cpp"
  }
}

void vmls4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16858 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26441 "ad_aarch64.cpp"
  }
}

void vmls8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16872 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26461 "ad_aarch64.cpp"
  }
}

void vmls2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16886 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26481 "ad_aarch64.cpp"
  }
}

void vmls4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16900 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26501 "ad_aarch64.cpp"
  }
}

void vmls2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16915 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26521 "ad_aarch64.cpp"
  }
}

void vmls2F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16915 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26541 "ad_aarch64.cpp"
  }
}

void vmls4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16930 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26561 "ad_aarch64.cpp"
  }
}

void vmls4F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16930 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26581 "ad_aarch64.cpp"
  }
}

void vmls2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16945 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26601 "ad_aarch64.cpp"
  }
}

void vmls2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16945 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmls(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 26621 "ad_aarch64.cpp"
  }
}

void vmuladdS2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16963 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ smullv(as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */), __ T4H,
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ smullv(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T8H,
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ addpv(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), __ T4S,
             as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp */),
             as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */));
  
#line 26648 "ad_aarch64.cpp"
  }
}

void vdiv2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16985 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26667 "ad_aarch64.cpp"
  }
}

void vdiv4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 16999 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26686 "ad_aarch64.cpp"
  }
}

void vdiv2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17013 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26705 "ad_aarch64.cpp"
  }
}

void vsqrt2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17028 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsqrt(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26721 "ad_aarch64.cpp"
  }
}

void vsqrt4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17039 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsqrt(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26737 "ad_aarch64.cpp"
  }
}

void vsqrt2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17050 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fsqrt(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26754 "ad_aarch64.cpp"
  }
}

void vabs8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17066 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26770 "ad_aarch64.cpp"
  }
}

void vabs16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17078 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26786 "ad_aarch64.cpp"
  }
}

void vabs4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17090 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26802 "ad_aarch64.cpp"
  }
}

void vabs8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17102 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26818 "ad_aarch64.cpp"
  }
}

void vabs2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17114 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26834 "ad_aarch64.cpp"
  }
}

void vabs4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17126 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26850 "ad_aarch64.cpp"
  }
}

void vabs2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17138 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ absr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26866 "ad_aarch64.cpp"
  }
}

void vabs2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17150 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fabs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26883 "ad_aarch64.cpp"
  }
}

void vabs4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17163 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fabs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26900 "ad_aarch64.cpp"
  }
}

void vabs2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17176 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fabs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26917 "ad_aarch64.cpp"
  }
}

void vneg2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17191 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fneg(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26934 "ad_aarch64.cpp"
  }
}

void vneg4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17204 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fneg(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26951 "ad_aarch64.cpp"
  }
}

void vneg2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17217 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fneg(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 26968 "ad_aarch64.cpp"
  }
}

void vand8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17233 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 26987 "ad_aarch64.cpp"
  }
}

void vand16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17247 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ andr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 27006 "ad_aarch64.cpp"
  }
}

void vor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17264 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 27025 "ad_aarch64.cpp"
  }
}

void vor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17278 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ orr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 27044 "ad_aarch64.cpp"
  }
}

void vxor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17295 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 27063 "ad_aarch64.cpp"
  }
}

void vxor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17309 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 27082 "ad_aarch64.cpp"
  }
}

void vshiftcnt8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17323 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 27098 "ad_aarch64.cpp"
  }
}

void vshiftcnt8B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17323 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 27114 "ad_aarch64.cpp"
  }
}

void vshiftcnt16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17334 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 27130 "ad_aarch64.cpp"
  }
}

void vshiftcnt16B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17334 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 27146 "ad_aarch64.cpp"
  }
}

void vsll8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17346 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27165 "ad_aarch64.cpp"
  }
}

void vsll16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17359 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27184 "ad_aarch64.cpp"
  }
}

void vsra8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17396 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27206 "ad_aarch64.cpp"
  }
}

void vsra16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17413 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27228 "ad_aarch64.cpp"
  }
}

void vsrl8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17431 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27250 "ad_aarch64.cpp"
  }
}

void vsrl16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17448 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27272 "ad_aarch64.cpp"
  }
}

void vsll8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17464 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27297 "ad_aarch64.cpp"
  }
}

void vsll16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17483 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27322 "ad_aarch64.cpp"
  }
}

void vsra8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17503 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) sh = 7;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 27342 "ad_aarch64.cpp"
  }
}

void vsra16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17517 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) sh = 7;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 27362 "ad_aarch64.cpp"
  }
}

void vsrl8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17532 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27387 "ad_aarch64.cpp"
  }
}

void vsrl16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17551 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27412 "ad_aarch64.cpp"
  }
}

void vsll4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17571 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27431 "ad_aarch64.cpp"
  }
}

void vsll8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17584 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27450 "ad_aarch64.cpp"
  }
}

void vsra4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17600 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27472 "ad_aarch64.cpp"
  }
}

void vsra8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17617 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27494 "ad_aarch64.cpp"
  }
}

void vsrl4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17635 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27516 "ad_aarch64.cpp"
  }
}

void vsrl8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17652 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27538 "ad_aarch64.cpp"
  }
}

void vsll4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17668 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27563 "ad_aarch64.cpp"
  }
}

void vsll8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17687 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27588 "ad_aarch64.cpp"
  }
}

void vsra4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17707 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) sh = 15;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 27608 "ad_aarch64.cpp"
  }
}

void vsra8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17721 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) sh = 15;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 27628 "ad_aarch64.cpp"
  }
}

void vsrl4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17736 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27653 "ad_aarch64.cpp"
  }
}

void vsrl8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17755 "src/hotspot/cpu/aarch64/aarch64.ad"

    int sh = (int)opnd_array(2)->constant();
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 27678 "ad_aarch64.cpp"
  }
}

void vsll2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17774 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27697 "ad_aarch64.cpp"
  }
}

void vsll4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17787 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27716 "ad_aarch64.cpp"
  }
}

void vsra2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17802 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27738 "ad_aarch64.cpp"
  }
}

void vsra4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17819 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27760 "ad_aarch64.cpp"
  }
}

void vsrl2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17836 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T8B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27782 "ad_aarch64.cpp"
  }
}

void vsrl4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17853 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27804 "ad_aarch64.cpp"
  }
}

void vsll2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17868 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant());
  
#line 27823 "ad_aarch64.cpp"
  }
}

void vsll4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17881 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant());
  
#line 27842 "ad_aarch64.cpp"
  }
}

void vsra2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17894 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 27861 "ad_aarch64.cpp"
  }
}

void vsra4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17907 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 27880 "ad_aarch64.cpp"
  }
}

void vsrl2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17920 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 27899 "ad_aarch64.cpp"
  }
}

void vsrl4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17933 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 27918 "ad_aarch64.cpp"
  }
}

void vsll2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17946 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 27937 "ad_aarch64.cpp"
  }
}

void vsra2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17961 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27959 "ad_aarch64.cpp"
  }
}

void vsrl2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17978 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ negr(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), __ T16B,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 27981 "ad_aarch64.cpp"
  }
}

void vsll2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 17993 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant());
  
#line 28000 "ad_aarch64.cpp"
  }
}

void vsra2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18006 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 28019 "ad_aarch64.cpp"
  }
}

void vsrl2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18019 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            (int)opnd_array(2)->constant());
  
#line 28038 "ad_aarch64.cpp"
  }
}

void vmax2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18033 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmax(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28057 "ad_aarch64.cpp"
  }
}

void vmax4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18047 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmax(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28076 "ad_aarch64.cpp"
  }
}

void vmax2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18061 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmax(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28095 "ad_aarch64.cpp"
  }
}

void vmin2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18075 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmin(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28114 "ad_aarch64.cpp"
  }
}

void vmin4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18089 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmin(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28133 "ad_aarch64.cpp"
  }
}

void vmin2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18103 "src/hotspot/cpu/aarch64/aarch64.ad"

    __ fmin(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 28152 "ad_aarch64.cpp"
  }
}

void vround2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rmode
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18115 "src/hotspot/cpu/aarch64/aarch64.ad"

    switch (opnd_array(2)->constant()) {
      case RoundDoubleModeNode::rmode_rint:
        __ frintn(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
                  as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_floor:
        __ frintm(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
                  as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_ceil:
        __ frintp(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
                  as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
    }
  
#line 28182 "ad_aarch64.cpp"
  }
}

void vpopcount4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18142 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ cnt(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
     __ uaddlp(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
               as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
     __ uaddlp(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
               as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 28203 "ad_aarch64.cpp"
  }
}

void vpopcount2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 18161 "src/hotspot/cpu/aarch64/aarch64.ad"

     __ cnt(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
     __ uaddlp(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
               as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
     __ uaddlp(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
               as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 28224 "ad_aarch64.cpp"
  }
}

void compareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {

#line 30 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ false, /*release*/ true, /*weak*/ false, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28247 "ad_aarch64.cpp"
  }
}

void compareAndSwapN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 76 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, /*acquire*/ false, /*release*/ true, /*weak*/ false, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28268 "ad_aarch64.cpp"
  }
}

void compareAndSwapPAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {

#line 39 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ true, /*release*/ true, /*weak*/ false, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28291 "ad_aarch64.cpp"
  }
}

void compareAndSwapNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 114 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, /*acquire*/ true, /*release*/ true, /*weak*/ false, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28312 "ad_aarch64.cpp"
  }
}

void compareAndExchangeN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 130 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ false, /*release*/ true, /*weak*/ false, /*is_cae*/ true, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 28335 "ad_aarch64.cpp"
  }
}

void compareAndExchangeP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 146 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ false, /*release*/ true, /*weak*/ false, /*is_cae*/ true, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 28358 "ad_aarch64.cpp"
  }
}

void compareAndExchangeNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 163 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ true, /*release*/ true, /*weak*/ false, /*is_cae*/ true, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 28381 "ad_aarch64.cpp"
  }
}

void compareAndExchangePAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 180 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ true, /*release*/ true, /*weak*/ false, /*is_cae*/ true, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 28404 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 197 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ false, /*release*/ true, /*weak*/ true, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28426 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 213 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ false, /*release*/ true, /*weak*/ true, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28448 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 231 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ true, /*release*/ true, /*weak*/ true, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28470 "ad_aarch64.cpp"
  }
}

void weakCompareAndSwapPAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 249 "src/hotspot/cpu/aarch64/gc/shenandoah/shenandoah_aarch64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mov(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   /*acquire*/ true, /*release*/ true, /*weak*/ true, /*is_cae*/ false, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 28492 "ad_aarch64.cpp"
  }
}

void zLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 60 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    const Address ref_addr = mem2address(opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ ldr(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, ref_addr);
    if (barrier_data() != ZLoadBarrierElided) {
      z_load_barrier(_masm, this, ref_addr, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, rscratch2 /* tmp */, false /* weak */);
    }
  
#line 28513 "ad_aarch64.cpp"
  }
}

void zLoadWeakPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 82 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    const Address ref_addr = mem2address(opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ ldr(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, ref_addr);
    z_load_barrier(_masm, this, ref_addr, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, rscratch2 /* tmp */, true /* weak */);
  
#line 28532 "ad_aarch64.cpp"
  }
}

void zLoadPVolatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 102 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    __ ldar(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */);
    if (barrier_data() != ZLoadBarrierElided) {
      z_load_barrier(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, rscratch2 /* tmp */, false /* weak */);
    }
  
#line 28552 "ad_aarch64.cpp"
  }
}

void zCompareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 123 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               false /* acquire */, true /* release */, false /* weak */, rscratch2);
    __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, rscratch2);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), rscratch2 /* ref */, rscratch1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 false /* acquire */, true /* release */, false /* weak */, rscratch2);
      __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
      __ bind(good);
    }
  
#line 28585 "ad_aarch64.cpp"
  }
}

void zCompareAndSwapP_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 123 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               false /* acquire */, true /* release */, false /* weak */, rscratch2);
    __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, rscratch2);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), rscratch2 /* ref */, rscratch1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 false /* acquire */, true /* release */, false /* weak */, rscratch2);
      __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
      __ bind(good);
    }
  
#line 28618 "ad_aarch64.cpp"
  }
}

void zCompareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 155 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               true /* acquire */, true /* release */, false /* weak */, rscratch2);
    __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, rscratch2);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), rscratch2 /* ref */, rscratch1 /* tmp */ );
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 true /* acquire */, true /* release */, false /* weak */, rscratch2);
      __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
      __ bind(good);
    }
  
#line 28651 "ad_aarch64.cpp"
  }
}

void zCompareAndSwapPAcq_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 155 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               true /* acquire */, true /* release */, false /* weak */, rscratch2);
    __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, rscratch2);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), rscratch2 /* ref */, rscratch1 /* tmp */ );
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 true /* acquire */, true /* release */, false /* weak */, rscratch2);
      __ cset(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, Assembler::EQ);
      __ bind(good);
    }
  
#line 28684 "ad_aarch64.cpp"
  }
}

void zCompareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 185 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               false /* acquire */, true /* release */, false /* weak */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), opnd_array(4)->as_Register(ra_,this,idx4)/* res */, rscratch1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 false /* acquire */, true /* release */, false /* weak */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ bind(good);
    }
  
#line 28715 "ad_aarch64.cpp"
  }
}

void zCompareAndExchangePAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 213 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
               true /* acquire */, true /* release */, false /* weak */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ldr(rscratch1, Address(rthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(rscratch1, rscratch1, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ cbz(rscratch1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), opnd_array(4)->as_Register(ra_,this,idx4)/* res */, rscratch1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::xword,
                 true /* acquire */, true /* release */, false /* weak */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ bind(good);
    }
  
#line 28746 "ad_aarch64.cpp"
  }
}

void zGetAndSetPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// prev
  {
    C2_MacroAssembler _masm(&cbuf);

#line 241 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    __ atomic_xchg(opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */);
    if (barrier_data() != ZLoadBarrierElided) {
      z_load_barrier(_masm, this, Address(noreg, 0), opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, rscratch2 /* tmp */, false /* weak */);
    }
  
#line 28767 "ad_aarch64.cpp"
  }
}

void zGetAndSetPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// prev
  {
    C2_MacroAssembler _masm(&cbuf);

#line 260 "src/hotspot/cpu/aarch64/gc/z/z_aarch64.ad"

    __ atomic_xchgal(opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */);
    if (barrier_data() != ZLoadBarrierElided) {
      z_load_barrier(_masm, this, Address(noreg, 0), opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, rscratch2 /* tmp */, false /* weak */);
    }
  
#line 28788 "ad_aarch64.cpp"
  }
}

const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0_orderedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchallocNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadP_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadN_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadF_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeC_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeP_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeN_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* cacheWBNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV16Node::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* zLoadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zLoadWeakPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zLoadPVolatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapP_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPAcq_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndExchangePNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndExchangePAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* zGetAndSetPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zGetAndSetPAcqNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    true ,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    true ,  // CacheWB
    true ,  // CacheWBPreSync
    true ,  // CacheWBPostSync
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    false,  // CMoveVD
    true ,  // CMoveF
    false,  // CMoveVF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    true ,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    true ,  // CompareAndExchangeN
    false,  // GetAndAddB
    false,  // GetAndAddS
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    false,  // GetAndSetB
    false,  // GetAndSetS
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    true ,  // EncodeISOArray
    true ,  // EncodeP
    true ,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    true ,  // HasNegatives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    false,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MulAddS2I
    true ,  // MaxD
    true ,  // MaxF
    true ,  // MaxI
    true ,  // MemBarAcquire
    true ,  // LoadFence
    false,  // SetVectMaskI
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinD
    true ,  // MinF
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // Opaque2
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    true ,  // OverflowAddI
    true ,  // OverflowSubI
    true ,  // OverflowMulI
    true ,  // OverflowAddL
    true ,  // OverflowSubL
    true ,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // SubTypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    true ,  // PopCountVI
    true ,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    false,  // RoundDouble
    true ,  // RoundDoubleMode
    true ,  // RoundDoubleModeV
    false,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    true ,  // ShenandoahCompareAndExchangeP
    true ,  // ShenandoahCompareAndExchangeN
    true ,  // ShenandoahCompareAndSwapN
    true ,  // ShenandoahCompareAndSwapP
    true ,  // ShenandoahWeakCompareAndSwapN
    true ,  // ShenandoahWeakCompareAndSwapP
    false,  // ShenandoahEnqueueBarrier
    false,  // ShenandoahLoadReferenceBarrier
    false,  // SCMemProj
    true ,  // SqrtD
    true ,  // SqrtF
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    true ,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    false,  // MacroLogicV
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddReductionVI
    true ,  // AddVL
    false,  // AddReductionVL
    true ,  // AddVF
    true ,  // AddReductionVF
    true ,  // AddVD
    true ,  // AddReductionVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVB
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulReductionVI
    false,  // MulVL
    false,  // MulReductionVL
    true ,  // MulVF
    true ,  // MulReductionVF
    true ,  // MulVD
    true ,  // MulReductionVD
    true ,  // MulAddVS2VI
    true ,  // FmaVD
    true ,  // FmaVF
    true ,  // DivVF
    true ,  // DivVD
    true ,  // AbsVB
    true ,  // AbsVS
    true ,  // AbsVI
    true ,  // AbsVL
    true ,  // AbsVF
    true ,  // AbsVD
    true ,  // NegVF
    true ,  // NegVD
    true ,  // SqrtVD
    true ,  // SqrtVF
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    false,  // AndReductionV
    true ,  // OrV
    false,  // OrReductionV
    true ,  // XorV
    false,  // XorReductionV
    true ,  // MinV
    true ,  // MaxV
    true ,  // MinReductionV
    true ,  // MaxReductionV
    true ,  // LoadVector
    true ,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false   // Whitespace
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::in_preserve_stack_slots() { return 4; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes/BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 3757 "src/hotspot/cpu/aarch64/aarch64.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 29382 "ad_aarch64.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 3763 "src/hotspot/cpu/aarch64/aarch64.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, NULL, length);
  
#line 29393 "ad_aarch64.cpp"

}

OptoRegPair Matcher::return_value(uint ideal_reg, bool is_outgoing) {

#line 3770 "src/hotspot/cpu/aarch64/aarch64.ad"

    // TODO do we allow ideal_reg == Op_RegN???
    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R0_num,                            // Op_RegN
      R0_num,                            // Op_RegI
      R0_num,                            // Op_RegP
      V0_num,                            // Op_RegF
      V0_num,                            // Op_RegD
      R0_num                             // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R0_H_num,                          // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      V0_H_num,                          // Op_RegD
      R0_H_num                           // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 29429 "ad_aarch64.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg, bool is_outgoing) {

#line 3770 "src/hotspot/cpu/aarch64/aarch64.ad"

    // TODO do we allow ideal_reg == Op_RegN???
    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R0_num,                            // Op_RegN
      R0_num,                            // Op_RegI
      R0_num,                            // Op_RegP
      V0_num,                            // Op_RegF
      V0_num,                            // Op_RegD
      R0_num                             // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R0_H_num,                          // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      V0_H_num,                          // Op_RegD
      R0_H_num                           // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 29465 "ad_aarch64.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R12_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(R12_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(R29_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R31_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DAARCH64=1
#ifndef AARCH64
#  error "AARCH64 must be defined"
#endif // AARCH64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
